@@ -62,7 +62,6 @@ if TYPE_CHECKING:
     from daft.io import DataSink
     from daft.logical.schema import Schema
 
-
 # A PhysicalPlan that is still being built - may yield both PartitionTaskBuilders and PartitionTasks.
 InProgressPhysicalPlan = Iterator[Union[None, PartitionTask[PartitionT], PartitionTaskBuilder[PartitionT]]]
 
@@ -282,9 +281,9 @@ def actor_pool_project(
 ) -> InProgressPhysicalPlan[PartitionT]:
     stage_id = next(stage_id_counter)
 
-    from daft.daft import get_udf_names
+    from daft.daft import try_get_udf_name
 
-    udf_names = "-".join(name for expr in projection for name in get_udf_names(expr._expr))
+    udf_names = "-".join(udf_name for expr in projection if (udf_name := try_get_udf_name(expr._expr)) is not None)
     actor_pool_name = f"{udf_names}-stage={stage_id}"
 
     # Keep track of materializations of the children tasks
@@ -1326,11 +1325,14 @@ def concat(
 def local_limit(
     child_plan: InProgressPhysicalPlan[PartitionT],
     limit: int,
+    offset: int,
 ) -> Generator[None | PartitionTask[PartitionT] | PartitionTaskBuilder[PartitionT], int, None]:
     """Apply a limit instruction to each partition in the child_plan.
 
     limit:
         The value of the limit to apply to each partition.
+    offset:
+        The value of the offset to apply to each partition.
 
     Yields: PartitionTask with the limit applied.
     Send back: A new value to the limit (optional). This allows you to update the limit after each partition if desired.
@@ -1340,7 +1342,7 @@ def local_limit(
             yield step
         else:
             maybe_new_limit = yield step.add_instruction(
-                execution_step.LocalLimit(limit),
+                execution_step.LocalLimit(limit=limit, offset=offset),
             )
             if maybe_new_limit is not None:
                 limit = maybe_new_limit
@@ -1349,12 +1351,16 @@ def local_limit(
 def global_limit(
     child_plan: InProgressPhysicalPlan[PartitionT],
     limit_rows: int,
+    offset_rows: int,
     eager: bool,
     num_partitions: int,
 ) -> InProgressPhysicalPlan[PartitionT]:
-    """Return the first n rows from the `child_plan`."""
-    remaining_rows = limit_rows
-    assert remaining_rows >= 0, f"Invalid value for limit: {remaining_rows}"
+    """Return n rows after offset from the `child_plan`."""
+    assert offset_rows >= 0, f"Invalid value for offset: {offset_rows}"
+    assert limit_rows >= 0, f"Invalid value for limit: {limit_rows}"
+
+    remaining_skip = offset_rows
+    remaining_take = limit_rows
     remaining_partitions = num_partitions
 
     materializations: deque[SingleOutputPartitionTask[PartitionT]] = deque()
@@ -1365,7 +1371,7 @@ def global_limit(
 
     # As an optimization, push down a limit into each partition to reduce what gets materialized,
     # since we will never take more than the remaining limit anyway.
-    child_plan = local_limit(child_plan=child_plan, limit=remaining_rows)
+    child_plan = local_limit(child_plan=child_plan, limit=(remaining_skip + remaining_take), offset=0)
     started = False
     while True:
         # Check if any inputs finished executing.
@@ -1373,21 +1379,24 @@ def global_limit(
         while len(materializations) > 0 and materializations[0].done():
             done_task = materializations.popleft()
             done_task_metadata = done_task.partition_metadata()
-            limit = remaining_rows and min(remaining_rows, done_task_metadata.num_rows)
+            task_num_rows = done_task_metadata.num_rows
 
+            offset = min(remaining_skip, task_num_rows)
+            limit = remaining_take and min(remaining_take, task_num_rows - offset)
             global_limit_step = PartitionTaskBuilder[PartitionT](
                 inputs=[done_task.partition()],
                 partial_metadatas=[done_task_metadata],
                 resource_request=ResourceRequest(memory_bytes=done_task_metadata.size_bytes),
             ).add_instruction(
-                instruction=execution_step.GlobalLimit(limit),
+                instruction=execution_step.GlobalLimit(limit=limit, offset=offset),
             )
 
             yield global_limit_step
             remaining_partitions -= 1
-            remaining_rows -= limit
+            remaining_take -= limit
+            remaining_skip -= offset
 
-            if remaining_rows == 0:
+            if remaining_take == 0:
                 # We only need to return empty partitions now.
                 # Instead of computing new ones and applying limit(0),
                 # we can just reuse an existing computed partition.
@@ -1402,14 +1411,14 @@ def global_limit(
                         partial_metadatas=[done_task.partition_metadata()],
                         resource_request=ResourceRequest(memory_bytes=done_task.partition_metadata().size_bytes),
                     ).add_instruction(
-                        instruction=execution_step.GlobalLimit(0),
+                        instruction=execution_step.GlobalLimit(limit=0, offset=0),
                     )
                     for _ in range(remaining_partitions)
                 )
                 return
 
         # (Optimization. If we are doing limit(0) and already have a partition executing to use for it, just wait.)
-        if remaining_rows == 0 and len(materializations) > 0:
+        if remaining_take == 0 and len(materializations) > 0:
             logger.debug("global_limit blocked on completion of: %s", materializations[0])
             yield None
             continue
@@ -1422,19 +1431,27 @@ def global_limit(
 
         # Execute a single child partition.
         try:
-            child_step = child_plan.send(remaining_rows) if started else next(child_plan)
+            child_step = child_plan.send(remaining_skip + remaining_take) if started else next(child_plan)
             started = True
             if isinstance(child_step, PartitionTaskBuilder):
                 # If this is the very next partition to apply a nonvacuous global limit on,
                 # see if it has any row metadata already.
                 # If so, we can deterministically apply and deduct the rolling limit without materializing.
                 [partial_meta] = child_step.partial_metadatas
-                if len(materializations) == 0 and remaining_rows > 0 and partial_meta.num_rows is not None:
-                    limit = min(remaining_rows, partial_meta.num_rows)
-                    child_step = child_step.add_instruction(instruction=execution_step.LocalLimit(limit))
-
+                if (
+                    len(materializations) == 0
+                    and (remaining_skip > 0 or remaining_take > 0)
+                    and partial_meta.num_rows is not None
+                ):
+                    partial_num_rows = partial_meta.num_rows
+                    offset = min(remaining_skip, partial_num_rows)
+                    limit = min(remaining_take, partial_num_rows - offset)
+                    child_step = child_step.add_instruction(
+                        instruction=execution_step.LocalLimit(limit=limit, offset=offset)
+                    )
                     remaining_partitions -= 1
-                    remaining_rows -= limit
+                    remaining_take -= limit
+                    remaining_skip -= offset
                 else:
                     child_step = child_step.finalize_partition_task_single_output(stage_id=stage_id)
                     materializations.append(child_step)
@@ -1740,6 +1757,7 @@ def top_n(
     descending: list[bool],
     nulls_first: list[bool],
     limit: int,
+    offset: int,
     num_partitions: int,
 ) -> InProgressPhysicalPlan[PartitionT]:
     """Take the top N values from the result of `child_plan` according to `sort_info` and `limit`."""
@@ -1755,7 +1773,9 @@ def top_n(
         nulls_first=nulls_first,
         num_partitions=num_partitions,
     )
-    yield from global_limit(child_plan=child_plan, limit_rows=limit, eager=False, num_partitions=num_partitions)
+    yield from global_limit(
+        child_plan=child_plan, offset_rows=offset, limit_rows=limit, eager=False, num_partitions=num_partitions
+    )
 
 
 def fanout_random(
