@@ -1,10 +1,13 @@
 from __future__ import annotations
 
-import pickle
+import logging
+import os
 import sys
 from multiprocessing.connection import Client
+from pickle import PicklingError
 from traceback import TracebackException
 
+import daft.pickle
 from daft.errors import UDFException
 from daft.execution.udf import (
     _ENTER,
@@ -17,7 +20,7 @@ from daft.execution.udf import (
     SharedMemoryTransport,
 )
 from daft.expressions.expressions import ExpressionsProjection
-from daft.recordbatch.micropartition import MicroPartition
+from daft.recordbatch import RecordBatch
 
 
 def udf_event_loop(
@@ -44,28 +47,41 @@ def udf_event_loop(
 
             # We initialize after ready to avoid blocking the main thread
             if expression_projection is None:
-                uninitialized_projection: ExpressionsProjection = pickle.loads(expr_projection_bytes)
-                initialized_projection = ExpressionsProjection([e._initialize_udfs() for e in uninitialized_projection])
-                expression_projection = initialized_projection
+                uninitialized_projection: ExpressionsProjection = daft.pickle.loads(expr_projection_bytes)
+                expression_projection = ExpressionsProjection([e._initialize_udfs() for e in uninitialized_projection])
 
             input_bytes = transport.read_and_release(name, size)
-            input = MicroPartition.from_ipc_stream(input_bytes)
+            input = RecordBatch.from_ipc_stream(input_bytes)
             evaluated = input.eval_expression_list(expression_projection)
 
             output_bytes = evaluated.to_ipc_stream()
             out_name, out_size = transport.write_and_close(output_bytes)
 
+            # Mark end of UDF's stdout and flush
             print(_OUTPUT_DIVIDER.decode(), end="", file=sys.stderr, flush=True)
             sys.stdout.flush()
             sys.stderr.flush()
+
             conn.send((_SUCCESS, out_name, out_size))
     except UDFException as e:
         exc = e.__cause__
         assert exc is not None
-        conn.send((_UDF_ERROR, e.message, TracebackException.from_exception(exc), pickle.dumps(exc)))
+        try:
+            exc_bytes = daft.pickle.dumps(exc)
+        except (PicklingError, AttributeError):
+            exc_bytes = None
+        conn.send((_UDF_ERROR, e.message, TracebackException.from_exception(exc), exc_bytes))
     except Exception as e:
         try:
-            conn.send((_ERROR, TracebackException.from_exception(e)))
+            tb = "\n".join(TracebackException.from_exception(e).format())
+        except Exception:
+            # If serialization fails, just send the exception's repr
+            # This sometimes happens on 3.9 & 3.10, but unclear why
+            # The repr doesn't contain the full traceback
+            tb = repr(e)
+
+        try:
+            conn.send((_ERROR, tb))
         except Exception:
             # If the connection is broken, it's because the parent process has died.
             # We can just exit here.
@@ -82,4 +98,10 @@ if __name__ == "__main__":
     socket_path = sys.argv[1]
     secret = bytes.fromhex(sys.argv[2])
 
+    logging.basicConfig(
+        level=int(os.getenv("LOG_LEVEL", logging.WARNING)),
+        format=os.getenv("LOG_FORMAT", "%(asctime)s - %(name)s - %(levelname)s - %(message)s"),
+        datefmt=os.getenv("LOG_DATE_FORMAT", "%Y-%m-%d %H:%M:%S"),
+    )
+
     udf_event_loop(secret, socket_path)
