@@ -10,6 +10,7 @@ from .expressions import (
     interval,
     struct,
     coalesce,
+    WhenExpr,
 )
 
 from .visitor import ExpressionVisitor
@@ -18,6 +19,7 @@ __all__ = [
     "Expression",
     "ExpressionVisitor",
     "ExpressionsProjection",
+    "WhenExpr",
     "coalesce",
     "col",
     "element",
