@@ -2,16 +2,14 @@
 
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Any, Literal
+from typing import Any, Literal
 
 import daft.daft as native
 from daft.datatype import DataType, DataTypeLike
 from daft.expressions import Expression
+from daft.expressions.expressions import WhenExpr
 from daft.series import item_to_series
 
-if TYPE_CHECKING:
-    from daft.io import IOConfig
-
 
 def monotonically_increasing_id() -> Expression:
     """Generates a column of monotonically increasing unique ids.
@@ -20,7 +18,7 @@ def monotonically_increasing_id() -> Expression:
     in the lower 36 bits. This allows for 2^28 ≈ 268 million partitions and 2^36 ≈ 68 billion rows per partition.
 
     Returns:
-        Expression: An expression that generates monotonically increasing IDs
+        Expression (UInt64 Expression): An expression that generates monotonically increasing IDs
 
     Examples:
         >>> import daft
@@ -51,15 +49,6 @@ def monotonically_increasing_id() -> Expression:
     return Expression._from_pyexpr(f())
 
 
-def file(expr: Expression, io_config: IOConfig | None = None) -> Expression:
-    """Converts either a string containing a file reference, or a binary column to a `daft.File` reference.
-
-    If the input is a string, it is assumed to be a file path and is converted to a `daft.File`.
-    If the input is a binary column, it is converted to a `daft.File` where the entire contents are buffered in memory.
-    """
-    return expr._eval_expressions("file", io_config=io_config)
-
-
 def eq_null_safe(left: Expression, right: Expression) -> Expression:
     """Performs a null-safe equality comparison between two expressions.
 
@@ -69,7 +58,7 @@ def eq_null_safe(left: Expression, right: Expression) -> Expression:
     - Behaves like regular equality for non-NULL values
 
     Returns:
-        Expression: A boolean expression indicating if the values are equal
+        Expression (Boolean Expression): A boolean expression indicating if the values are equal
     """
     left = Expression._to_expression(left)
     right = Expression._to_expression(right)
@@ -79,39 +68,12 @@ def eq_null_safe(left: Expression, right: Expression) -> Expression:
 def cast(expr: Expression, dtype: DataTypeLike) -> Expression:
     """Casts an expression to the given datatype if possible.
 
-    The following combinations of datatype casting is valid:
-
-    | Target →           | Null | Boolean | Integers | Floats | Decimal128 | String | Binary | Fixed-size Binary | Image | Fixed-shape Image | Embedding | Tensor | Fixed-shape Tensor | Python | List | Fixed-size List | Struct | Map | Timestamp | Date | Time | Duration |
-    | ------------------ | ---- | ------- | -------- | ------ | ---------- | ------ | ------ | ----------------- | ----- | ----------------- | --------- | ------ | ------------------ | ------ | ---- | --------------- | ------ | --- | --------- | ---- | ---- | -------- |
-    | **Source ↓**       |
-    | Null               | Y    | Y       | Y        | Y      | Y          | Y      | Y      | Y                 | N     | N                 | Y         | N      | N                  | Y      | Y    | Y               | Y      | Y   | Y         | Y    | Y    | Y        |
-    | Boolean            | Y    | Y       | Y        | Y      | N          | Y      | Y      | N                 | N     | N                 | N         | N      | N                  | Y      | N    | N               | N      | N   | N         | N    | N    | N        |
-    | Integers           | Y    | Y       | Y        | Y      | Y          | Y      | Y      | N                 | N     | N                 | N         | N      | N                  | Y      | N    | N               | N      | N   | Y         | Y    | Y    | Y        |
-    | Floats             | Y    | Y       | Y        | Y      | Y          | Y      | Y      | N                 | N     | N                 | N         | N      | N                  | Y      | N    | M               | N      | N   | Y         | Y    | Y    | Y        |
-    | Decimal128         | Y    | N       | Y        | Y      | Y          | N      | N      | N                 | N     | N                 | N         | N      | N                  | Y      | N    | N               | N      | N   | N         | N    | N    | N        |
-    | String             | Y    | N       | Y        | Y      | N          | Y      | Y      | N                 | N     | N                 | N         | N      | N                  | Y      | N    | N               | N      | N   | Y         | Y    | N    | N        |
-    | Binary             | Y    | N       | Y        | Y      | N          | Y      | Y      | Y                 | N     | N                 | N         | N      | N                  | Y      | N    | N               | N      | N   | N         | N    | N    | N        |
-    | Fixed-size Binary  | Y    | N       | N        | N      | N          | N      | Y      | N                 | N     | N                 | N         | N      | N                  | Y      | N    | N               | N      | N   | N         | N    | N    | N        |
-    | Image              | N    | N       | N        | N      | N          | N      | N      | N                 | Y     | Y                 | N         | Y      | Y                  | Y      | N    | N               | Y      | N   | N         | N    | N    | N        |
-    | Fixed-size Image   | N    | N       | N        | N      | N          | N      | N      | N                 | Y     | Y                 | N         | Y      | Y                  | Y      | Y    | Y               | N      | N   | N         | N    | N    | N        |
-    | Embedding          | Y    | N       | N        | N      | N          | N      | N      | N                 | N     | Y                 | N         | Y      | Y                  | Y      | Y    | Y               | N      | N   | N         | N    | N    | N        |
-    | Tensor             | Y    | N       | N        | N      | N          | N      | N      | N                 | Y     | Y                 | N         | Y      | Y                  | Y      | N    | N               | Y      | N   | N         | N    | N    | N        |
-    | Fixed-shape Tensor | N    | N       | N        | N      | N          | N      | N      | N                 | N     | Y                 | N         | Y      | Y                  | Y      | Y    | Y               | N      | N   | N         | N    | N    | N        |
-    | Python             | Y    | Y       | Y        | Y      | N          | Y      | Y      | Y                 | Y     | Y                 | Y         | Y      | Y                  | Y      | Y    | Y               | Y      | N   | N         | N    | N    | N        |
-    | List               | N    | N       | N        | N      | N          | N      | N      | N                 | N     | N                 | Y         | N      | N                  | N      | Y    | Y               | N      | Y   | N         | N    | N    | N        |
-    | Fixed-size List    | N    | N       | N        | N      | N          | N      | N      | N                 | N     | Y                 | N         | N      | Y                  | N      | Y    | Y               | N      | N   | N         | N    | N    | N        |
-    | Struct             | N    | N       | N        | N      | N          | N      | N      | N                 | Y     | N                 | N         | Y      | N                  | N      | N    | N               | Y      | N   | N         | N    | N    | N        |
-    | Map                | N    | N       | N        | N      | N          | N      | N      | N                 | N     | N                 | Y         | N      | N                  | N      | Y    | Y               | N      | Y   | N         | N    | N    | N        |
-    | Timestamp          | Y    | N       | Y        | Y      | N          | Y      | N      | N                 | N     | N                 | N         | N      | N                  | Y      | N    | N               | N      | N   | Y         | Y    | Y    | N        |
-    | Date               | Y    | N       | Y        | Y      | N          | Y      | N      | N                 | N     | N                 | N         | N      | N                  | Y      | N    | N               | N      | N   | Y         | Y    | N    | N        |
-    | Time               | Y    | N       | Y        | Y      | N          | Y      | N      | N                 | N     | N                 | N         | N      | N                  | Y      | N    | N               | N      | N   | N         | N    | Y    | N        |
-    | Duration           | Y    | N       | Y        | Y      | N          | N      | N      | N                 | N     | N                 | N         | N      | N                  | Y      | N    | N               | N      | N   | N         | N    | N    | N        |
+    See the [casting matrix](https://docs.daft.ai/en/stable/api/datatypes/casting/) for supported casts.
 
     Returns:
         Expression: Expression with the specified new datatype
 
     Note:
-        - Overflowing values will be wrapped, e.g. 256 will be cast to 0 for an unsigned 8-bit integer.
         - If a string is provided, it will use the sql engine to parse the string into a data type. See the [SQL Reference](https://docs.daft.ai/en/stable/sql/datatypes/) for supported datatypes.
         - a python `type` can also be provided, in which case the corresponding Daft data type will be used.
 
@@ -162,10 +124,7 @@ def cast(expr: Expression, dtype: DataTypeLike) -> Expression:
         <BLANKLINE>
         (Showing first 3 of 3 rows)
     """
-    if isinstance(dtype, str):
-        dtype = DataType._from_pydatatype(native.sql_datatype(dtype))
-    else:
-        dtype = DataType._infer_type(dtype)
+    dtype = DataType._infer(dtype)
     expr = Expression._to_expression(expr)
     return Expression._from_pyexpr(expr._expr.cast(dtype._dtype))
 
@@ -174,7 +133,7 @@ def is_null(expr: Expression) -> Expression:
     """Checks if values in the Expression are Null (a special value indicating missing data).
 
     Returns:
-        Expression: Boolean Expression indicating whether values are missing
+        Expression (Boolean Expression): expression indicating whether values are missing
 
     Examples:
         >>> import daft
@@ -206,7 +165,7 @@ def not_null(expr: Expression) -> Expression:
     """Checks if values in the Expression are not Null (a special value indicating missing data).
 
     Returns:
-        Expression: Boolean Expression indicating whether values are not missing
+        Expression (Boolean Expression): expression indicating whether values are not missing
 
     Examples:
         >>> import daft
@@ -271,7 +230,7 @@ def is_in(expr: Expression, other: Any) -> Expression:
     """Checks if values in the Expression are in the provided list.
 
     Returns:
-        Expression: Boolean Expression indicating whether values are in the provided list
+        Expression (Boolean Expression): expression indicating whether values are in the provided list
 
     Examples:
         >>> import daft
@@ -321,6 +280,9 @@ def hash(
         seed (optional): Seed used for generating the hash. Defaults to 0.
         hash_function (optional): Hash function to use. One of "xxhash", "murmurhash3", or "sha1". Defaults to "xxhash".
 
+    Returns:
+        Expression (UInt64 Expression): The hashed expression.
+
     Note:
         Null values will produce a hash value instead of being propagated as null.
 
@@ -335,7 +297,7 @@ def hash(
 
 
 def minhash(
-    expr: Expression,
+    text: Expression,
     *,
     num_hashes: int,
     ngram_size: int,
@@ -352,28 +314,35 @@ def minhash(
     to normalize the strings yourself.
 
     Args:
-        expr: The expression to hash.
-        num_hashes: The number of hash permutations to compute.
-        ngram_size: The number of tokens in each shingle/ngram.
-        seed (optional): Seed used for generating permutations and the initial string hashes. Defaults to 1.
-        hash_function (optional): Hash function to use for initial string hashing. One of "murmurhash3", "xxhash", or "sha1". Defaults to "murmurhash3".
+        text (String Expression): expression to hash.
+        num_hashes (int): The number of hash permutations to compute.
+        ngram_size (int): The number of tokens in each shingle/ngram.
+        seed (int, default=1): Seed used for generating permutations and the initial string hashes. Defaults to 1.
+        hash_function (str, default="murmurhash3"): Hash function to use for initial string hashing. One of "murmurhash3", "xxhash", or "sha1". Defaults to "murmurhash3".
+
+    Returns:
+        Expression (FixedSizedList[UInt32, num_hashes] Expression):
+            expression representing the MinHash values.
 
     """
     return Expression._call_builtin_scalar_fn(
-        "minhash", expr, num_hashes=num_hashes, ngram_size=ngram_size, seed=seed, hash_function=hash_function
+        "minhash", text, num_hashes=num_hashes, ngram_size=ngram_size, seed=seed, hash_function=hash_function
     )
 
 
 def length(expr: Expression) -> Expression:
     """Retrieves the length of the given expression.
 
+    Args:
+        expr (List or Binary or String Expression): expression to compute the length of.
+
     The behavior depends on the input type:
     - For strings, returns the number of characters.
     - For binary, returns the number of bytes.
     - For lists, returns the number of elements.
 
     Returns:
-        Expression: an UInt64 expression with the length
+        Expression (UInt64 Expression): an expression with the length
 
     Examples:
         String length:
@@ -439,6 +408,10 @@ def length(expr: Expression) -> Expression:
 def concat(left: Expression | str | bytes, right: Expression | str | bytes) -> Expression:
     r"""Concatenates two string or binary values.
 
+    Args:
+        left ((String or Binary Expression) | str | bytes): the left value to concatenate
+        right ((String or Binary Expression) | str | bytes): the right value to concatenate
+
     Returns:
         Expression: an expression with the same type as the inputs
 
@@ -528,7 +501,7 @@ def get(expr: Expression, key: int | str | Expression, default: Any = None) -> E
     """Get an index from a list expression or a field from a struct expression.
 
     Args:
-        expr: list or struct expression to get value from
+        expr (List or Struct Expression): to get value from
         key: integer index for list or string field for struct. List index can be negative to index from the end of the list.
         default: default value if out of bounds. Only supported for list get
 
@@ -737,3 +710,141 @@ def slice(expr: Expression, start: int | Expression, end: int | Expression | Non
         (Showing first 3 of 3 rows)
     """
     return Expression._call_builtin_scalar_fn("slice", expr, start, end=end)
+
+
+def when(condition: Expression | bool, then: Expression | Any) -> WhenExpr:
+    """Start a conditional expression, similar to SQL CASE WHEN.
+
+    If the condition is true, the `then` value will be returned. Otherwise, the next `when` condition will be evaluated.
+    If no conditions are true, the value will be set to the value provided in the `otherwise` clause, or null if not provided.
+
+    Args:
+        condition: The Boolean expression to evaluate
+        then: Expression to return when the condition is true
+
+    Returns:
+        A WhenExpr that can be chained with more `when` clauses and ended with `otherwise`
+
+    Examples:
+        Simple conditional assignment:
+        >>> import daft
+        >>> from daft.functions import when
+        >>>
+        >>> df = daft.from_pydict({"x": [1, 2, 3, 4, 5]})
+        >>> df = df.select(when(df["x"] > 3, then="high").otherwise("low").alias("category"))
+        >>> df.show()
+        ╭──────────╮
+        │ category │
+        │ ---      │
+        │ Utf8     │
+        ╞══════════╡
+        │ low      │
+        ├╌╌╌╌╌╌╌╌╌╌┤
+        │ low      │
+        ├╌╌╌╌╌╌╌╌╌╌┤
+        │ low      │
+        ├╌╌╌╌╌╌╌╌╌╌┤
+        │ high     │
+        ├╌╌╌╌╌╌╌╌╌╌┤
+        │ high     │
+        ╰──────────╯
+        <BLANKLINE>
+        (Showing first 5 of 5 rows)
+
+        Multiple conditions using chained `when` clauses:
+        >>> df = daft.from_pydict({"score": [85, 92, 78, 65, 88]})
+        >>> df = df.select(
+        ...     when(df["score"] >= 90, then="A")
+        ...     .when(df["score"] >= 80, then="B")
+        ...     .when(df["score"] >= 70, then="C")
+        ...     .otherwise("F")
+        ...     .alias("grade")
+        ... )
+        >>> df.show()
+        ╭───────╮
+        │ grade │
+        │ ---   │
+        │ Utf8  │
+        ╞═══════╡
+        │ B     │
+        ├╌╌╌╌╌╌╌┤
+        │ A     │
+        ├╌╌╌╌╌╌╌┤
+        │ C     │
+        ├╌╌╌╌╌╌╌┤
+        │ F     │
+        ├╌╌╌╌╌╌╌┤
+        │ B     │
+        ╰───────╯
+        <BLANKLINE>
+        (Showing first 5 of 5 rows)
+
+        Using complex conditions and returning different data types:
+        >>> df = daft.from_pydict({"name": ["Alice", "Bob", "Charlie"], "age": [25, 17, 35]})
+        >>> df = df.select(
+        ...     df["name"],
+        ...     when((df["age"] >= 18) & (df["age"] < 65), then=df["age"])
+        ...     .when(df["age"] < 18, then=-1)
+        ...     .otherwise(0)
+        ...     .alias("working_age"),
+        ... )
+        >>> df.show()
+        ╭─────────┬─────────────╮
+        │ name    ┆ working_age │
+        │ ---     ┆ ---         │
+        │ Utf8    ┆ Int64       │
+        ╞═════════╪═════════════╡
+        │ Alice   ┆ 25          │
+        ├╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+        │ Bob     ┆ -1          │
+        ├╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌┤
+        │ Charlie ┆ 35          │
+        ╰─────────┴─────────────╯
+        <BLANKLINE>
+        (Showing first 3 of 3 rows)
+
+        Handling null values:
+        >>> df = daft.from_pydict({"value": [10, None, 20, 0]})
+        >>> df = df.select(
+        ...     when(df["value"].is_null(), then="missing")
+        ...     .when(df["value"] == 0, then="zero")
+        ...     .when(df["value"] > 15, then="high")
+        ...     .otherwise("normal")
+        ...     .alias("status")
+        ... )
+        >>> df.show()
+        ╭─────────╮
+        │ status  │
+        │ ---     │
+        │ Utf8    │
+        ╞═════════╡
+        │ normal  │
+        ├╌╌╌╌╌╌╌╌╌┤
+        │ missing │
+        ├╌╌╌╌╌╌╌╌╌┤
+        │ high    │
+        ├╌╌╌╌╌╌╌╌╌┤
+        │ zero    │
+        ╰─────────╯
+        <BLANKLINE>
+        (Showing first 4 of 4 rows)
+
+        Without `otherwise` clause (returns null when no conditions match):
+        >>> df = daft.from_pydict({"x": [1, 2, 3]})
+        >>> df = df.select(when(df["x"] > 1, then="big").alias("result"))
+        >>> df.show()
+        ╭────────╮
+        │ result │
+        │ ---    │
+        │ Utf8   │
+        ╞════════╡
+        │ None   │
+        ├╌╌╌╌╌╌╌╌┤
+        │ big    │
+        ├╌╌╌╌╌╌╌╌┤
+        │ big    │
+        ╰────────╯
+        <BLANKLINE>
+        (Showing first 3 of 3 rows)
+    """
+    return WhenExpr([]).when(condition, then)
