@@ -6,24 +6,19 @@ from daft import context
 from daft.api_annotations import PublicAPI
 from daft.daft import IOConfig, ScanOperatorHandle
 from daft.dataframe import DataFrame
-from daft.dependencies import pa
 from daft.io.lance.lance_merge_column import merge_columns_internal
 from daft.io.lance.lance_scan import LanceDBScanOperator
 from daft.io.object_store_options import io_config_to_storage_options
 from daft.logical.builder import LogicalPlanBuilder
 
 if TYPE_CHECKING:
+    from daft.dependencies import pa
+
     try:
         from lance.udf import BatchUDF
     except ImportError:
         BatchUDF = None
 
-# Runtime import for BatchUDF
-try:
-    from lance.udf import BatchUDF
-except ImportError:
-    BatchUDF = None
-
 
 @PublicAPI
 def read_lance(
@@ -36,6 +31,7 @@ def read_lance(
     index_cache_size: Optional[int] = None,
     default_scan_options: Optional[dict[str, str]] = None,
     metadata_cache_size_bytes: Optional[int] = None,
+    fragment_group_size: Optional[int] = None,
 ) -> DataFrame:
     """Create a DataFrame from a LanceDB table.
 
@@ -60,12 +56,8 @@ def read_lance(
 
             Roughly, for an ``IVF_PQ`` partition with ``n`` rows, the size of each index
             page equals the combination of the pq code (``np.array([n,pq], dtype=uint8))``
-            and the row ids (``np.array([n], dtype=uint64)``).
             Approximately, ``n = Total Rows / number of IVF partitions``.
             ``pq = number of PQ sub-vectors``.
-        storage_options : optional, dict
-            Extra options that make sense for a particular storage connection. This is
-            used to store connection parameters like credentials, endpoint, etc.
         default_scan_options : optional, dict
             Default scan options that are used when scanning the dataset.  This accepts
             the same arguments described in :py:meth:`lance.LanceDataset.scanner`.  The
@@ -82,11 +74,13 @@ def read_lance(
             Size of the metadata cache in bytes. This cache is used to store metadata
             information about the dataset, such as schema and statistics. If not specified,
             a default size will be used.
+        fragment_group_size : optional, int
+            Number of fragments to group together in a single scan task. If None or <= 1,
+            each fragment will be processed individually (default behavior).
 
     Returns:
         DataFrame: a DataFrame with the schema converted from the specified LanceDB table
 
-    Note:
         This function requires the use of [LanceDB](https://lancedb.github.io/lancedb/), which is the Python library for the LanceDB project.
         To ensure that this is installed with Daft, you may install: `pip install daft[lance]`
 
@@ -104,6 +98,10 @@ def read_lance(
         Read a local LanceDB table and specify a version:
         >>> df = daft.read_lance("s3://my-lancedb-bucket/data/", version=1)
         >>> df.show()
+
+        Read a local LanceDB table with fragment grouping:
+        >>> df = daft.read_lance("s3://my-lancedb-bucket/data/", fragment_group_size=5)
+        >>> df.show()
     """
     try:
         import lance
@@ -126,7 +124,7 @@ def read_lance(
         default_scan_options=default_scan_options,
         metadata_cache_size_bytes=metadata_cache_size_bytes,
     )
-    lance_operator = LanceDBScanOperator(ds)
+    lance_operator = LanceDBScanOperator(ds, fragment_group_size=fragment_group_size)
 
     handle = ScanOperatorHandle.from_python_scan_operator(lance_operator)
     builder = LogicalPlanBuilder.from_tabular_scan(scan_operator=handle)
@@ -138,9 +136,9 @@ def merge_columns(
     url: str,
     io_config: Optional[IOConfig] = None,
     *,
-    transform: Union[dict[str, str], BatchUDF, Callable[[pa.lib.RecordBatch], pa.lib.RecordBatch]] = None,
+    transform: Union[dict[str, str], "BatchUDF", Callable[["pa.lib.RecordBatch"], "pa.lib.RecordBatch"]] = None,
     read_columns: Optional[list[str]] = None,
-    reader_schema: Optional[pa.Schema] = None,
+    reader_schema: Optional["pa.Schema"] = None,
     storage_options: Optional[dict[str, Any]] = None,
     daft_remote_args: Optional[dict[str, Any]] = None,
     concurrency: Optional[int] = None,
