@@ -21,6 +21,18 @@ if TYPE_CHECKING:
     from daft.daft import IOConfig
 
 
+def pyarrow_schema_castable(src: pa.Schema, dst: pa.Schema) -> bool:
+    if len(src) != len(dst):
+        return False
+    for src_field, dst_field in zip(src, dst):
+        empty_array = pa.array([], type=src_field.type)
+        try:
+            empty_array.cast(dst_field.type)
+        except Exception:
+            return False
+    return True
+
+
 class LanceDataSink(DataSink[list[lance.FragmentMetadata]]):
     """WriteSink for writing data to a Lance dataset."""
 
@@ -35,7 +47,7 @@ class LanceDataSink(DataSink[list[lance.FragmentMetadata]]):
     def __init__(
         self,
         uri: str | pathlib.Path,
-        schema: Schema,
+        schema: Schema | pa.Schema,
         mode: Literal["create", "append", "overwrite"],
         io_config: IOConfig | None = None,
         **kwargs: Any,
@@ -43,7 +55,6 @@ class LanceDataSink(DataSink[list[lance.FragmentMetadata]]):
         from daft.io.object_store_options import io_config_to_storage_options
 
         lance = self._import_lance()
-
         if not isinstance(uri, (str, pathlib.Path)):
             raise TypeError(f"Expected URI to be str or pathlib.Path, got {type(uri)}")
         self._table_uri = str(uri)
@@ -53,22 +64,29 @@ class LanceDataSink(DataSink[list[lance.FragmentMetadata]]):
 
         self._storage_options = io_config_to_storage_options(self._io_config, self._table_uri)
 
-        self._pyarrow_schema = schema.to_pyarrow_schema()
+        if isinstance(schema, Schema):
+            self._pyarrow_schema = schema.to_pyarrow_schema()
+        elif isinstance(schema, pa.Schema):
+            self._pyarrow_schema = schema
+        else:
+            raise TypeError(f"Expected schema to be Schema or pa.Schema, got {type(schema)}")
 
         try:
             table = lance.dataset(self._table_uri, storage_options=self._storage_options)
-
         except ValueError:
             table = None
 
-        self._version = 0
-        if table:
-            table_schema = table.schema
+        self._version: int = 0
+        self._table_schema: pa.Schema | None = None
+        if table is not None:
+            self._table_schema = table.schema
             self._version = table.latest_version
-            if self._pyarrow_schema != table_schema and not (self._mode == "overwrite"):
+            if not pyarrow_schema_castable(self._pyarrow_schema, self._table_schema) and not (
+                self._mode == "overwrite"
+            ):
                 raise ValueError(
                     "Schema of data does not match table schema\n"
-                    f"Data schema:\n{self._pyarrow_schema}\nTable Schema:\n{table_schema}"
+                    f"Data schema:\n{self._pyarrow_schema}\nTable Schema:\n{self._table_schema}"
                 )
 
         self._schema = Schema._from_field_name_and_types(
@@ -92,7 +110,21 @@ class LanceDataSink(DataSink[list[lance.FragmentMetadata]]):
         lance = self._import_lance()
 
         for micropartition in micropartitions:
-            arrow_table = pa.Table.from_batches(micropartition.to_arrow().to_batches(), self._pyarrow_schema)
+            # Build an Arrow table that conforms to either:
+            # - the existing dataset schema (if table already exists), or
+            # - the user-provided schema (if specified and different from incoming data), or
+            # - the incoming data schema (no-op) while attaching metadata/order from _pyarrow_schema.
+            input_table = micropartition.to_arrow()
+            if self._table_schema is not None:
+                # Dataset exists: always cast to the table schema to ensure compatibility on append
+                arrow_table = input_table.cast(self._table_schema)
+            elif not pa.Schema.equals(self._pyarrow_schema, input_table.schema):
+                # New dataset or overwrite with a user-provided schema: cast to enforce order/types/nullability
+                arrow_table = input_table.cast(self._pyarrow_schema)
+            else:
+                # Schemas are identical: rebuild table with the desired schema instance to preserve metadata
+                arrow_table = pa.Table.from_batches(input_table.to_batches(), self._pyarrow_schema)
+
             bytes_written = arrow_table.nbytes
             rows_written = arrow_table.num_rows
 
@@ -121,7 +153,10 @@ class LanceDataSink(DataSink[list[lance.FragmentMetadata]]):
             operation = lance.LanceOperation.Append(fragments)
 
         dataset = lance.LanceDataset.commit(
-            self._table_uri, operation, read_version=self._version, storage_options=self._storage_options
+            self._table_uri,
+            operation,
+            read_version=self._version,
+            storage_options=self._storage_options,
         )
         stats = dataset.stats.dataset_stats()
 
