@@ -56,7 +56,7 @@ class RaySwordfishActor:
     async def run_plan(
         self,
         plan: LocalPhysicalPlan,
-        config: PyDaftExecutionConfig,
+        exec_cfg: PyDaftExecutionConfig,
         psets: dict[str, list[ray.ObjectRef]],
         context: dict[str, str] | None,
     ) -> AsyncGenerator[MicroPartition | list[PartitionMetadata], None]:
@@ -67,7 +67,7 @@ class RaySwordfishActor:
 
             metas = []
             native_executor = NativeExecutor()
-            async for partition in native_executor.run_async(plan, psets_mp, config, None, context):
+            async for partition in native_executor.run_async(plan, psets_mp, exec_cfg, None, context):
                 if partition is None:
                     break
                 mp = MicroPartition._from_pymicropartition(partition)
@@ -287,6 +287,7 @@ class FlotillaRunner:
             name=FLOTILLA_RUNNER_NAME,
             namespace=FLOTILLA_RUNNER_NAMESPACE,
             get_if_exists=True,
+            lifetime="detached",
             scheduling_strategy=(
                 ray.util.scheduling_strategies.NodeAffinitySchedulingStrategy(
                     node_id=head_node_id,
