@@ -1,4 +1,3 @@
-#![feature(let_chains)]
 pub use common_io_config::IOConfig;
 use serde::{Deserialize, Serialize};
 
@@ -48,7 +47,6 @@ pub struct DaftExecutionConfig {
     pub scan_tasks_max_size_bytes: usize,
     pub max_sources_per_scan_task: usize,
     pub broadcast_join_size_bytes_threshold: usize,
-    pub sort_merge_join_sort_with_aligned_boundaries: bool,
     pub hash_join_partition_size_leniency: f64,
     pub sample_size_for_sort: usize,
     pub parquet_split_row_groups_max_files: usize,
@@ -73,8 +71,9 @@ pub struct DaftExecutionConfig {
     pub scantask_splitting_level: i32,
     pub scantask_max_parallel: usize,
     pub native_parquet_writer: bool,
-    pub use_experimental_distributed_engine: bool,
+    pub use_legacy_ray_runner: bool,
     pub min_cpu_per_task: f64,
+    pub actor_udf_ready_timeout: usize,
 }
 
 impl Default for DaftExecutionConfig {
@@ -84,7 +83,6 @@ impl Default for DaftExecutionConfig {
             scan_tasks_max_size_bytes: 384 * 1024 * 1024, // 384MB
             max_sources_per_scan_task: 10,
             broadcast_join_size_bytes_threshold: 10 * 1024 * 1024, // 10 MiB
-            sort_merge_join_sort_with_aligned_boundaries: false,
             hash_join_partition_size_leniency: 0.5,
             sample_size_for_sort: 20,
             parquet_split_row_groups_max_files: 10,
@@ -109,8 +107,9 @@ impl Default for DaftExecutionConfig {
             scantask_splitting_level: 1,
             scantask_max_parallel: 8,
             native_parquet_writer: true,
-            use_experimental_distributed_engine: true,
+            use_legacy_ray_runner: false,
             min_cpu_per_task: 0.5,
+            actor_udf_ready_timeout: 120,
         }
     }
 }
@@ -161,8 +160,7 @@ impl DaftExecutionConfig {
         }
         let flotilla_env_var_name = "DAFT_FLOTILLA";
         if let Ok(val) = std::env::var(flotilla_env_var_name) {
-            cfg.use_experimental_distributed_engine =
-                !matches!(val.trim().to_lowercase().as_str(), "0" | "false");
+            cfg.use_legacy_ray_runner = matches!(val.trim().to_lowercase().as_str(), "0" | "false");
         }
         let min_cpu_var = "DAFT_MIN_CPU_PER_TASK";
         if let Ok(val) = std::env::var(min_cpu_var) {
