@@ -94,7 +94,6 @@ impl PyDaftExecutionConfig {
         max_sources_per_scan_task=None,
         broadcast_join_size_bytes_threshold=None,
         parquet_split_row_groups_max_files=None,
-        sort_merge_join_sort_with_aligned_boundaries=None,
         hash_join_partition_size_leniency=None,
         sample_size_for_sort=None,
         num_preview_rows=None,
@@ -116,8 +115,9 @@ impl PyDaftExecutionConfig {
         scantask_splitting_level=None,
         scantask_max_parallel=None,
         native_parquet_writer=None,
-        use_experimental_distributed_engine=None,
+        use_legacy_ray_runner=None,
         min_cpu_per_task=None,
+        actor_udf_ready_timeout=None,
     ))]
     fn with_config_values(
         &self,
@@ -126,7 +126,6 @@ impl PyDaftExecutionConfig {
         max_sources_per_scan_task: Option<usize>,
         broadcast_join_size_bytes_threshold: Option<usize>,
         parquet_split_row_groups_max_files: Option<usize>,
-        sort_merge_join_sort_with_aligned_boundaries: Option<bool>,
         hash_join_partition_size_leniency: Option<f64>,
         sample_size_for_sort: Option<usize>,
         num_preview_rows: Option<usize>,
@@ -148,8 +147,9 @@ impl PyDaftExecutionConfig {
         scantask_splitting_level: Option<i32>,
         scantask_max_parallel: Option<usize>,
         native_parquet_writer: Option<bool>,
-        use_experimental_distributed_engine: Option<bool>,
+        use_legacy_ray_runner: Option<bool>,
         min_cpu_per_task: Option<f64>,
+        actor_udf_ready_timeout: Option<usize>,
     ) -> PyResult<Self> {
         let mut config = self.config.as_ref().clone();
 
@@ -168,12 +168,6 @@ impl PyDaftExecutionConfig {
         if let Some(parquet_split_row_groups_max_files) = parquet_split_row_groups_max_files {
             config.parquet_split_row_groups_max_files = parquet_split_row_groups_max_files;
         }
-        if let Some(sort_merge_join_sort_with_aligned_boundaries) =
-            sort_merge_join_sort_with_aligned_boundaries
-        {
-            config.sort_merge_join_sort_with_aligned_boundaries =
-                sort_merge_join_sort_with_aligned_boundaries;
-        }
         if let Some(hash_join_partition_size_leniency) = hash_join_partition_size_leniency {
             config.hash_join_partition_size_leniency = hash_join_partition_size_leniency;
         }
@@ -258,14 +252,18 @@ impl PyDaftExecutionConfig {
             config.native_parquet_writer = native_parquet_writer;
         }
 
-        if let Some(use_experimental_distributed_engine) = use_experimental_distributed_engine {
-            config.use_experimental_distributed_engine = use_experimental_distributed_engine;
+        if let Some(use_legacy_ray_runner) = use_legacy_ray_runner {
+            config.use_legacy_ray_runner = use_legacy_ray_runner;
         }
 
         if let Some(min_cpu_per_task) = min_cpu_per_task {
             config.min_cpu_per_task = min_cpu_per_task;
         }
 
+        if let Some(actor_udf_ready_timeout) = actor_udf_ready_timeout {
+            config.actor_udf_ready_timeout = actor_udf_ready_timeout;
+        }
+
         Ok(Self {
             config: Arc::new(config),
         })
@@ -291,11 +289,6 @@ impl PyDaftExecutionConfig {
         Ok(self.config.broadcast_join_size_bytes_threshold)
     }
 
-    #[getter]
-    fn get_sort_merge_join_sort_with_aligned_boundaries(&self) -> PyResult<bool> {
-        Ok(self.config.sort_merge_join_sort_with_aligned_boundaries)
-    }
-
     #[getter]
     fn get_hash_join_partition_size_leniency(&self) -> PyResult<f64> {
         Ok(self.config.hash_join_partition_size_leniency)
@@ -388,14 +381,19 @@ impl PyDaftExecutionConfig {
     }
 
     #[getter]
-    fn use_experimental_distributed_engine(&self) -> PyResult<bool> {
-        Ok(self.config.use_experimental_distributed_engine)
+    fn use_legacy_ray_runner(&self) -> PyResult<bool> {
+        Ok(self.config.use_legacy_ray_runner)
     }
 
     #[getter]
     fn min_cpu_per_task(&self) -> PyResult<f64> {
         Ok(self.config.min_cpu_per_task)
     }
+
+    #[getter]
+    fn actor_udf_ready_timeout(&self) -> PyResult<usize> {
+        Ok(self.config.actor_udf_ready_timeout)
+    }
 }
 
 impl_bincode_py_state_serialization!(PyDaftExecutionConfig);
