@@ -1,6 +1,8 @@
 mod bbox;
 mod cow_image;
 mod image;
+#[cfg(feature = "python")]
+mod python;
 
 pub use bbox::BBox;
 pub use cow_image::CowImage;
