@@ -1,17 +1,19 @@
 use common_display::table_display::StrValue;
 use common_error::DaftResult;
 
+#[cfg(feature = "python")]
+use crate::prelude::PythonArray;
 use crate::{
     array::{DataArray, FixedSizeListArray, ListArray, StructArray},
     datatypes::{
+        BinaryArray, BooleanArray, DaftNumericType, DataType, Decimal128Array, ExtensionArray,
+        FileArray, FixedSizeBinaryArray, IntervalArray, IntervalValue, NullArray, UInt64Array,
+        Utf8Array,
         logical::{
             DateArray, DurationArray, EmbeddingArray, FixedShapeImageArray,
             FixedShapeSparseTensorArray, FixedShapeTensorArray, ImageArray, MapArray,
             SparseTensorArray, TensorArray, TimeArray, TimestampArray,
         },
-        BinaryArray, BooleanArray, DaftNumericType, DataType, Decimal128Array, ExtensionArray,
-        FileArray, FixedSizeBinaryArray, IntervalArray, IntervalValue, NullArray, UInt64Array,
-        Utf8Array,
     },
     series::Series,
     utils::display::{
@@ -141,19 +143,23 @@ impl FixedSizeBinaryArray {
         }
     }
 }
+
 #[cfg(feature = "python")]
-impl crate::datatypes::PythonArray {
+impl PythonArray {
     pub fn str_value(&self, idx: usize) -> DaftResult<String> {
         use pyo3::prelude::*;
 
         let val = self.get(idx);
 
-        let call_result =
-            Python::with_gil(|py| val.call_method0(py, pyo3::intern!(py, "__str__")))?;
-
-        let extracted = Python::with_gil(|py| call_result.extract(py))?;
-
-        Ok(extracted)
+        if let Some(val) = val {
+            Ok(Python::with_gil(|py| {
+                val.bind(py)
+                    .call_method0(pyo3::intern!(py, "__str__"))?
+                    .extract()
+            })?)
+        } else {
+            Ok("None".to_string())
+        }
     }
 }
 
@@ -488,24 +494,28 @@ impl_array_html_value!(TimestampArray);
 impl_array_html_value!(EmbeddingArray);
 
 #[cfg(feature = "python")]
-impl crate::datatypes::PythonArray {
+impl PythonArray {
     pub fn html_value(&self, idx: usize, truncate: bool) -> String {
         use pyo3::prelude::*;
 
         let val = self.get(idx);
 
         let custom_viz_hook_result: Option<String> = Python::with_gil(|py| {
-            // Find visualization hooks for this object's class
-            let pyany = val.bind(py);
-            let get_viz_hook = py
-                .import(pyo3::intern!(py, "daft.viz.html_viz_hooks"))?
-                .getattr(pyo3::intern!(py, "get_viz_hook"))?;
-            let hook = get_viz_hook.call1((pyany,))?;
+            if let Some(val) = val {
+                // Find visualization hooks for this object's class
+                let pyany = val.bind(py);
+                let get_viz_hook = py
+                    .import(pyo3::intern!(py, "daft.viz.html_viz_hooks"))?
+                    .getattr(pyo3::intern!(py, "get_viz_hook"))?;
+                let hook = get_viz_hook.call1((pyany,))?;
 
-            if hook.is_none() {
-                Ok(None)
+                if hook.is_none() {
+                    Ok(None)
+                } else {
+                    hook.call1((pyany,))?.extract()
+                }
             } else {
-                hook.call1((pyany,))?.extract()
+                Ok(None)
             }
         })
         .unwrap();
