@@ -1,15 +1,15 @@
 use common_error::{DaftError, DaftResult};
-use common_file::DaftFile;
+use common_file::FileReference;
 use common_io_config::IOConfig;
 #[cfg(feature = "python")]
 use pyo3::{PyClass, Python};
 
-use super::{deserializer::LiteralDeserializer, FromLiteral, Literal};
+use super::{FromLiteral, Literal, deserializer::LiteralDeserializer};
 #[cfg(feature = "python")]
 use crate::python::{PyDataType, PyTimeUnit};
 use crate::{
     datatypes::IntervalValue,
-    prelude::{CountMode, DataType, ImageFormat, ImageMode, TimeUnit},
+    prelude::{CountMode, DataType, ImageFormat, ImageMode, ImageProperty, TimeUnit},
     series::Series,
 };
 
@@ -44,7 +44,7 @@ impl_literal!(f64, Float64);
 impl_literal!(IntervalValue, Interval);
 impl_literal!(String, Utf8);
 impl_literal!(Series, List);
-impl_literal!(DaftFile, File);
+impl_literal!(FileReference, File);
 
 impl_literal!(&'_ str, Utf8, |s: &'_ str| s.to_owned());
 impl_literal!(&'_ [u8], Binary, |s: &'_ [u8]| s.to_vec());
@@ -113,7 +113,7 @@ macro_rules! impl_int_fromliteral {
                     _ => {
                         return Err(DaftError::ValueError(format!(
                             "Expected integer number, received: {lit}"
-                        )))
+                        )));
                     }
                 };
 
@@ -147,7 +147,7 @@ macro_rules! impl_float_fromliteral {
                     _ => {
                         return Err(DaftError::ValueError(format!(
                             "Expected floating point number, received: {lit}"
-                        )))
+                        )));
                     }
                 };
 
@@ -208,7 +208,7 @@ where
 
 impl_strict_fromliteral!(String, Utf8);
 impl_strict_fromliteral!(bool, Boolean);
-impl_strict_fromliteral!(DaftFile, File);
+impl_strict_fromliteral!(FileReference, File);
 impl_int_fromliteral!(i8);
 impl_int_fromliteral!(u8);
 impl_int_fromliteral!(i16);
@@ -223,7 +223,8 @@ impl_float_fromliteral!(f32);
 impl_float_fromliteral!(f64);
 impl_pyobj_fromliteral!(IOConfig, common_io_config::python::IOConfig);
 impl_pyobj_fromliteral!(ImageMode, ImageMode);
-impl_pyobj_fromliteral!(ImageFormat, ImageFormat);
+impl_pyobj_fromliteral!(ImageProperty, ImageProperty);
 impl_pyobj_fromliteral!(CountMode, CountMode);
 impl_pyobj_fromliteral!(TimeUnit, PyTimeUnit);
 impl_pyobj_fromliteral!(DataType, PyDataType);
+impl_pyobj_fromliteral!(ImageFormat, ImageFormat);
