@@ -10,7 +10,7 @@ use std::{
 
 use common_display::table_display::StrValue;
 use common_error::{DaftError, DaftResult, ensure};
-use common_file::DaftFile;
+use common_file::FileReference;
 use common_hashable_float_wrapper::FloatWrapper;
 use common_image::{CowImage, Image};
 #[cfg(feature = "python")]
@@ -22,6 +22,7 @@ use serde::{Deserialize, Serialize};
 use crate::{
     datatypes::IntervalValue,
     prelude::*,
+    series::from_lit::combine_lit_types,
     utils::display::{
         display_date32, display_decimal128, display_duration, display_series_in_literal,
         display_time64, display_timestamp,
@@ -91,8 +92,8 @@ pub enum Literal {
     #[cfg(feature = "python")]
     Python(PyObjectWrapper),
     /// TODO chore: audit struct literal vs. struct expression support.
-    Struct(IndexMap<Field, Literal>),
-    File(DaftFile),
+    Struct(IndexMap<String, Literal>),
+    File(FileReference),
     /// A tensor
     Tensor {
         data: Series,
@@ -236,11 +237,11 @@ impl Display for Literal {
             }),
             Self::Struct(entries) => {
                 write!(f, "Struct(")?;
-                for (i, (field, v)) in entries.iter().enumerate() {
+                for (i, (k, v)) in entries.iter().enumerate() {
                     if i > 0 {
                         write!(f, ", ")?;
                     }
-                    write!(f, "{}: {}", field.name, v)?;
+                    write!(f, "{}: {}", k, v)?;
                 }
                 write!(f, ")")
             }
@@ -323,7 +324,12 @@ impl Literal {
             Self::List(series) => DataType::List(Box::new(series.data_type().clone())),
             #[cfg(feature = "python")]
             Self::Python(_) => DataType::Python,
-            Self::Struct(entries) => DataType::Struct(entries.keys().cloned().collect()),
+            Self::Struct(entries) => DataType::Struct(
+                entries
+                    .iter()
+                    .map(|(k, v)| Field::new(k, v.get_type()))
+                    .collect(),
+            ),
             Self::File(_) => DataType::File,
             Self::Tensor { data, .. } => DataType::Tensor(Box::new(data.data_type().clone())),
             Self::SparseTensor {
@@ -548,7 +554,7 @@ impl Literal {
     }
 
     /// If the literal is a struct, return the reference to its map. Otherwise, return None.
-    pub fn as_struct(&self) -> Option<&IndexMap<Field, Self>> {
+    pub fn as_struct(&self) -> Option<&IndexMap<String, Self>> {
         match self {
             Self::Struct(map) => Some(map),
             _ => None,
@@ -571,10 +577,23 @@ impl Literal {
         // A "struct" literal is a strange concept, and only makes
         // sense that it predates the struct expression. The literals
         // tell us the type, so need to give before construction.
-        let iter_with_types = iter
-            .into_iter()
-            .map(|(name, lit)| (Field::new(name, lit.get_type()), lit));
-        Self::Struct(IndexMap::from_iter(iter_with_types))
+        Self::Struct(IndexMap::from_iter(iter))
+    }
+
+    /// Cast the literal to a data type.
+    ///
+    /// This method is lossy, AKA it is not guaranteed that `lit.cast(dtype).get_type() == dtype`.
+    /// This is because null literals always have the null data type.
+    pub fn cast(self, dtype: &DataType) -> DaftResult<Self> {
+        if &self.get_type() == dtype
+            || (combine_lit_types(&self.get_type(), dtype).as_ref() == Some(dtype))
+        {
+            Ok(self)
+        } else {
+            Series::from(self)
+                .cast(dtype)
+                .and_then(|s| Self::try_from_single_value_series(&s))
+        }
     }
 }
 
