@@ -1,7 +1,7 @@
 use pyo3::prelude::*;
 pub mod series;
 
-pub use daft_schema::python::{field::PyField, schema::PySchema, PyDataType, PyTimeUnit};
+pub use daft_schema::python::{PyDataType, PyTimeUnit, field::PyField, schema::PySchema};
 pub use series::PySeries;
 
 pub fn register_modules(parent: &Bound<PyModule>) -> PyResult<()> {
