@@ -15,12 +15,16 @@ use pyo3::{
 };
 
 use crate::{
-    array::{ops::DaftLogical, pseudo_arrow::PseudoArrowArray, DataArray},
+    array::ops::DaftLogical,
     count_mode::CountMode,
-    datatypes::{DataType, Field, PythonType},
+    datatypes::{DataType, Field},
     lit::Literal,
-    series::{self, IntoSeries, Series},
-    utils::arrow::{cast_array_for_daft_if_needed, cast_array_from_daft_if_needed},
+    prelude::PythonArray,
+    series::{self, IntoSeries, Series, from_lit::combine_lit_types},
+    utils::{
+        arrow::{cast_array_for_daft_if_needed, cast_array_from_daft_if_needed},
+        supertype::try_get_collection_supertype,
+    },
 };
 
 #[pyclass]
@@ -31,15 +35,18 @@ pub struct PySeries {
 impl PySeries {
     pub fn from_pylist_impl(
         name: &str,
-        vec_pyobj: Vec<Py<PyAny>>,
+        vec_pyobj: Vec<Bound<PyAny>>,
         dtype: DataType,
     ) -> PyResult<Self> {
-        let vec_pyobj_arced = vec_pyobj.into_iter().map(Arc::new).collect();
-        let arrow_array: Box<dyn arrow2::array::Array> =
-            Box::new(PseudoArrowArray::from_pyobj_vec(vec_pyobj_arced));
-        let field = Field::new(name, DataType::Python);
-        let data_array = DataArray::<PythonType>::new(field.into(), arrow_array)?;
-        let series = data_array.cast(&dtype)?;
+        let pyobjs_arced = vec_pyobj.into_iter().map(|obj| {
+            if obj.is_none() {
+                None
+            } else {
+                Some(Arc::new(obj.unbind()))
+            }
+        });
+        let arr = PythonArray::from_iter(name, pyobjs_arced);
+        let series = arr.cast(&dtype)?;
 
         Ok(series.into())
     }
@@ -68,14 +75,51 @@ impl PySeries {
         Ok(series.into())
     }
 
-    // This ingests a Python list[object] directly into a Rust PythonArray.
     #[staticmethod]
-    pub fn from_pylist(name: &str, pylist: Bound<PyAny>, dtype: PyDataType) -> PyResult<Self> {
-        let vec_pyobj: Vec<PyObject> = pylist.extract()?;
+    #[pyo3(signature = (list, name=None, dtype=None))]
+    pub fn from_pylist(
+        list: &Bound<PyList>,
+        name: Option<&str>,
+        dtype: Option<PyDataType>,
+    ) -> PyResult<Self> {
+        let dtype = dtype.map(|t| t.dtype);
+        let literals = list
+            .iter()
+            .map(|elem| Literal::from_pyobj(&elem, dtype.as_ref()))
+            .collect::<PyResult<Vec<_>>>()?;
 
-        Self::from_pylist_impl(name, vec_pyobj, dtype.dtype)
+        let (literals, dtype) = if let Some(dtype) = dtype {
+            (literals, dtype)
+        } else {
+            let supertype = try_get_collection_supertype(literals.iter().map(Literal::get_type))
+                .unwrap_or(DataType::Python);
+
+            let literals_with_supertype = literals
+                .into_iter()
+                .zip(list)
+                .map(|(daft_lit, py_lit)| {
+                    if combine_lit_types(&daft_lit.get_type(), &supertype).as_ref()
+                        == Some(&supertype)
+                    {
+                        Ok(daft_lit)
+                    } else {
+                        // if literal doesn't match supertype, redo conversion so that for the python data type
+                        // as well as nested types with python type, we avoid any lossy conversions and just keep
+                        // stuff as Python objects
+                        Literal::from_pyobj(&py_lit, Some(&supertype))
+                    }
+                })
+                .collect::<PyResult<Vec<_>>>()?;
+
+            (literals_with_supertype, supertype)
+        };
+
+        let mut series = Series::from_literals(literals)?.cast(&dtype)?;
+        if let Some(name) = name {
+            series = series.rename(name);
+        }
+        Ok(series.into())
     }
-
     pub fn to_pylist<'a>(&self, py: Python<'a>) -> PyResult<Bound<'a, PyList>> {
         PyList::new(py, self.series.to_literals())
     }
