@@ -3,13 +3,11 @@ use std::sync::Arc;
 use common_error::DaftResult;
 
 use super::{ArrayWrapper, IntoSeries, Series};
-#[cfg(feature = "python")]
-use crate::datatypes::PythonArray;
 use crate::{
     array::{
-        ops::{broadcast::Broadcastable, DaftListAggable, DaftSetAggable, GroupIndices},
-        prelude::*,
         DataArray,
+        ops::{DaftListAggable, DaftSetAggable, GroupIndices, broadcast::Broadcastable},
+        prelude::*,
     },
     datatypes::{DaftArrowBackedType, DataType, FixedSizeBinaryArray, IntervalArray},
     lit::Literal,
@@ -28,15 +26,6 @@ where
     }
 }
 
-#[cfg(feature = "python")]
-impl IntoSeries for PythonArray {
-    fn into_series(self) -> Series {
-        Series {
-            inner: Arc::new(ArrayWrapper(self)),
-        }
-    }
-}
-
 macro_rules! impl_series_like_for_data_array {
     ($da:ident) => {
         impl SeriesLike for ArrayWrapper<$da> {
@@ -198,5 +187,3 @@ impl_series_like_for_data_array!(Utf8Array);
 impl_series_like_for_data_array!(ExtensionArray);
 impl_series_like_for_data_array!(IntervalArray);
 impl_series_like_for_data_array!(Decimal128Array);
-#[cfg(feature = "python")]
-impl_series_like_for_data_array!(PythonArray);
