@@ -1,22 +1,73 @@
 use common_error::{DaftError, DaftResult};
 use common_image::CowImage;
 
-use crate::{array::ops::image::image_array_from_img_buffers, prelude::*};
+use crate::{array::ops::image::image_array_from_img_buffers, datatypes::FileArray, prelude::*};
 
-impl TryFrom<Vec<Literal>> for Series {
-    type Error = DaftError;
+/// Combine literal types that are likely the same but do not equal due to lossy literal casting.
+///
+/// For example, null literals are always null type, and image literals always have a mode.
+pub(crate) fn combine_lit_types(left: &DataType, right: &DataType) -> Option<DataType> {
+    match (left, right) {
+        (dtype, DataType::Null) | (DataType::Null, dtype) => Some(dtype.clone()),
+        (left, right) if left == right => Some(left.clone()),
+        (DataType::List(left_child), DataType::List(right_child)) => {
+            combine_lit_types(left_child, right_child).map(|child| DataType::List(Box::new(child)))
+        }
+        (DataType::Struct(left_fields), DataType::Struct(right_fields)) => {
+            if left_fields.len() != right_fields.len() {
+                return None;
+            }
 
-    fn try_from(values: Vec<Literal>) -> DaftResult<Self> {
-        let dtype = values
-            .iter()
-            .try_fold(DataType::Null, |acc, v| match (acc, v.get_type()) {
-                (dtype, DataType::Null) | (DataType::Null, dtype) => Ok(dtype),
-                (acc, dtype) if acc == dtype => Ok(acc),
-                (acc, dtype) => Err(DaftError::ValueError(format!(
+            let fields = left_fields
+                .iter()
+                .zip(right_fields)
+                .map(|(l, r)| {
+                    if l.name != r.name {
+                        return None;
+                    }
+
+                    let field_dtype = combine_lit_types(&l.dtype, &r.dtype)?;
+                    Some(Field::new(l.name.clone(), field_dtype))
+                })
+                .collect::<Option<_>>()?;
+
+            Some(DataType::Struct(fields))
+        }
+        (
+            DataType::Map {
+                key: left_key,
+                value: left_value,
+            },
+            DataType::Map {
+                key: right_key,
+                value: right_value,
+            },
+        ) => {
+            let key = Box::new(combine_lit_types(left_key, right_key)?);
+            let value = Box::new(combine_lit_types(left_value, right_value)?);
+
+            Some(DataType::Map { key, value })
+        }
+        (DataType::Image(_), DataType::Image(_)) => Some(DataType::Image(None)),
+        _ => None,
+    }
+}
+
+impl Series {
+    /// Converts a vec of literals into a Series.
+    ///
+    /// Literals must all be the same type or null, this function does not do any casting or coercion.
+    /// If that is desired, you should handle it for each literal before converting it into a series.
+    pub fn from_literals(values: Vec<Literal>) -> DaftResult<Self> {
+        let dtype = values.iter().try_fold(DataType::Null, |acc, v| {
+            let dtype = v.get_type();
+            combine_lit_types(&acc, &dtype).ok_or_else(|| {
+                DaftError::ValueError(format!(
                     "All literals must have the same data type or null. Found: {} vs {}",
                     acc, dtype
-                ))),
-            })?;
+                ))
+            })
+        })?;
 
         let field = Field::new("literal", dtype.clone());
 
@@ -109,11 +160,15 @@ impl TryFrom<Vec<Literal>> for Series {
 
                 DurationArray::new(field, physical).into_series()
             }
-            DataType::List(_) => {
+            DataType::List(child_dtype) => {
                 let data = values
                     .iter()
-                    .map(|v| unwrap_inner!(v, List))
-                    .collect::<Vec<_>>();
+                    .map(|v| {
+                        unwrap_inner!(v, List)
+                            .map(|s| s.cast(&child_dtype))
+                            .transpose()
+                    })
+                    .collect::<DaftResult<Vec<_>>>()?;
                 ListArray::try_from(("literal", data.as_slice()))?.into_series()
             }
             DataType::Struct(fields) => {
@@ -129,7 +184,7 @@ impl TryFrom<Vec<Literal>> for Series {
                             })
                             .collect::<Vec<_>>();
 
-                        Ok(Self::try_from(child_values)?.rename(&f.name))
+                        Ok(Self::from_literals(child_values)?.rename(&f.name))
                     })
                     .collect::<DaftResult<_>>()?;
 
@@ -142,15 +197,63 @@ impl TryFrom<Vec<Literal>> for Series {
 
             #[cfg(feature = "python")]
             DataType::Python => {
-                use std::sync::Arc;
+                let iter = values
+                    .into_iter()
+                    .map(|lit| unwrap_inner!(lit, Python).map(|pyobj| pyobj.0));
 
-                let pynone = Arc::new(pyo3::Python::with_gil(|py| py.None()));
+                PythonArray::from_iter("literal", iter).into_series()
+            }
+            DataType::File => {
+                use common_file::FileReference;
 
-                let data = values.into_iter().map(|lit| {
-                    unwrap_inner!(lit, Python).map_or_else(|| pynone.clone(), |pyobj| pyobj.0)
-                });
+                let discriminant = UInt8Array::from_iter(
+                    Field::new("discriminant", DataType::UInt8),
+                    values
+                        .iter()
+                        .map(|lit| unwrap_inner!(lit, File).map(|f| f.get_type() as u8)),
+                )
+                .into_series();
+                let validity = discriminant.validity().cloned();
 
-                PythonArray::from(("literal", data.collect::<Vec<_>>())).into_series()
+                let (files_and_configs, data): (Vec<_>, Vec<Option<Vec<u8>>>) = values
+                    .into_iter()
+                    .map(|lit| {
+                        let Some(f) = unwrap_inner!(lit, File) else {
+                            return ((None, None), None);
+                        };
+
+                        match f {
+                            FileReference::Reference(file, ioconfig) => {
+                                let io_conf = ioconfig.map(|c| {
+                                    bincode::serialize(&c)
+                                        .expect("Failed to serialize IO configuration")
+                                });
+
+                                ((Some(file), io_conf), None)
+                            }
+                            FileReference::Data(items) => {
+                                ((None, None), Some(items.as_ref().clone()))
+                            }
+                        }
+                    })
+                    .unzip();
+                let (files, io_confs): (Vec<Option<String>>, Vec<_>) =
+                    files_and_configs.into_iter().unzip();
+                let sa_field = Field::new("literal", DataType::File.to_physical());
+                let io_configs = BinaryArray::from_iter("io_config", io_confs.into_iter());
+                let urls = Utf8Array::from_iter("url", files.into_iter());
+                let data = BinaryArray::from_iter("data", data.into_iter());
+                let sa = StructArray::new(
+                    sa_field,
+                    vec![
+                        discriminant,
+                        data.into_series(),
+                        urls.into_series(),
+                        io_configs.into_series(),
+                    ],
+                    validity,
+                );
+                FileArray::new(Field::new("literal", DataType::File), sa).into_series()
             }
             DataType::Tensor(_) => {
                 let (data, shapes) = values
@@ -174,7 +277,7 @@ impl TryFrom<Vec<Literal>> for Series {
                     .iter()
                     .map(|v| {
                         match unwrap_inner!(v, Literal::SparseTensor { values, indices, shape, .. } => (values, indices, shape)) {
-                            Some((v, i, s)) => (Some(v), Some(i.as_slice()), Some(s.as_slice())),
+                            Some((v, i, s)) => (Some(v), Some(i), Some(s.as_slice())),
                             None => (None, None, None)
                         }
                     })
@@ -182,7 +285,8 @@ impl TryFrom<Vec<Literal>> for Series {
 
                 let values_array =
                     ListArray::try_from(("values", values.as_slice()))?.into_series();
-                let indices_array = ListArray::from(("indices", indices.as_slice())).into_series();
+                let indices_array =
+                    ListArray::try_from(("indices", indices.as_slice()))?.into_series();
                 let shape_array = ListArray::from(("shape", shapes.as_slice())).into_series();
 
                 let validity = values_array.validity().cloned();
@@ -213,25 +317,31 @@ impl TryFrom<Vec<Literal>> for Series {
 
                 EmbeddingArray::new(field, physical).into_series()
             }
-            DataType::Map { .. } => {
+            DataType::Map {
+                key: key_dtype,
+                value: value_dtype,
+            } => {
                 let data = values
                     .into_iter()
                     .map(|v| {
-                        unwrap_inner!(v, Literal::Map { keys, values } => (keys, values)).map(
-                            |(k, v)| {
-                                StructArray::new(
+                        unwrap_inner!(v, Literal::Map { keys, values } => (keys, values))
+                            .map(|(k, v)| {
+                                Ok(StructArray::new(
                                     field
                                         .to_physical()
                                         .to_exploded_field()
                                         .expect("Expected physical type of map to be list"),
-                                    vec![k.rename("key"), v.rename("value")],
+                                    vec![
+                                        k.cast(&key_dtype)?.rename("key"),
+                                        v.cast(&value_dtype)?.rename("value"),
+                                    ],
                                     None,
                                 )
-                                .into_series()
-                            },
-                        )
+                                .into_series())
+                            })
+                            .transpose()
                     })
-                    .collect::<Vec<_>>();
+                    .collect::<DaftResult<Vec<_>>>()?;
 
                 let physical = ListArray::try_from(("literal", data.as_slice()))?;
 
@@ -259,7 +369,7 @@ impl TryFrom<Vec<Literal>> for Series {
 
 impl From<Literal> for Series {
     fn from(value: Literal) -> Self {
-        Self::try_from(vec![value])
+        Self::from_literals(vec![value])
             .expect("Series::try_from should not fail on single literal value")
     }
 }
@@ -274,113 +384,119 @@ mod test {
     use crate::{datatypes::IntervalValue, prelude::*, series};
 
     #[rstest]
-    #[case(vec![Literal::Null, Literal::Null])]
-    #[case(vec![Literal::Boolean(true), Literal::Boolean(false)])]
-    #[case(vec![Literal::Utf8("foo".to_string()), Literal::Utf8("bar".to_string())])]
-    #[case(vec![Literal::Binary(vec![]), Literal::Binary(vec![1]), Literal::Binary(vec![2, 3])])]
-    #[case(vec![Literal::Int8(0), Literal::Int8(1), Literal::Int8(-2)])]
-    #[case(vec![Literal::UInt8(0), Literal::UInt8(1)])]
-    #[case(vec![Literal::Int16(0), Literal::Int16(1), Literal::Int16(-2)])]
-    #[case(vec![Literal::UInt16(0), Literal::UInt16(1)])]
-    #[case(vec![Literal::Int32(0), Literal::Int32(1), Literal::Int32(-2)])]
-    #[case(vec![Literal::UInt32(0), Literal::UInt32(1)])]
-    #[case(vec![Literal::Int64(0), Literal::Int64(1), Literal::Int64(-2)])]
-    #[case(vec![Literal::UInt64(0), Literal::UInt64(1)])]
-    #[case(vec![
+    #[case::null(vec![Literal::Null, Literal::Null])]
+    #[case::bool(vec![Literal::Boolean(true), Literal::Boolean(false)])]
+    #[case::utf8(vec![Literal::Utf8("foo".to_string()), Literal::Utf8("bar".to_string())])]
+    #[case::binary(vec![Literal::Binary(vec![]), Literal::Binary(vec![1]), Literal::Binary(vec![2, 3])])]
+    #[case::int8(vec![Literal::Int8(0), Literal::Int8(1), Literal::Int8(-2)])]
+    #[case::uint8(vec![Literal::UInt8(0), Literal::UInt8(1)])]
+    #[case::int16(vec![Literal::Int16(0), Literal::Int16(1), Literal::Int16(-2)])]
+    #[case::uint16(vec![Literal::UInt16(0), Literal::UInt16(1)])]
+    #[case::int32(vec![Literal::Int32(0), Literal::Int32(1), Literal::Int32(-2)])]
+    #[case::uint32(vec![Literal::UInt32(0), Literal::UInt32(1)])]
+    #[case::int64(vec![Literal::Int64(0), Literal::Int64(1), Literal::Int64(-2)])]
+    #[case::uint64(vec![Literal::UInt64(0), Literal::UInt64(1)])]
+    #[case::ts_us(vec![
         Literal::Timestamp(0, TimeUnit::Microseconds, None),
         Literal::Timestamp(1, TimeUnit::Microseconds, None),
         Literal::Timestamp(-2, TimeUnit::Microseconds, None)
     ])]
-    #[case(vec![
+    #[case::ts_s_tz(vec![
         Literal::Timestamp(0, TimeUnit::Seconds, Some("America/Los_Angeles".to_string())),
         Literal::Timestamp(1, TimeUnit::Seconds, Some("America/Los_Angeles".to_string())),
         Literal::Timestamp(-2, TimeUnit::Seconds, Some("America/Los_Angeles".to_string()))
     ])]
-    #[case(vec![Literal::Date(0), Literal::Date(1), Literal::Date(-2)])]
-    #[case(vec![
+    #[case::date(vec![Literal::Date(0), Literal::Date(1), Literal::Date(-2)])]
+    #[case::time_us(vec![
         Literal::Time(0, TimeUnit::Microseconds),
         Literal::Time(1, TimeUnit::Microseconds),
         Literal::Time(-2, TimeUnit::Microseconds)
     ])]
-    #[case(vec![
+    #[case::time_s(vec![
         Literal::Time(0, TimeUnit::Seconds),
         Literal::Time(1, TimeUnit::Seconds),
         Literal::Time(-2, TimeUnit::Seconds)
     ])]
-    #[case(vec![
+    #[case::duration_us(vec![
         Literal::Duration(0, TimeUnit::Microseconds),
         Literal::Duration(1, TimeUnit::Microseconds),
         Literal::Duration(-2, TimeUnit::Microseconds)
     ])]
-    #[case(vec![
+    #[case::duration_s(vec![
         Literal::Duration(0, TimeUnit::Seconds),
         Literal::Duration(1, TimeUnit::Seconds),
         Literal::Duration(-2, TimeUnit::Seconds)
     ])]
-    #[case(vec![Literal::Interval(IntervalValue::new(0, 0, 0)), Literal::Interval(IntervalValue::new(1, -2, 3))])]
-    #[case(vec![Literal::Float32(0.0), Literal::Float32(0.5), Literal::Float32(-1.5)])]
-    #[case(vec![Literal::Float64(0.0), Literal::Float64(0.5), Literal::Float64(-1.5)])]
-    #[case(vec![Literal::Decimal(0, 0, 1), Literal::Decimal(1, 0, 1), Literal::Decimal(-2, 0, 1)])]
-    #[case(vec![Literal::Decimal(0, 5, 10), Literal::Decimal(1, 5, 10), Literal::Decimal(-2, 5, 10)])]
-    #[case(vec![Literal::List(Series::empty("literal", &DataType::Int32)), Literal::List(series![1, 2, 3])])]
-    #[case(vec![
-        Literal::Struct(indexmap!{Field::new("a", DataType::Boolean) => Literal::Boolean(true)}),
-        Literal::Struct(indexmap!{Field::new("a", DataType::Boolean) => Literal::Boolean(false)}),
+    #[case::interval(vec![Literal::Interval(IntervalValue::new(0, 0, 0)), Literal::Interval(IntervalValue::new(1, -2, 3))])]
+    #[case::float32(vec![Literal::Float32(0.0), Literal::Float32(0.5), Literal::Float32(-1.5)])]
+    #[case::float64(vec![Literal::Float64(0.0), Literal::Float64(0.5), Literal::Float64(-1.5)])]
+    #[case::decimal_1(vec![Literal::Decimal(0, 0, 1), Literal::Decimal(1, 0, 1), Literal::Decimal(-2, 0, 1)])]
+    #[case::decimal_2(vec![Literal::Decimal(0, 5, 10), Literal::Decimal(1, 5, 10), Literal::Decimal(-2, 5, 10)])]
+    #[case::list(vec![
+        Literal::List(Series::empty("literal", &DataType::Int32)),
+        Literal::List(series![1, 2, 3]),
+        Literal::List(Series::empty("literal", &DataType::Null))
     ])]
-    #[case(vec![
+    #[case::struct_(vec![
+        Literal::Struct(indexmap!{"a".to_string() => Literal::Boolean(true)}),
+        Literal::Struct(indexmap!{"a".to_string() => Literal::Boolean(false)}),
+        Literal::Struct(indexmap!{"a".to_string() => Literal::Null}),
+    ])]
+    #[case::tensor(vec![
         Literal::Tensor { data: series![0, 0], shape: vec![1, 2] },
         Literal::Tensor { data: series![1, 2, 3, 4, 5, 6], shape: vec![1, 2, 3] },
     ])]
-    #[case(vec![
+    #[case::sparse_tensor_no_offset(vec![
         Literal::SparseTensor {
             values: series![0],
-            indices: vec![0],
+            indices: series![0].cast(&DataType::UInt64).unwrap(),
             shape: vec![1, 2],
             indices_offset: false
         },
         Literal::SparseTensor {
             values: series![1, 2, 3, 4],
-            indices: vec![1, 2, 3, 4],
+            indices: series![1, 2, 3, 4].cast(&DataType::UInt64).unwrap(),
             shape: vec![1, 2, 3],
             indices_offset: false
         },
     ])]
-    #[case(vec![
+    #[case::sparse_tensor_with_offset(vec![
         Literal::SparseTensor {
             values: series![0],
-            indices: vec![0],
+            indices: series![0].cast(&DataType::UInt64).unwrap(),
             shape: vec![1, 2],
             indices_offset: true
         },
         Literal::SparseTensor {
             values: series![1, 2, 3, 4],
-            indices: vec![1, 1, 1, 1],
+            indices: series![1, 1, 1, 1].cast(&DataType::UInt64).unwrap(),
             shape: vec![1, 2, 3],
             indices_offset: true
         },
     ])]
-    #[case(vec![
+    #[case::embedding(vec![
         Literal::Embedding(series![0, 0, 0, 0]),
         Literal::Embedding(series![1, 2, 3, 4]),
     ])]
-    #[case(vec![
+    #[case::map(vec![
         Literal::Map { keys: series!["a", "b", "c"], values: series![1, 2, 3] },
         Literal::Map { keys: series!["d", "e"], values: series![4, 5] },
         Literal::Map { keys: Series::empty("literal", &DataType::Utf8), values: Series::empty("literal", &DataType::Int32) },
+        Literal::Map { keys: Series::empty("literal", &DataType::Null), values: Series::empty("literal", &DataType::Null) },
     ])]
-    #[case(vec![
+    #[case::image_gray(vec![
         Literal::Image(Image(GrayImage::new(1, 1).into())),
         Literal::Image(Image(GrayImage::new(100, 100).into())),
         Literal::Image(Image(GrayImage::from_raw(10, 10, vec![10; 100]).unwrap().into())),
     ])]
-    #[case(vec![
+    #[case::image_rgba(vec![
         Literal::Image(Image(RgbaImage::new(1, 1).into())),
         Literal::Image(Image(RgbaImage::new(100, 100).into())),
         Literal::Image(Image(RgbaImage::from_raw(10, 10, vec![10; 400]).unwrap().into())),
     ])]
     fn test_literal_series_roundtrip_basics(#[case] literals: Vec<Literal>) {
         let expected = [vec![Literal::Null], literals, vec![Literal::Null]].concat();
-        let series = Series::try_from(expected.clone()).unwrap();
+        let series = Series::from_literals(expected.clone()).unwrap();
         let actual = series.to_literals().collect::<Vec<_>>();
 
         assert_eq!(expected, actual)
@@ -389,7 +505,7 @@ mod test {
     #[test]
     fn test_literals_to_series_mismatched() {
         let values = vec![Literal::UInt64(1), Literal::Utf8("test".to_string())];
-        let actual = Series::try_from(values);
+        let actual = Series::from_literals(values);
         assert!(actual.is_err());
     }
 }
