@@ -1,6 +1,6 @@
 mod array_impl;
 mod from;
-mod from_lit;
+pub mod from_lit;
 mod ops;
 mod serdes;
 mod series_like;
@@ -262,7 +262,7 @@ macro_rules! series {
             // put into a vec first for compile-time type consistency checking
             let elements = vec![$($element),+];
             let elements_lit = elements.into_iter().map(Literal::from).collect::<Vec<_>>();
-            Series::try_from(elements_lit).unwrap()
+            Series::from_literals(elements_lit).unwrap()
         }
     };
 }
