@@ -1,7 +1,10 @@
 import type { NextConfig } from "next";
 
 const nextConfig: NextConfig = {
-    output: "export",
+  output: "export",
+  experimental: {
+    reactCompiler: true,
+  },
 };
 
 export default nextConfig;
