@@ -1,6 +1,5 @@
 use std::sync::Arc;
 
-use common_display::{DisplayLevel, tree::TreeDisplay};
 use common_error::DaftResult;
 use daft_local_plan::LocalPhysicalPlan;
 use daft_logical_plan::{partitioning::RepartitionSpec, stats::StatsState};
@@ -9,13 +8,13 @@ use daft_schema::schema::SchemaRef;
 use crate::{
     pipeline_node::{
         DistributedPipelineNode, NodeID, NodeName, PipelineNodeConfig, PipelineNodeContext,
-        SubmittableTaskStream, make_in_memory_task_from_materialized_outputs,
+        PipelineNodeImpl, SubmittableTaskStream, make_in_memory_task_from_materialized_outputs,
     },
+    plan::{PlanConfig, PlanExecutionContext, TaskIDCounter},
     scheduling::{
         scheduler::{SchedulerHandle, SubmittableTask},
         task::{SwordfishTask, TaskContext},
     },
-    stage::{StageConfig, StageExecutionContext, TaskIDCounter},
     utils::{
         channel::{Sender, create_channel},
         transpose::transpose_materialized_outputs_from_stream,
@@ -27,7 +26,7 @@ pub(crate) struct RepartitionNode {
     context: PipelineNodeContext,
     repartition_spec: RepartitionSpec,
     num_partitions: usize,
-    child: Arc<dyn DistributedPipelineNode>,
+    child: DistributedPipelineNode,
 }
 
 impl RepartitionNode {
@@ -36,14 +35,14 @@ impl RepartitionNode {
     pub fn new(
         node_id: NodeID,
         logical_node_id: Option<NodeID>,
-        stage_config: &StageConfig,
+        plan_config: &PlanConfig,
         repartition_spec: RepartitionSpec,
         num_partitions: usize,
         schema: SchemaRef,
-        child: Arc<dyn DistributedPipelineNode>,
+        child: DistributedPipelineNode,
     ) -> Self {
         let context = PipelineNodeContext::new(
-            stage_config,
+            plan_config.plan_id,
             node_id,
             Self::NODE_NAME,
             vec![child.node_id()],
@@ -52,7 +51,7 @@ impl RepartitionNode {
         );
         let config = PipelineNodeConfig::new(
             schema,
-            stage_config.config.clone(),
+            plan_config.config.clone(),
             repartition_spec
                 .to_clustering_spec(child.config().clustering_spec.num_partitions())
                 .into(),
@@ -67,14 +66,8 @@ impl RepartitionNode {
         }
     }
 
-    pub fn arced(self) -> Arc<dyn DistributedPipelineNode> {
-        Arc::new(self)
-    }
-
-    fn multiline_display(&self) -> Vec<String> {
-        let mut res = vec![format!("Repartition: {}", self.repartition_spec.var_name())];
-        res.extend(self.repartition_spec.multiline_display());
-        res
+    pub fn into_node(self) -> DistributedPipelineNode {
+        DistributedPipelineNode::new(Arc::new(self))
     }
 
     // Async execution to get all partitions out
@@ -98,7 +91,8 @@ impl RepartitionNode {
             let task = make_in_memory_task_from_materialized_outputs(
                 TaskContext::from((&self_clone.context, task_id_counter.next())),
                 partition_group,
-                &(self_clone as Arc<dyn DistributedPipelineNode>),
+                self_clone.config.schema.clone(),
+                &(self_clone as Arc<dyn PipelineNodeImpl>),
                 None,
             )?;
 
@@ -109,32 +103,7 @@ impl RepartitionNode {
     }
 }
 
-impl TreeDisplay for RepartitionNode {
-    fn display_as(&self, level: DisplayLevel) -> String {
-        use std::fmt::Write;
-        let mut display = String::new();
-        match level {
-            DisplayLevel::Compact => {
-                writeln!(display, "{}", self.context.node_name).unwrap();
-            }
-            _ => {
-                let multiline_display = self.multiline_display().join("\n");
-                writeln!(display, "{}", multiline_display).unwrap();
-            }
-        }
-        display
-    }
-
-    fn get_children(&self) -> Vec<&dyn TreeDisplay> {
-        vec![self.child.as_tree_display()]
-    }
-
-    fn get_name(&self) -> String {
-        self.context.node_name.to_string()
-    }
-}
-
-impl DistributedPipelineNode for RepartitionNode {
+impl PipelineNodeImpl for RepartitionNode {
     fn context(&self) -> &PipelineNodeContext {
         &self.context
     }
@@ -143,20 +112,26 @@ impl DistributedPipelineNode for RepartitionNode {
         &self.config
     }
 
-    fn children(&self) -> Vec<Arc<dyn DistributedPipelineNode>> {
+    fn children(&self) -> Vec<DistributedPipelineNode> {
         vec![self.child.clone()]
     }
 
+    fn multiline_display(&self, _verbose: bool) -> Vec<String> {
+        let mut res = vec![format!("Repartition: {}", self.repartition_spec.var_name())];
+        res.extend(self.repartition_spec.multiline_display());
+        res
+    }
+
     fn produce_tasks(
         self: Arc<Self>,
-        stage_context: &mut StageExecutionContext,
+        plan_context: &mut PlanExecutionContext,
     ) -> SubmittableTaskStream {
-        let input_node = self.child.clone().produce_tasks(stage_context);
+        let input_node = self.child.clone().produce_tasks(plan_context);
         let self_arc = self.clone();
 
         // First pipeline the local repartition op
         let self_clone = self.clone();
-        let local_repartition_node = input_node.pipeline_instruction(self.clone(), move |input| {
+        let local_repartition_node = input_node.pipeline_instruction(self, move |input| {
             LocalPhysicalPlan::repartition(
                 input,
                 self_clone.repartition_spec.clone(),
@@ -168,8 +143,8 @@ impl DistributedPipelineNode for RepartitionNode {
 
         let (result_tx, result_rx) = create_channel(1);
 
-        let task_id_counter = stage_context.task_id_counter();
-        let scheduler_handle = stage_context.scheduler_handle();
+        let task_id_counter = plan_context.task_id_counter();
+        let scheduler_handle = plan_context.scheduler_handle();
 
         let map_reduce_execution = async move {
             self_arc
@@ -182,11 +157,7 @@ impl DistributedPipelineNode for RepartitionNode {
                 .await
         };
 
-        stage_context.spawn(map_reduce_execution);
+        plan_context.spawn(map_reduce_execution);
         SubmittableTaskStream::from(result_rx)
     }
-
-    fn as_tree_display(&self) -> &dyn TreeDisplay {
-        self
-    }
 }
