@@ -1,5 +1,3 @@
-use std::sync::Arc;
-
 use common_error::DaftResult;
 use daft_logical_plan::partitioning::RepartitionSpec;
 use daft_schema::schema::SchemaRef;
@@ -18,8 +16,8 @@ impl LogicalPlanToPipelineNodeTranslator {
         logical_node_id: Option<NodeID>,
         repartition_spec: RepartitionSpec,
         schema: SchemaRef,
-        child: Arc<dyn DistributedPipelineNode>,
-    ) -> DaftResult<Arc<dyn DistributedPipelineNode>> {
+        child: DistributedPipelineNode,
+    ) -> DaftResult<DistributedPipelineNode> {
         let num_partitions = match &repartition_spec {
             RepartitionSpec::Hash(config) => config
                 .num_partitions
@@ -40,46 +38,46 @@ impl LogicalPlanToPipelineNodeTranslator {
             let merge_node = PreShuffleMergeNode::new(
                 self.get_next_pipeline_node_id(),
                 logical_node_id,
-                &self.stage_config,
-                self.stage_config.config.pre_shuffle_merge_threshold,
+                &self.plan_config,
+                self.plan_config.config.pre_shuffle_merge_threshold,
                 schema.clone(),
                 child,
             )
-            .arced();
+            .into_node();
 
             Ok(RepartitionNode::new(
                 self.get_next_pipeline_node_id(),
                 logical_node_id,
-                &self.stage_config,
+                &self.plan_config,
                 repartition_spec,
                 num_partitions,
                 schema,
                 merge_node,
             )
-            .arced())
+            .into_node())
         } else {
             Ok(RepartitionNode::new(
                 self.get_next_pipeline_node_id(),
                 logical_node_id,
-                &self.stage_config,
+                &self.plan_config,
                 repartition_spec,
                 num_partitions,
                 schema,
                 child,
             )
-            .arced())
+            .into_node())
         }
     }
 
     /// Determine if we should use pre-shuffle merge strategy
     fn should_use_pre_shuffle_merge(
         &self,
-        child: &Arc<dyn DistributedPipelineNode>,
+        child: &DistributedPipelineNode,
         target_num_partitions: usize,
     ) -> DaftResult<bool> {
         let input_num_partitions = child.config().clustering_spec.num_partitions();
 
-        match self.stage_config.config.shuffle_algorithm.as_str() {
+        match self.plan_config.config.shuffle_algorithm.as_str() {
             "pre_shuffle_merge" => Ok(true),
             "map_reduce" => Ok(false),
             "flight_shuffle" => Err(common_error::DaftError::ValueError(
@@ -98,8 +96,8 @@ impl LogicalPlanToPipelineNodeTranslator {
     pub fn gen_gather_node(
         &mut self,
         logical_node_id: Option<NodeID>,
-        input_node: Arc<dyn DistributedPipelineNode>,
-    ) -> Arc<dyn DistributedPipelineNode> {
+        input_node: DistributedPipelineNode,
+    ) -> DistributedPipelineNode {
         if input_node.config().clustering_spec.num_partitions() == 1 {
             return input_node;
         }
@@ -107,10 +105,10 @@ impl LogicalPlanToPipelineNodeTranslator {
         GatherNode::new(
             self.get_next_pipeline_node_id(),
             logical_node_id,
-            &self.stage_config,
+            &self.plan_config,
             input_node.config().schema.clone(),
             input_node,
         )
-        .arced()
+        .into_node()
     }
 }
