@@ -3,6 +3,7 @@ pub mod ray;
 use std::{collections::HashMap, sync::Arc};
 
 use common_daft_config::PyDaftExecutionConfig;
+use common_display::DisplayLevel;
 use common_partitioning::Partition;
 use common_py_serde::impl_bincode_py_state_serialization;
 use daft_logical_plan::PyLogicalPlanBuilder;
@@ -14,7 +15,11 @@ use serde::{Deserialize, Serialize};
 use tokio::sync::Mutex;
 
 use crate::{
-    plan::{DistributedPhysicalPlan, PlanResultStream, PlanRunner},
+    pipeline_node::{
+        logical_plan_to_pipeline_node, viz_distributed_pipeline_ascii,
+        viz_distributed_pipeline_mermaid,
+    },
+    plan::{DistributedPhysicalPlan, PlanConfig, PlanResultStream, PlanRunner},
     python::ray::RayTaskResult,
     statistics::{HttpSubscriber, StatisticsManager, StatisticsSubscriber},
 };
@@ -81,16 +86,38 @@ impl PyDistributedPhysicalPlan {
 
     /// Visualize the distributed pipeline as ASCII text
     fn repr_ascii(&self, simple: bool) -> PyResult<String> {
-        // Create a pipeline node from the stage plan
-        let stage_plan = self.plan.stage_plan();
-        Ok(stage_plan.repr_ascii(self.plan.id().into(), simple)?)
+        // Create pipeline nodes from the logical plan
+        let plan_config = PlanConfig::new(self.plan.id(), self.plan.execution_config().clone());
+        let pipeline_node = logical_plan_to_pipeline_node(
+            plan_config,
+            self.plan.logical_plan().clone(),
+            Default::default(),
+        )?;
+
+        Ok(viz_distributed_pipeline_ascii(&pipeline_node, simple))
     }
 
     /// Visualize the distributed pipeline as Mermaid markdown
     fn repr_mermaid(&self, simple: bool, bottom_up: bool) -> PyResult<String> {
         // Create a pipeline node from the stage plan
-        let stage_plan = self.plan.stage_plan();
-        Ok(stage_plan.repr_mermaid(self.plan.id().into(), simple, bottom_up)?)
+        let plan_config = PlanConfig::new(self.plan.id(), self.plan.execution_config().clone());
+        let pipeline_node = logical_plan_to_pipeline_node(
+            plan_config,
+            self.plan.logical_plan().clone(),
+            Default::default(),
+        )?;
+
+        let display_level = if simple {
+            DisplayLevel::Compact
+        } else {
+            DisplayLevel::Default
+        };
+        Ok(viz_distributed_pipeline_mermaid(
+            &pipeline_node,
+            display_level,
+            bottom_up,
+            None,
+        ))
     }
 }
 impl_bincode_py_state_serialization!(PyDistributedPhysicalPlan);
