@@ -1,6 +1,3 @@
-#[cfg(feature = "python")]
-use std::time::Duration;
-
 use common_error::{DaftError, DaftResult};
 use daft_core::prelude::*;
 #[cfg(feature = "python")]
@@ -65,9 +62,7 @@ fn run_udf(
 
 impl LegacyPythonUDF {
     #[cfg(feature = "python")]
-    pub fn call_udf(&self, inputs: &[Series]) -> DaftResult<(Series, Duration)> {
-        use std::time::Instant;
-
+    pub fn call_udf(&self, inputs: &[Series]) -> DaftResult<Series> {
         use pyo3::Python;
 
         use crate::functions::python::{MaybeInitializedUDF, py_udf_initialize};
@@ -80,10 +75,7 @@ impl LegacyPythonUDF {
             )));
         }
 
-        let start_time = Instant::now();
         Python::with_gil(|py| {
-            let gil_contention_time = start_time.elapsed();
-
             let func = match &self.func {
                 MaybeInitializedUDF::Initialized(func) => func.clone().unwrap().clone_ref(py),
                 MaybeInitializedUDF::Uninitialized { inner, init_args } => {
@@ -100,9 +92,13 @@ impl LegacyPythonUDF {
                 &self.return_dtype,
                 self.batch_size,
             )
-            .map(|result| (result, gil_contention_time))
         })
     }
+
+    #[cfg(not(feature = "python"))]
+    pub fn call_udf(&self, inputs: &[Series]) -> DaftResult<Series> {
+        panic!("Cannot evaluate a PythonUDF without compiling for Python");
+    }
 }
 
 impl FunctionEvaluator for LegacyPythonUDF {
@@ -127,13 +123,6 @@ impl FunctionEvaluator for LegacyPythonUDF {
     }
 
     fn evaluate(&self, inputs: &[Series], _: &FunctionExpr) -> DaftResult<Series> {
-        #[cfg(not(feature = "python"))]
-        {
-            panic!("Cannot evaluate a PythonUDF without compiling for Python");
-        }
-        #[cfg(feature = "python")]
-        {
-            Ok(self.call_udf(inputs)?.0)
-        }
+        self.call_udf(inputs)
     }
 }
