@@ -1,5 +1,3 @@
-#![feature(let_chains)]
-#![feature(io_error_more)]
 #![feature(if_let_guard)]
 mod azure_blob;
 mod counting_reader;
@@ -35,13 +33,13 @@ use std::{borrow::Cow, collections::HashMap, hash::Hash, sync::Arc};
 
 use common_error::{DaftError, DaftResult};
 pub use common_io_config::{AzureConfig, GCSConfig, HTTPConfig, IOConfig, S3Config};
-use futures::{stream::BoxStream, FutureExt};
+use futures::{FutureExt, stream::BoxStream};
 use object_io::StreamingRetryParams;
-pub use object_io::{FileMetadata, GetResult, ObjectSource};
+pub use object_io::{FileMetadata, FileType, GetResult, ObjectSource};
 #[cfg(feature = "python")]
 pub use python::register_modules;
-pub use s3_like::{s3_config_from_env, S3LikeSource, S3MultipartWriter, S3PartBuffer};
-use snafu::{prelude::*, Snafu};
+pub use s3_like::{S3LikeSource, S3MultipartWriter, S3PartBuffer, s3_config_from_env};
+use snafu::{Snafu, prelude::*};
 pub use stats::{IOStatsContext, IOStatsRef};
 use url::ParseError;
 
@@ -126,7 +124,9 @@ pub enum Error {
     #[snafu(display("Failed to load Credentials for store: {store}\nDetails:\n{source:?}"))]
     UnableToCreateClient { store: SourceType, source: DynError },
 
-    #[snafu(display("Unauthorized to access store: {store} for file: {path}\nYou may need to set valid Credentials\n{source}"))]
+    #[snafu(display(
+        "Unauthorized to access store: {store} for file: {path}\nYou may need to set valid Credentials\n{source}"
+    ))]
     Unauthorized {
         store: SourceType,
         path: String,
@@ -337,7 +337,6 @@ impl IOClient {
 
     pub async fn single_url_download(
         &self,
-        index: usize,
         input: Option<String>,
         raise_error_on_failure: bool,
         io_stats: Option<IOStatsRef>,
@@ -359,10 +358,6 @@ impl IOClient {
                 if raise_error_on_failure {
                     Err(err)
                 } else {
-                    log::warn!(
-                        "Error occurred during url_download at index: {index} {} (falling back to Null)",
-                        err
-                    );
                     Ok(None)
                 }
             }
@@ -372,7 +367,6 @@ impl IOClient {
 
     pub async fn single_url_upload(
         &self,
-        index: usize,
         dest: String,
         data: Option<bytes::Bytes>,
         raise_error_on_failure: bool,
@@ -391,10 +385,6 @@ impl IOClient {
                 if raise_error_on_failure {
                     Err(err)
                 } else {
-                    log::warn!(
-                        "Error occurred during file upload at index: {index} {} (falling back to Null)",
-                        err
-                    );
                     Ok(None)
                 }
             }
