@@ -1,7 +1,7 @@
 use super::ProtoResult;
 use crate::{
     non_null, not_implemented_err,
-    proto::{from_proto, from_proto_vec, to_proto_vec, ToFromProto},
+    proto::{ToFromProto, from_proto, from_proto_vec, to_proto_vec},
 };
 
 /// Export daft_ir types under an `ir` namespace to concisely disambiguate domains.
@@ -36,13 +36,33 @@ pub fn from_proto_function(message: proto::ScalarFn) -> ProtoResult<ir::Expr> {
             proto::scalar_fn::py_fn::Variant::RowWise(row_wise_fn) => {
                 let func = ir::functions::RowWisePyFn {
                     function_name: row_wise_fn.name.into(),
-                    inner: from_proto(row_wise_fn.inner)?,
+                    cls: from_proto(row_wise_fn.cls)?,
+                    method: from_proto(row_wise_fn.method)?,
+                    is_async: row_wise_fn.is_async,
                     return_dtype: from_proto(row_wise_fn.return_dtype)?,
                     original_args: from_proto(row_wise_fn.original_args)?,
                     args: args.into_inner(),
+                    gpus: row_wise_fn.gpus as usize,
+                    use_process: row_wise_fn.use_process,
+                    max_concurrency: row_wise_fn.max_concurrency.map(|c| c as usize),
                 };
                 ir::rex::from_py_rowwise_func(func)
             }
+            proto::scalar_fn::py_fn::Variant::Batch(batch_fn) => {
+                let func = ir::functions::BatchPyFn {
+                    function_name: batch_fn.name.into(),
+                    cls: from_proto(batch_fn.cls)?,
+                    method: from_proto(batch_fn.method)?,
+                    return_dtype: from_proto(batch_fn.return_dtype)?,
+                    gpus: batch_fn.gpus as usize,
+                    use_process: batch_fn.use_process,
+                    max_concurrency: batch_fn.max_concurrency.map(|c| c as usize),
+                    batch_size: batch_fn.batch_size.map(|b| b as usize),
+                    original_args: from_proto(batch_fn.original_args)?,
+                    args: args.into_inner(),
+                };
+                ir::rex::from_py_batch_func(func)
+            }
         },
         proto::scalar_fn::Descriptor::Builtin(builtin_fn) => {
             // handle special form, otherwise it's a ScalarFn
@@ -98,8 +118,41 @@ pub fn scalar_fn_to_proto(sf: &ir::functions::scalar::ScalarFn) -> ProtoResult<p
                         proto::scalar_fn::py_fn::RowWiseFn {
                             name: row_wise_fn.function_name.to_string(),
                             return_dtype: Some(row_wise_fn.return_dtype.to_proto()?),
-                            inner: Some(row_wise_fn.inner.to_proto()?),
+                            cls: Some(row_wise_fn.cls.to_proto()?),
+                            method: Some(row_wise_fn.method.to_proto()?),
+                            is_async: row_wise_fn.is_async,
                             original_args: Some(row_wise_fn.original_args.to_proto()?),
+                            gpus: row_wise_fn.gpus as u64,
+                            use_process: row_wise_fn.use_process,
+                            max_concurrency: row_wise_fn.max_concurrency.map(|c| c as u64),
+                        },
+                    )),
+                })),
+                args: Some(args),
+            })
+        }
+        ir::functions::scalar::ScalarFn::Python(ir::functions::PyScalarFn::Batch(batch_fn)) => {
+            let function_args = batch_fn
+                .args
+                .iter()
+                .map(|arg| ir::functions::FunctionArg::Unnamed(arg.clone()))
+                .collect();
+            let function_args = ir::functions::FunctionArgs::new_unchecked(function_args);
+            let args = function_args.to_proto()?;
+
+            Ok(proto::ScalarFn {
+                descriptor: Some(proto::scalar_fn::Descriptor::Py(proto::scalar_fn::PyFn {
+                    variant: Some(proto::scalar_fn::py_fn::Variant::Batch(
+                        proto::scalar_fn::py_fn::BatchFn {
+                            name: batch_fn.function_name.to_string(),
+                            return_dtype: Some(batch_fn.return_dtype.to_proto()?),
+                            cls: Some(batch_fn.cls.to_proto()?),
+                            method: Some(batch_fn.method.to_proto()?),
+                            original_args: Some(batch_fn.original_args.to_proto()?),
+                            gpus: batch_fn.gpus as u64,
+                            use_process: batch_fn.use_process,
+                            max_concurrency: batch_fn.max_concurrency.map(|c| c as u64),
+                            batch_size: batch_fn.batch_size.map(|b| b as u64),
                         },
                     )),
                 })),
