@@ -1,10 +1,7 @@
-#![feature(let_chains)]
-
 mod buffer;
 mod channel;
 mod dispatcher;
 mod intermediate_ops;
-mod ops;
 mod pipeline;
 mod resource_manager;
 mod run;
@@ -28,7 +25,7 @@ use console::style;
 use resource_manager::MemoryManager;
 pub use run::{ExecutionEngineResult, NativeExecutor};
 use runtime_stats::{RuntimeStats, RuntimeStatsManagerHandle, TimedFuture};
-use snafu::{futures::TryFutureExt, ResultExt, Snafu};
+use snafu::{ResultExt, Snafu, futures::TryFutureExt};
 use tracing::Instrument;
 
 /// The `OperatorOutput` enum represents the output of an operator.
