@@ -3,21 +3,21 @@ use std::sync::Arc;
 use capitalize::Capitalize;
 use common_display::tree::TreeDisplay;
 use common_error::DaftResult;
+use common_metrics::ops::{NodeCategory, NodeInfo, NodeType};
 use common_runtime::{get_compute_pool_num_threads, get_compute_runtime};
 use daft_logical_plan::stats::StatsState;
 use daft_micropartition::MicroPartition;
 use tracing::{info_span, instrument};
 
 use crate::{
-    channel::{create_channel, Receiver},
+    ExecutionRuntimeContext, ExecutionTaskSpawner, OperatorOutput, TaskSet,
+    channel::{Receiver, create_channel},
     dispatcher::{DispatchSpawner, UnorderedDispatcher},
-    ops::{NodeCategory, NodeInfo, NodeType},
     pipeline::{MorselSizeRequirement, NodeName, PipelineNode, RuntimeContext},
     resource_manager::MemoryManager,
     runtime_stats::{
         CountingSender, DefaultRuntimeStats, InitializingCountingReceiver, RuntimeStats,
     },
-    ExecutionRuntimeContext, ExecutionTaskSpawner, OperatorOutput, TaskSet,
 };
 
 pub enum BlockingSinkStatus<Op: BlockingSink> {
