@@ -1,11 +1,15 @@
 use std::sync::Arc;
 
 use common_error::DaftResult;
-use daft_core::{prelude::SchemaRef, series::IntoSeries};
+use common_metrics::ops::NodeType;
+use daft_core::{
+    prelude::{SchemaRef, UInt64Array},
+    series::IntoSeries,
+};
 use daft_dsl::expr::bound_expr::BoundExpr;
 use daft_logical_plan::JoinType;
 use daft_micropartition::MicroPartition;
-use daft_recordbatch::{GrowableRecordBatch, ProbeState, Probeable, RecordBatch};
+use daft_recordbatch::ProbeState;
 use futures::{StreamExt, stream};
 use itertools::Itertools;
 use tracing::{Span, info_span, instrument};
@@ -15,7 +19,7 @@ use super::{
     outer_hash_join_probe::IndexBitmapBuilder,
 };
 use crate::{
-    ExecutionTaskSpawner, ops::NodeType, pipeline::NodeName, state_bridge::BroadcastStateBridgeRef,
+    ExecutionTaskSpawner, pipeline::NodeName, state_bridge::BroadcastStateBridgeRef,
     streaming_sink::base::StreamingSinkFinalizeResult,
 };
 
@@ -32,7 +36,7 @@ impl AntiSemiProbeState {
         if let Self::Building(bridge) = self {
             let probe_state = bridge.get_state().await;
             let builder = if needs_bitmap {
-                Some(IndexBitmapBuilder::new(probe_state.get_tables()))
+                Some(IndexBitmapBuilder::new(probe_state.get_record_batch()))
             } else {
                 None
             };
@@ -58,8 +62,6 @@ pub(crate) struct AntiSemiProbeSink {
 }
 
 impl AntiSemiProbeSink {
-    const DEFAULT_GROWABLE_SIZE: usize = 20;
-
     pub fn new(
         probe_on: Vec<BoundExpr>,
         join_type: &JoinType,
@@ -82,43 +84,38 @@ impl AntiSemiProbeSink {
     // on the right side and are streaming the left side.
     fn probe_anti_semi(
         probe_on: &[BoundExpr],
-        probe_set: &Arc<dyn Probeable>,
+        probe_state: &Arc<ProbeState>,
         input: &Arc<MicroPartition>,
         is_semi: bool,
     ) -> DaftResult<Arc<MicroPartition>> {
-        let _growables = info_span!("AntiSemiOperator::build_growables").entered();
-
         let input_tables = input.get_tables()?;
+        let mut input_idxs = vec![vec![]; input_tables.len()];
+        for (probe_side_table_idx, table) in input_tables.iter().enumerate() {
+            let join_keys = table.eval_expression_list(probe_on)?;
+            let iter = probe_state.probe_exists(&join_keys)?;
 
-        let mut probe_side_growable = GrowableRecordBatch::new(
-            &input_tables.iter().collect::<Vec<_>>(),
-            false,
-            Self::DEFAULT_GROWABLE_SIZE,
-        )?;
-
-        drop(_growables);
-        {
-            let _loop = info_span!("AntiSemiOperator::eval_and_probe").entered();
-            for (probe_side_table_idx, table) in input_tables.iter().enumerate() {
-                let join_keys = table.eval_expression_list(probe_on)?;
-                let iter = probe_set.probe_exists(&join_keys)?;
-
-                for (probe_row_idx, matched) in iter.enumerate() {
-                    // 1. If this is a semi join, we keep the row if it matches.
-                    // 2. If this is an anti join, we keep the row if it doesn't match.
-                    match (is_semi, matched) {
-                        (true, true) | (false, false) => {
-                            probe_side_growable.extend(probe_side_table_idx, probe_row_idx, 1);
-                        }
-                        _ => {}
+            for (probe_row_idx, matched) in iter.enumerate() {
+                // 1. If this is a semi join, we keep the row if it matches.
+                // 2. If this is an anti join, we keep the row if it doesn't match.
+                match (is_semi, matched) {
+                    (true, true) | (false, false) => {
+                        input_idxs[probe_side_table_idx].push(probe_row_idx as u64);
                     }
+                    _ => {}
                 }
             }
         }
-        let probe_side_table = probe_side_growable.build()?;
+        let probe_side_tables = input_idxs
+            .into_iter()
+            .zip(input_tables.iter())
+            .map(|(idxs, table)| {
+                let idxs_as_series = UInt64Array::from(("idxs", idxs)).into_series();
+                table.take(&idxs_as_series)
+            })
+            .collect::<DaftResult<Vec<_>>>()?;
         Ok(Arc::new(MicroPartition::new_loaded(
-            probe_side_table.schema.clone(),
-            Arc::new(vec![probe_side_table]),
+            probe_side_tables[0].schema.clone(),
+            Arc::new(probe_side_tables),
             None,
         )))
     }
@@ -128,20 +125,19 @@ impl AntiSemiProbeSink {
     // emit a final result at the end.
     fn probe_anti_semi_with_bitmap(
         probe_on: &[BoundExpr],
-        probe_set: &Arc<dyn Probeable>,
+        probe_state: &Arc<ProbeState>,
         bitmap_builder: &mut IndexBitmapBuilder,
         input: &Arc<MicroPartition>,
     ) -> DaftResult<()> {
         let input_tables = input.get_tables()?;
-
         let _loop = info_span!("AntiSemiOperator::eval_and_probe").entered();
         for table in input_tables.iter() {
             let join_keys = table.eval_expression_list(probe_on)?;
-            let idx_mapper = probe_set.probe_indices(&join_keys)?;
+            let idx_iter = probe_state.probe_indices(&join_keys)?;
 
-            for inner_iter in idx_mapper.make_iter().flatten() {
-                for (build_side_table_idx, build_row_idx) in inner_iter {
-                    bitmap_builder.mark_used(build_side_table_idx as usize, build_row_idx as usize);
+            for inner_iter in idx_iter.flatten() {
+                for build_row_idx in inner_iter {
+                    bitmap_builder.mark_used(build_row_idx as usize);
                 }
             }
         }
@@ -159,7 +155,7 @@ impl AntiSemiProbeSink {
             .expect("at least one state should be present");
         let (first_probe_state, first_bitmap_builder) =
             first_state.get_or_await_probe_state(true).await;
-        let tables = first_probe_state.get_tables();
+        let table = first_probe_state.get_record_batch();
         let first_bitmap = first_bitmap_builder
             .take()
             .expect("bitmap should be set")
@@ -191,13 +187,9 @@ impl AntiSemiProbeSink {
             merged_bitmap = merged_bitmap.negate();
         }
 
-        let leftovers = merged_bitmap
-            .convert_to_boolean_arrays()
-            .zip(tables.iter())
-            .map(|(bitmap, table)| table.mask_filter(&bitmap.into_series()))
-            .collect::<DaftResult<Vec<_>>>()?;
+        let build_side_table =
+            table.mask_filter(&merged_bitmap.convert_to_boolean_array().into_series())?;
 
-        let build_side_table = RecordBatch::concat(&leftovers)?;
         Ok(Some(Arc::new(MicroPartition::new_loaded(
             build_side_table.schema.clone(),
             Arc::new(vec![build_side_table]),
@@ -230,18 +222,14 @@ impl StreamingSink for AntiSemiProbeSink {
                     if let Some(bm_builder) = bitmap_builder {
                         Self::probe_anti_semi_with_bitmap(
                             &params.probe_on,
-                            ps.get_probeable(),
+                            ps,
                             bm_builder,
                             &input,
                         )?;
                         Ok((state, StreamingSinkOutput::NeedMoreInput(None)))
                     } else {
-                        let res = Self::probe_anti_semi(
-                            &params.probe_on,
-                            ps.get_probeable(),
-                            &input,
-                            params.is_semi,
-                        );
+                        let res =
+                            Self::probe_anti_semi(&params.probe_on, ps, &input, params.is_semi);
                         Ok((state, StreamingSinkOutput::NeedMoreInput(Some(res?))))
                     }
                 },
