@@ -1,19 +1,20 @@
 use std::sync::Arc;
 
 use common_error::{DaftError, DaftResult, ensure};
+use common_io_config::IOConfig;
 #[cfg(feature = "python")]
 use common_py_serde::{PyObjectWrapper, deserialize_py_object, serialize_py_object};
 use common_resource_request::ResourceRequest;
 use common_scan_info::{Pushdowns, ScanTaskLikeRef};
 use common_treenode::{DynTreeNode, TreeNode, TreeNodeRecursion};
-use daft_core::prelude::*;
+use daft_core::{join::JoinSide, prelude::*};
 use daft_dsl::{
-    Column, WindowExpr, WindowFrame, WindowSpec,
+    Column, ExprRef, WindowExpr, WindowFrame, WindowSpec,
     expr::{
         BoundColumn,
         bound_expr::{BoundAggExpr, BoundExpr, BoundWindowExpr},
     },
-    functions::python::get_resource_request,
+    functions::python::{UDFProperties, get_resource_request},
 };
 use daft_logical_plan::{
     InMemoryInfo, OutputFileInfo,
@@ -27,6 +28,7 @@ pub type LocalPhysicalPlanRef = Arc<LocalPhysicalPlan>;
 pub enum LocalPhysicalPlan {
     InMemoryScan(InMemoryScan),
     PhysicalScan(PhysicalScan),
+    GlobScan(GlobScan),
     EmptyScan(EmptyScan),
     PlaceholderScan(PlaceholderScan),
     Project(Project),
@@ -54,7 +56,6 @@ pub enum LocalPhysicalPlan {
     Concat(Concat),
     HashJoin(HashJoin),
     CrossJoin(CrossJoin),
-    // SortMergeJoin(SortMergeJoin),
     // BroadcastJoin(BroadcastJoin),
     PhysicalWrite(PhysicalWrite),
     CommitWrite(CommitWrite),
@@ -74,6 +75,7 @@ pub enum LocalPhysicalPlan {
     // Flotilla Only Nodes
     Repartition(Repartition),
     IntoPartitions(IntoPartitions),
+    SortMergeJoin(SortMergeJoin),
     #[cfg(feature = "python")]
     DistributedActorPoolProject(DistributedActorPoolProject),
 }
@@ -94,6 +96,7 @@ impl LocalPhysicalPlan {
         match self {
             Self::InMemoryScan(InMemoryScan { stats_state, .. })
             | Self::PhysicalScan(PhysicalScan { stats_state, .. })
+            | Self::GlobScan(GlobScan { stats_state, .. })
             | Self::PlaceholderScan(PlaceholderScan { stats_state, .. })
             | Self::EmptyScan(EmptyScan { stats_state, .. })
             | Self::Project(Project { stats_state, .. })
@@ -114,6 +117,7 @@ impl LocalPhysicalPlan {
             | Self::Concat(Concat { stats_state, .. })
             | Self::HashJoin(HashJoin { stats_state, .. })
             | Self::CrossJoin(CrossJoin { stats_state, .. })
+            | Self::SortMergeJoin(SortMergeJoin { stats_state, .. })
             | Self::PhysicalWrite(PhysicalWrite { stats_state, .. })
             | Self::CommitWrite(CommitWrite { stats_state, .. })
             | Self::Repartition(Repartition { stats_state, .. })
@@ -161,6 +165,23 @@ impl LocalPhysicalPlan {
         .arced()
     }
 
+    pub fn glob_scan(
+        glob_paths: Arc<Vec<String>>,
+        pushdowns: Pushdowns,
+        schema: SchemaRef,
+        stats_state: StatsState,
+        io_config: Option<IOConfig>,
+    ) -> LocalPhysicalPlanRef {
+        Self::GlobScan(GlobScan {
+            glob_paths,
+            pushdowns,
+            schema,
+            stats_state,
+            io_config,
+        })
+        .arced()
+    }
+
     pub fn placeholder_scan(schema: SchemaRef, stats_state: StatsState) -> LocalPhysicalPlanRef {
         Self::PlaceholderScan(PlaceholderScan {
             schema,
@@ -258,14 +279,16 @@ impl LocalPhysicalPlan {
 
     pub fn udf_project(
         input: LocalPhysicalPlanRef,
-        project: BoundExpr,
+        expr: BoundExpr,
+        udf_properties: UDFProperties,
         passthrough_columns: Vec<BoundExpr>,
         schema: SchemaRef,
         stats_state: StatsState,
     ) -> LocalPhysicalPlanRef {
         Self::UDFProject(UDFProject {
             input,
-            project,
+            expr,
+            udf_properties,
             passthrough_columns,
             schema,
             stats_state,
@@ -569,6 +592,7 @@ impl LocalPhysicalPlan {
         right: LocalPhysicalPlanRef,
         left_on: Vec<BoundExpr>,
         right_on: Vec<BoundExpr>,
+        build_on_left: Option<bool>,
         null_equals_null: Option<Vec<bool>>,
         join_type: JoinType,
         schema: SchemaRef,
@@ -579,6 +603,7 @@ impl LocalPhysicalPlan {
             right,
             left_on,
             right_on,
+            build_on_left,
             null_equals_null,
             join_type,
             schema,
@@ -602,6 +627,27 @@ impl LocalPhysicalPlan {
         .arced()
     }
 
+    pub fn sort_merge_join(
+        left: LocalPhysicalPlanRef,
+        right: LocalPhysicalPlanRef,
+        left_on: Vec<BoundExpr>,
+        right_on: Vec<BoundExpr>,
+        join_type: JoinType,
+        schema: SchemaRef,
+        stats_state: StatsState,
+    ) -> LocalPhysicalPlanRef {
+        Self::SortMergeJoin(SortMergeJoin {
+            left,
+            right,
+            left_on,
+            right_on,
+            join_type,
+            schema,
+            stats_state,
+        })
+        .arced()
+    }
+
     pub(crate) fn concat(
         input: LocalPhysicalPlanRef,
         other: LocalPhysicalPlanRef,
@@ -736,6 +782,7 @@ impl LocalPhysicalPlan {
     pub fn schema(&self) -> &SchemaRef {
         match self {
             Self::PhysicalScan(PhysicalScan { schema, .. })
+            | Self::GlobScan(GlobScan { schema, .. })
             | Self::PlaceholderScan(PlaceholderScan { schema, .. })
             | Self::EmptyScan(EmptyScan { schema, .. })
             | Self::Filter(Filter { schema, .. })
@@ -752,6 +799,7 @@ impl LocalPhysicalPlan {
             | Self::Sample(Sample { schema, .. })
             | Self::HashJoin(HashJoin { schema, .. })
             | Self::CrossJoin(CrossJoin { schema, .. })
+            | Self::SortMergeJoin(SortMergeJoin { schema, .. })
             | Self::Explode(Explode { schema, .. })
             | Self::Unpivot(Unpivot { schema, .. })
             | Self::Concat(Concat { schema, .. })
@@ -783,9 +831,13 @@ impl LocalPhysicalPlan {
     pub fn resource_request(self: &Arc<Self>) -> ResourceRequest {
         let mut base = ResourceRequest::default_cpu();
         self.apply(|plan| match plan.as_ref() {
-            Self::UDFProject(UDFProject { project, .. }) => {
-                if let Some(resource_request) = get_resource_request([project]) {
-                    base = base.max(&resource_request);
+            Self::UDFProject(UDFProject {
+                expr,
+                udf_properties,
+                ..
+            }) => {
+                if let Some(resource_request) = &udf_properties.resource_request {
+                    base = base.max(resource_request);
                 }
                 Ok(TreeNodeRecursion::Continue)
             }
@@ -808,6 +860,7 @@ impl LocalPhysicalPlan {
     fn children(&self) -> Vec<LocalPhysicalPlanRef> {
         match self {
             Self::PhysicalScan(_)
+            | Self::GlobScan(_)
             | Self::PlaceholderScan(_)
             | Self::EmptyScan(_)
             | Self::InMemoryScan(_) => vec![],
@@ -836,6 +889,9 @@ impl LocalPhysicalPlan {
 
             Self::HashJoin(HashJoin { left, right, .. }) => vec![left.clone(), right.clone()],
             Self::CrossJoin(CrossJoin { left, right, .. }) => vec![left.clone(), right.clone()],
+            Self::SortMergeJoin(SortMergeJoin { left, right, .. }) => {
+                vec![left.clone(), right.clone()]
+            }
             #[cfg(feature = "python")]
             Self::CatalogWrite(CatalogWrite { input, .. }) => vec![input.clone()],
             #[cfg(feature = "python")]
@@ -890,13 +946,15 @@ impl LocalPhysicalPlan {
                     StatsState::NotMaterialized,
                 ),
                 Self::UDFProject(UDFProject {
-                    project,
+                    expr,
+                    udf_properties,
                     passthrough_columns,
                     schema,
                     ..
                 }) => Self::udf_project(
                     new_child.clone(),
-                    project.clone(),
+                    expr.clone(),
+                    udf_properties.clone(),
                     passthrough_columns.clone(),
                     schema.clone(),
                     StatsState::NotMaterialized,
@@ -1216,9 +1274,17 @@ impl LocalPhysicalPlan {
                 Self::CrossJoin(_) => {
                     panic!("LocalPhysicalPlan::with_new_children: CrossJoin should have 2 children")
                 }
+                Self::SortMergeJoin(_) => {
+                    panic!(
+                        "LocalPhysicalPlan::with_new_children: SortMergeJoin should have 2 children"
+                    )
+                }
                 Self::Concat(_) => {
                     panic!("LocalPhysicalPlan::with_new_children: Concat should have 2 children")
                 }
+                Self::GlobScan(_) => {
+                    panic!("LocalPhysicalPlan::with_new_children: GlobScan does not have children")
+                }
             },
             [new_left, new_right] => match self {
                 Self::HashJoin(HashJoin {
@@ -1226,6 +1292,7 @@ impl LocalPhysicalPlan {
                     right_on,
                     null_equals_null,
                     join_type,
+                    build_on_left,
                     schema,
                     stats_state,
                     ..
@@ -1234,6 +1301,7 @@ impl LocalPhysicalPlan {
                     new_right.clone(),
                     left_on.clone(),
                     right_on.clone(),
+                    *build_on_left,
                     null_equals_null.clone(),
                     *join_type,
                     schema.clone(),
@@ -1249,6 +1317,22 @@ impl LocalPhysicalPlan {
                     schema.clone(),
                     stats_state.clone(),
                 ),
+                Self::SortMergeJoin(SortMergeJoin {
+                    left_on,
+                    right_on,
+                    join_type,
+                    schema,
+                    stats_state,
+                    ..
+                }) => Self::sort_merge_join(
+                    new_left.clone(),
+                    new_right.clone(),
+                    left_on.clone(),
+                    right_on.clone(),
+                    *join_type,
+                    schema.clone(),
+                    stats_state.clone(),
+                ),
                 Self::Concat(Concat { .. }) => Self::concat(
                     new_left.clone(),
                     new_right.clone(),
@@ -1313,6 +1397,15 @@ pub struct PhysicalScan {
     pub stats_state: StatsState,
 }
 
+#[derive(Debug, Serialize, Deserialize)]
+pub struct GlobScan {
+    pub glob_paths: Arc<Vec<String>>,
+    pub pushdowns: Pushdowns,
+    pub schema: SchemaRef,
+    pub stats_state: StatsState,
+    pub io_config: Option<common_io_config::IOConfig>,
+}
+
 #[derive(Debug, Serialize, Deserialize)]
 pub struct PlaceholderScan {
     pub schema: SchemaRef,
@@ -1336,7 +1429,8 @@ pub struct Project {
 #[derive(Debug, Serialize, Deserialize)]
 pub struct UDFProject {
     pub input: LocalPhysicalPlanRef,
-    pub project: BoundExpr,
+    pub expr: BoundExpr,
+    pub udf_properties: UDFProperties,
     pub passthrough_columns: Vec<BoundExpr>,
     pub schema: SchemaRef,
     pub stats_state: StatsState,
@@ -1488,6 +1582,7 @@ pub struct HashJoin {
     pub right: LocalPhysicalPlanRef,
     pub left_on: Vec<BoundExpr>,
     pub right_on: Vec<BoundExpr>,
+    pub build_on_left: Option<bool>,
     pub null_equals_null: Option<Vec<bool>>,
     pub join_type: JoinType,
     pub schema: SchemaRef,
@@ -1502,6 +1597,17 @@ pub struct CrossJoin {
     pub stats_state: StatsState,
 }
 
+#[derive(Debug, Serialize, Deserialize)]
+pub struct SortMergeJoin {
+    pub left: LocalPhysicalPlanRef,
+    pub right: LocalPhysicalPlanRef,
+    pub left_on: Vec<BoundExpr>,
+    pub right_on: Vec<BoundExpr>,
+    pub join_type: JoinType,
+    pub schema: SchemaRef,
+    pub stats_state: StatsState,
+}
+
 #[derive(Debug, Serialize, Deserialize)]
 pub struct Concat {
     pub input: LocalPhysicalPlanRef,
