@@ -41,6 +41,13 @@ pub fn translate(plan: &LogicalPlanRef) -> DaftResult<LocalPhysicalPlanRef> {
                         ))
                     }
                 }
+                SourceInfo::GlobScan(info) => Ok(LocalPhysicalPlan::glob_scan(
+                    info.glob_paths.clone(),
+                    info.pushdowns.clone(),
+                    source.output_schema.clone(),
+                    source.stats_state.clone(),
+                    info.io_config.clone().map(|c| *c),
+                )),
                 SourceInfo::PlaceHolder(ph) => Ok(LocalPhysicalPlan::placeholder_scan(
                     ph.source_schema.clone(),
                     StatsState::NotMaterialized,
@@ -92,13 +99,14 @@ pub fn translate(plan: &LogicalPlanRef) -> DaftResult<LocalPhysicalPlanRef> {
         LogicalPlan::UDFProject(udf_project) => {
             let input = translate(&udf_project.input)?;
 
-            let project = BoundExpr::try_new(udf_project.project.clone(), input.schema())?;
             let passthrough_columns =
                 BoundExpr::bind_all(&udf_project.passthrough_columns, input.schema())?;
+            let expr = BoundExpr::try_new(udf_project.expr.clone(), input.schema())?;
 
             Ok(LocalPhysicalPlan::udf_project(
                 input,
-                project,
+                expr,
+                udf_project.udf_properties.clone(),
                 passthrough_columns,
                 udf_project.projected_schema.clone(),
                 udf_project.stats_state.clone(),
@@ -282,13 +290,20 @@ pub fn translate(plan: &LogicalPlanRef) -> DaftResult<LocalPhysicalPlanRef> {
             ))
         }
         LogicalPlan::Join(join) => {
-            if join
-                .join_strategy
-                .is_some_and(|x| !matches!(x, JoinStrategy::Hash))
-            {
-                log::warn!(
-                    "Only hash join strategy is supported on the native runner, falling back to hash join. Broadcast and sort merge joins are not implemented on the native runner as it is single node only."
-                );
+            if let Some(strategy) = join.join_strategy {
+                match strategy {
+                    JoinStrategy::Broadcast => {
+                        log::warn!(
+                            "Broadcast join is not supported on the native runner, falling back to hash join. Please use the ray runner, daft.set_runner_ray(), if you require broadcast joins."
+                        );
+                    }
+                    JoinStrategy::SortMerge => {
+                        log::warn!(
+                            "Sort merge join is not supported on the native runner, falling back to hash join."
+                        );
+                    }
+                    _ => {}
+                }
             }
             let left = translate(&join.left)?;
             let right = translate(&join.right)?;
@@ -318,6 +333,7 @@ pub fn translate(plan: &LogicalPlanRef) -> DaftResult<LocalPhysicalPlanRef> {
                     right,
                     left_on,
                     right_on,
+                    None,
                     Some(null_equals_nulls),
                     join.join_type,
                     join.output_schema.clone(),
