@@ -26,6 +26,7 @@ use resolve_expr::ExprResolver;
 use {
     crate::sink_info::{CatalogInfo, IcebergCatalogInfo},
     common_daft_config::PyDaftPlanningConfig,
+    common_io_config::python::IOConfig as PyIOConfig,
     daft_dsl::python::PyExpr,
     // daft_scan::python::pylib::ScanOperatorHandle,
     daft_schema::python::schema::PySchema,
@@ -46,7 +47,7 @@ use crate::{
         HashRepartitionConfig, IntoPartitionsConfig, RandomShuffleConfig, RepartitionSpec,
     },
     sink_info::{OutputFileInfo, SinkInfo},
-    source_info::{InMemoryInfo, SourceInfo},
+    source_info::{GlobScanInfo, InMemoryInfo, SourceInfo},
 };
 
 /// A logical plan builder, which simplifies constructing logical plans via
@@ -164,6 +165,19 @@ impl LogicalPlanBuilder {
         Ok(Self::from(Arc::new(logical_plan)))
     }
 
+    /// Creates a `LogicalPlan::Source` from glob paths.
+    pub fn from_glob_scan(
+        glob_paths: Vec<String>,
+        io_config: Option<IOConfig>,
+    ) -> DaftResult<Self> {
+        let glob_scan_info = GlobScanInfo::new(glob_paths, io_config);
+        let schema = glob_scan_info.schema.clone();
+        let logical_plan: LogicalPlan =
+            ops::Source::new(schema, SourceInfo::GlobScan(glob_scan_info).into()).into();
+
+        Ok(Self::from(Arc::new(logical_plan)))
+    }
+
     /// Creates a `LogicalPlan::Source` from a scan handle.
     pub fn table_scan(
         scan_operator: ScanOperatorRef,
@@ -1054,6 +1068,14 @@ impl PyLogicalPlanBuilder {
         .into())
     }
 
+    #[staticmethod]
+    pub fn from_glob_scan(
+        glob_paths: Vec<String>,
+        io_config: Option<PyIOConfig>,
+    ) -> PyResult<Self> {
+        Ok(LogicalPlanBuilder::from_glob_scan(glob_paths, io_config.map(|c| c.config))?.into())
+    }
+
     pub fn with_planning_config(
         &self,
         daft_planning_config: PyDaftPlanningConfig,
