@@ -6,9 +6,7 @@ use daft_core::prelude::*;
 use daft_dsl::{
     AggExpr, ApproxPercentileParams, Column, Expr, ExprRef,
     functions::{FunctionArgs, scalar::ScalarFn},
-    optimization,
-    python_udf::{PyScalarFn, RowWisePyFn},
-    resolved_col,
+    optimization, resolved_col,
 };
 use indexmap::{IndexMap, IndexSet};
 use itertools::Itertools;
@@ -480,14 +478,9 @@ fn replace_column_with_semantic_id(
                     Transformed::yes(Expr::InSubquery(expr.data, subquery.clone()).into())
                 }
             }
-            Expr::ScalarFn(ScalarFn::Python(PyScalarFn::RowWise(RowWisePyFn {
-                function_name: name,
-                inner: func,
-                return_dtype,
-                original_args,
-                args: children,
-            }))) => {
-                let transforms = children
+            Expr::ScalarFn(ScalarFn::Python(udf)) => {
+                let transforms = udf
+                    .args()
                     .iter()
                     .map(|e| {
                         replace_column_with_semantic_id(e.clone(), subexprs_to_replace, schema)
@@ -502,13 +495,7 @@ fn replace_column_with_semantic_id(
                         .map(|t| t.data.clone())
                         .collect::<Vec<_>>();
                     Transformed::yes(Arc::new(Expr::ScalarFn(ScalarFn::Python(
-                        PyScalarFn::RowWise(RowWisePyFn {
-                            function_name: name.clone(),
-                            inner: func.clone(),
-                            return_dtype: return_dtype.clone(),
-                            original_args: original_args.clone(),
-                            args: new_children,
-                        }),
+                        udf.with_new_children(new_children),
                     ))))
                 }
             }
