@@ -3,15 +3,15 @@ use std::sync::Arc;
 use common_error::DaftError;
 use daft_algebra::boolean::combine_conjunction;
 use daft_core::{count_mode::CountMode, join::JoinType, utils::supertype::get_supertype};
-use daft_dsl::{left_col, lit, null_lit, resolved_col, right_col, ExprRef};
+use daft_dsl::{ExprRef, left_col, lit, null_lit, resolved_col, right_col};
 use daft_functions_list::{explode, list_fill};
 use daft_schema::{dtype::DataType, field::Field, schema::SchemaRef};
 use indexmap::IndexSet;
 use serde::{Deserialize, Serialize};
 use snafu::ResultExt;
 
-use super::{join::JoinPredicate, Aggregate, Concat, Distinct, Filter, Project};
-use crate::{logical_plan, logical_plan::CreationSnafu, LogicalPlan};
+use super::{Aggregate, Concat, Distinct, Filter, Project, join::JoinPredicate};
+use crate::{LogicalPlan, logical_plan, logical_plan::CreationSnafu};
 
 #[derive(Copy, Clone, Debug, PartialEq, Eq, Hash, Serialize, Deserialize)]
 pub enum SetQuantifier {
