@@ -12,7 +12,8 @@ use super::{
         PushDownAntiSemiJoin, PushDownFilter, PushDownJoinPredicate, PushDownLimit,
         PushDownProjection, PushDownShard, ReorderJoins, RewriteCountDistinct, RewriteOffset,
         ShardScans, SimplifyExpressionsRule, SimplifyNullFilteredJoin, SplitExplodeFromProject,
-        SplitGranularProjection, SplitUDFs, UnnestPredicateSubquery, UnnestScalarSubquery,
+        SplitGranularProjection, SplitUDFs, SplitUDFsFromFilters, UnnestPredicateSubquery,
+        UnnestScalarSubquery,
     },
 };
 use crate::LogicalPlan;
@@ -36,7 +37,7 @@ impl OptimizerConfig {
 
 impl Default for OptimizerConfig {
     fn default() -> Self {
-        // Default to a max of 5 optimizer passes for a given batch.
+        // Default to a max of 20 optimizer passes for a given batch.
         Self::new(20, false)
     }
 }
@@ -170,6 +171,7 @@ impl OptimizerBuilder {
             // UDFs and monotonically increasing ids.
             RuleBatch::new(
                 vec![
+                    Box::new(SplitUDFsFromFilters::new()),
                     Box::new(SplitUDFs::new()),
                     Box::new(PushDownProjection::new()),
                     Box::new(DetectMonotonicId::new()),
@@ -183,7 +185,9 @@ impl OptimizerBuilder {
             ),
             // --- Push down aggregations ---
             RuleBatch::new(
-                vec![Box::new(PushDownAggregation::new())],
+                vec![Box::new(PushDownAggregation::new(
+                    self.config.strict_pushdown,
+                ))],
                 RuleExecutionStrategy::Once,
             ),
             // --- Shard pushdowns ---
@@ -266,11 +270,7 @@ impl OptimizerBuilder {
     }
 
     pub fn when(self, condition: bool, f: impl FnOnce(Self) -> Self) -> Self {
-        if condition {
-            f(self)
-        } else {
-            self
-        }
+        if condition { f(self) } else { self }
     }
 }
 
@@ -383,19 +383,20 @@ mod tests {
     use common_treenode::{Transformed, TreeNode};
     use daft_core::prelude::*;
     use daft_dsl::{
-        functions::{python::LegacyPythonUDF, FunctionExpr},
-        lit, resolved_col, unresolved_col, AggExpr, Expr,
+        AggExpr, Expr,
+        functions::{FunctionExpr, python::LegacyPythonUDF},
+        lit, resolved_col, unresolved_col,
     };
 
     use super::{Optimizer, OptimizerBuilder, OptimizerConfig, RuleBatch, RuleExecutionStrategy};
     use crate::{
+        LogicalPlan,
         ops::{Filter, Project, UDFProject},
         optimization::rules::{EnrichWithStats, MaterializeScans, OptimizerRule},
         test::{
             dummy_scan_node, dummy_scan_node_with_pushdowns, dummy_scan_operator,
             dummy_scan_operator_for_aggregation,
         },
-        LogicalPlan,
     };
 
     /// Test that the optimizer terminates early when the plan is not transformed
@@ -697,10 +698,12 @@ mod tests {
         let scan_op = dummy_scan_operator(vec![Field::new("a", DataType::Int64)]);
 
         // Create an actor pool project expression.
+        let inputs = vec![resolved_col("a").into()];
         let actor_pool_expr = Arc::new(Expr::Function {
             func: FunctionExpr::Python(LegacyPythonUDF::new_testing_udf()),
-            inputs: vec![resolved_col("a").into()],
-        });
+            inputs: inputs.clone(),
+        })
+        .alias("a");
 
         // Create a plan with Select using actor pool project followed by Limit.
         let plan = dummy_scan_node(scan_op.clone())
@@ -718,7 +721,7 @@ mod tests {
         let expected = LogicalPlan::UDFProject(UDFProject::try_new(
             expected.clone(),
             // Internally, splitting an actor pool project always re-aliases the column to its original name.
-            actor_pool_expr.alias("a"),
+            actor_pool_expr.clone(),
             vec![],
         )?)
         .arced();
@@ -754,10 +757,12 @@ mod tests {
     fn filter_commutes_with_actor_pool_project() -> DaftResult<()> {
         let scan_op = dummy_scan_operator(vec![Field::new("a", DataType::Int64)]);
         // Create an actor pool project expression.
+        let inputs = vec![resolved_col("a").into()];
         let actor_pool_expr = Arc::new(Expr::Function {
             func: FunctionExpr::Python(LegacyPythonUDF::new_testing_udf()),
-            inputs: vec![resolved_col("a").into()],
-        });
+            inputs,
+        })
+        .alias("renamed_col");
 
         // Create a plan with Select using actor pool project followed by Filter.
         let plan = dummy_scan_node(scan_op.clone())
@@ -777,7 +782,7 @@ mod tests {
         let expected = LogicalPlan::UDFProject(UDFProject::try_new(
             expected.clone(),
             // Internally, splitting an actor pool project always re-aliases the column to its original name.
-            actor_pool_expr.alias("renamed_col"),
+            actor_pool_expr.clone(),
             vec![resolved_col("a")],
         )?)
         .arced();
@@ -835,6 +840,7 @@ mod tests {
                 )))))
                 .with_columns(Some(Arc::new(vec!["a".to_string()]))),
         )
+        .aggregate(vec![unresolved_col("a").sum()], vec![])?
         .build();
 
         let scan_materializer_and_stats_enricher = get_scan_materializer_and_stats_enricher();
