@@ -3,14 +3,15 @@ use std::{any::TypeId, collections::HashSet, sync::Arc};
 use common_error::DaftResult;
 use common_treenode::{Transformed, TreeNode};
 use daft_dsl::{
-    functions::{scalar::ScalarFn, BuiltinScalarFn},
-    resolved_col, Expr,
+    Expr,
+    functions::{BuiltinScalarFn, scalar::ScalarFn},
+    resolved_col,
 };
 use daft_functions_uri::download::UrlDownload;
 use itertools::Itertools;
 
 use super::OptimizerRule;
-use crate::{ops::Project, LogicalPlan};
+use crate::{LogicalPlan, ops::Project};
 
 /// This rule will split projections into multiple projections such that expressions that
 /// need their own granular morsel sizing will be isolated. Right now, those would be
@@ -213,10 +214,10 @@ impl SplitGranularProjection {
 mod tests {
 
     use common_scan_info::Pushdowns;
-    use daft_dsl::{lit, Column, ExprRef, ResolvedColumn};
-    use daft_functions_binary::{BinaryConcat, BinaryDecode, Codec};
+    use daft_dsl::{Column, ExprRef, Operator, ResolvedColumn, lit};
+    use daft_functions_binary::{BinaryDecode, Codec};
     use daft_functions_uri::download::UrlDownload;
-    use daft_functions_utf8::{capitalize, lower, Capitalize};
+    use daft_functions_utf8::{Capitalize, capitalize, lower};
     use daft_schema::{dtype::DataType, field::Field};
 
     use super::*;
@@ -250,11 +251,9 @@ mod tests {
             dummy_scan_operator(vec![Field::new("url", DataType::Utf8)]),
             Pushdowns::default(),
         )
-        .with_columns(vec![BuiltinScalarFn::new(
-            UrlDownload,
-            vec![resolved_col("url")],
-        )
-        .into()])
+        .with_columns(vec![
+            BuiltinScalarFn::new(UrlDownload, vec![resolved_col("url")]).into(),
+        ])
         .unwrap()
         .build();
 
@@ -365,13 +364,12 @@ mod tests {
             Pushdowns::default(),
         )
         .select(vec![
-            ExprRef::from(BuiltinScalarFn::new(
-                BinaryConcat,
-                vec![
-                    BuiltinScalarFn::new(UrlDownload, vec![capitalize(resolved_col("url"))]).into(),
-                    resolved_col("extra"),
-                ],
-            ))
+            Arc::new(Expr::BinaryOp {
+                op: Operator::Plus,
+                left: BuiltinScalarFn::new(UrlDownload, vec![capitalize(resolved_col("url"))])
+                    .into(),
+                right: resolved_col("extra"),
+            })
             .alias("url_data"),
             lower(Expr::Column(Column::Resolved(ResolvedColumn::Basic("name".into()))).arced())
                 .alias("name_lower"),
@@ -408,7 +406,10 @@ mod tests {
         };
         assert!(matches!(
             func.as_ref(),
-            Expr::ScalarFn(ScalarFn::Builtin(BuiltinScalarFn { udf, .. })) if udf.as_ref().type_id() == TypeId::of::<BinaryConcat>()
+            Expr::BinaryOp {
+                op: Operator::Plus,
+                ..
+            }
         ));
 
         // Check that the top level project has a single child, which is a project
