@@ -81,6 +81,8 @@ impl PushDownFilter {
                 match source.source_info.as_ref() {
                     // Filter pushdown is not supported for in-memory sources.
                     SourceInfo::InMemory(_) => return Ok(Transformed::no(plan)),
+                    // Filter pushdown is not supported for glob scan sources.
+                    SourceInfo::GlobScan(_) => return Ok(Transformed::no(plan)),
                     // Do not pushdown if Source node already has a limit
                     SourceInfo::Physical(external_info)
                         if let Some(_) = external_info.pushdowns.limit =>
