@@ -169,6 +169,7 @@ impl PushDownProjection {
                         }
                     }
                     SourceInfo::InMemory(_) => Ok(Transformed::no(plan)),
+                    SourceInfo::GlobScan(_) => Ok(Transformed::no(plan)),
                     SourceInfo::PlaceHolder(..) => {
                         panic!("PlaceHolderInfo should not exist for optimization!");
                     }
@@ -237,9 +238,9 @@ impl PushDownProjection {
                     .map(|e| e.name().to_string())
                     .collect::<IndexSet<_>>();
                 // Assume that previous opt rule should have removed unused cols, thus this op
-                let output_column = upstream_udf.project.name().to_string();
-                debug_assert!(required_columns.contains(&output_column));
-                required_columns.shift_remove(&output_column);
+                let output_column = upstream_udf.expr.name();
+                debug_assert!(required_columns.contains(output_column));
+                required_columns.shift_remove(output_column);
 
                 // If required_columns ⊂ passthrough_set, then push down the projection
                 if required_columns.len() < passthrough_set.len()
@@ -253,7 +254,7 @@ impl PushDownProjection {
 
                     let new_upstream = LogicalPlan::UDFProject(UDFProject::try_new(
                         upstream_udf.input.clone(),
-                        upstream_udf.project.clone(),
+                        upstream_udf.expr.clone(),
                         pruned_passthrough,
                     )?)
                     .arced();
