@@ -5,9 +5,9 @@ use common_treenode::{DynTreeNode, Transformed, TreeNode};
 
 use super::OptimizerRule;
 use crate::{
+    LogicalPlan,
     ops::{Shard, Source},
     source_info::SourceInfo,
-    LogicalPlan,
 };
 
 /// Optimization rules for pushing Limits further into the logical plan.
@@ -73,6 +73,10 @@ impl PushDownShard {
                             SourceInfo::InMemory(_) => Err(DaftError::ValueError(
                                 "Sharding is not supported for in-memory sources".to_string(),
                             )),
+                            // Shard pushdown is not supported for glob scan sources.
+                            SourceInfo::GlobScan(_) => Err(DaftError::ValueError(
+                                "Sharding is not supported for glob scan sources".to_string(),
+                            )),
                             // If there are multiple shards, throw an error.
                             SourceInfo::Physical(external_info)
                                 if external_info.pushdowns.sharder.is_some() =>
