@@ -3,20 +3,88 @@ use std::{any::TypeId, collections::HashSet, sync::Arc};
 use common_error::DaftResult;
 use common_treenode::{Transformed, TreeNode, TreeNodeRecursion, TreeNodeRewriter};
 use daft_dsl::{
-    functions::{scalar::ScalarFn, BuiltinScalarFn},
+    Column, Expr, ExprRef, ResolvedColumn,
+    functions::{BuiltinScalarFn, scalar::ScalarFn},
     is_udf,
     optimization::{get_required_columns, requires_computation},
-    resolved_col, Column, Expr, ExprRef, ResolvedColumn,
+    resolved_col,
 };
 use daft_functions_list::ListMap;
 use itertools::Itertools;
 
 use super::OptimizerRule;
 use crate::{
-    ops::{Project, UDFProject},
     LogicalPlan,
+    ops::{Filter, Project, UDFProject},
 };
 
+/// Simple optimizer rule that checks if filters contain a UDF and if so, pulls it out of the filter.
+/// Expectation is that this rule will run before SplitUDFs, so that the UDFs are split out of the filter expression.
+#[derive(Default, Debug)]
+pub struct SplitUDFsFromFilters {}
+
+impl SplitUDFsFromFilters {
+    pub fn new() -> Self {
+        Self {}
+    }
+
+    pub fn try_optimize_filter(
+        &self,
+        count: &mut usize,
+        filter: &Filter,
+        plan: &Arc<LogicalPlan>,
+    ) -> DaftResult<Transformed<Arc<LogicalPlan>>> {
+        if filter.predicate.exists(is_udf) {
+            let col_name = format!("__SplitUDFsFromFilters_udf_{}__", count);
+            *count += 1;
+
+            let predicate_node = LogicalPlan::Project(Project::try_new(
+                filter.input.clone(),
+                filter
+                    .input
+                    .schema()
+                    .field_names()
+                    .map(resolved_col)
+                    .chain(std::iter::once(
+                        filter.predicate.clone().alias(col_name.as_str()),
+                    ))
+                    .collect(),
+            )?)
+            .into();
+
+            let new_filter = Arc::new(LogicalPlan::Filter(Filter::try_new(
+                predicate_node,
+                resolved_col(col_name.as_str()),
+            )?));
+
+            let exclude_project = Project::try_new(
+                new_filter,
+                filter
+                    .input
+                    .schema()
+                    .field_names()
+                    .map(resolved_col)
+                    .collect(),
+            )?
+            .into();
+
+            Ok(Transformed::yes(exclude_project))
+        } else {
+            Ok(Transformed::no(plan.clone()))
+        }
+    }
+}
+
+impl OptimizerRule for SplitUDFsFromFilters {
+    fn try_optimize(&self, plan: Arc<LogicalPlan>) -> DaftResult<Transformed<Arc<LogicalPlan>>> {
+        let mut udf_count = 0;
+        plan.transform_down(|node| match node.as_ref() {
+            LogicalPlan::Filter(filter) => self.try_optimize_filter(&mut udf_count, filter, &node),
+            _ => Ok(Transformed::no(node)),
+        })
+    }
+}
+
 #[derive(Default, Debug)]
 pub struct SplitUDFs {}
 
@@ -467,7 +535,7 @@ fn recursive_optimize_project(
     };
 
     // Start building a chain of `child -> Project -> ActorPoolProject -> ActorPoolProject -> ... -> Project`
-    let (actor_pool_stages, stateless_stages): (Vec<_>, Vec<_>) = truncated_exprs
+    let (udf_stages, stateless_stages): (Vec<_>, Vec<_>) = truncated_exprs
         .into_iter()
         .partition(|expr| exists_skip_list_map(expr, is_udf));
 
@@ -501,13 +569,14 @@ fn recursive_optimize_project(
     let new_plan = {
         let mut child = new_plan;
 
-        for expr in actor_pool_stages {
+        for expr in udf_stages {
             let passthrough_columns = child
                 .schema()
                 .field_names()
                 .map(resolved_col)
                 .filter(|c| c.name() != expr.name())
                 .collect();
+
             child = LogicalPlan::UDFProject(UDFProject::try_new(child, expr, passthrough_columns)?)
                 .arced();
         }
@@ -537,24 +606,25 @@ mod tests {
     use common_resource_request::ResourceRequest;
     use daft_core::prelude::*;
     use daft_dsl::{
+        Expr, ExprRef,
         functions::{
-            python::{LegacyPythonUDF, MaybeInitializedUDF, RuntimePyObject},
             FunctionExpr,
+            python::{LegacyPythonUDF, MaybeInitializedUDF, RuntimePyObject},
         },
-        resolved_col, Expr, ExprRef,
+        lit, resolved_col,
     };
     use indoc::indoc;
     use test_log::test;
 
     use super::SplitUDFs;
     use crate::{
+        LogicalPlan,
         optimization::{
             optimizer::{RuleBatch, RuleExecutionStrategy},
-            rules::PushDownProjection,
+            rules::{PushDownProjection, SplitUDFsFromFilters},
             test::assert_optimized_plan_with_rules_repr_eq,
         },
         test::{dummy_scan_node, dummy_scan_operator},
-        LogicalPlan,
     };
 
     /// Helper that creates an optimizer with the SplitExprByUDF rule registered, optimizes
@@ -618,6 +688,27 @@ mod tests {
         .arced()
     }
 
+    fn create_filter_udf(inputs: Vec<ExprRef>) -> ExprRef {
+        Expr::Function {
+            func: FunctionExpr::Python(LegacyPythonUDF {
+                name: Arc::new("foo".to_string()),
+                func: MaybeInitializedUDF::Uninitialized {
+                    inner: RuntimePyObject::new_none(),
+                    init_args: RuntimePyObject::new_none(),
+                },
+                bound_args: RuntimePyObject::new_none(),
+                num_expressions: inputs.len(),
+                return_dtype: DataType::Boolean,
+                resource_request: None,
+                batch_size: Some(32),
+                concurrency: None,
+                use_process: None,
+            }),
+            inputs,
+        }
+        .arced()
+    }
+
     fn create_resource_request() -> ResourceRequest {
         ResourceRequest::try_new_internal(Some(8.), Some(1.), None).unwrap()
     }
@@ -638,8 +729,8 @@ mod tests {
             project_plan,
             indoc! { "
             Project: col(a), col(b)
-              UDFProject:
-              UDF foo = py_udf(col(a)) as b
+              UDF: foo
+              Expr = py_udf(col(a)) as b
               Passthrough Columns = col(a)
               Concurrency = Some(8)
               Resource request = { num_cpus = 8, num_gpus = 1 }
@@ -673,25 +764,25 @@ mod tests {
             project_plan,
             indoc! {"
             Project: col(a), col(b), col(a_prime), col(b_prime)
-              UDFProject:
-              UDF foo = py_udf(col(__TruncateRootUDF_0-3-0__)) as b_prime
+              UDF: foo
+              Expr = py_udf(col(__TruncateRootUDF_0-3-0__)) as b_prime
               Passthrough Columns = col(__TruncateRootUDF_0-2-0__), col(__TruncateRootUDF_0-3-0__), col(a), col(b), col(a_prime)
               Concurrency = Some(8)
               Resource request = { num_cpus = 8, num_gpus = 1 }
-                UDFProject:
-                UDF foo = py_udf(col(__TruncateRootUDF_0-2-0__)) as a_prime
+                UDF: foo
+                Expr = py_udf(col(__TruncateRootUDF_0-2-0__)) as a_prime
                 Passthrough Columns = col(__TruncateRootUDF_0-2-0__), col(__TruncateRootUDF_0-3-0__), col(a), col(b)
                 Concurrency = Some(8)
                 Resource request = { num_cpus = 8, num_gpus = 1 }
                   Project: col(__TruncateRootUDF_0-2-0__), col(__TruncateRootUDF_0-3-0__), col(a), col(b)
                     Project: col(a), col(b), col(__TruncateRootUDF_0-2-0__), col(__TruncateRootUDF_0-3-0__)
-                      UDFProject:
-                      UDF foo = py_udf(col(b)) as __TruncateRootUDF_0-3-0__
+                      UDF: foo
+                      Expr = py_udf(col(b)) as __TruncateRootUDF_0-3-0__
                       Passthrough Columns = col(a), col(b), col(__TruncateRootUDF_0-2-0__)
                       Concurrency = Some(8)
                       Resource request = { num_cpus = 8, num_gpus = 1 }
-                        UDFProject:
-                        UDF foo = py_udf(col(a)) as __TruncateRootUDF_0-2-0__
+                        UDF: foo
+                        Expr = py_udf(col(a)) as __TruncateRootUDF_0-2-0__
                         Passthrough Columns = col(a), col(b)
                         Concurrency = Some(8)
                         Resource request = { num_cpus = 8, num_gpus = 1 }
@@ -722,15 +813,15 @@ mod tests {
             project_plan.clone(),
             indoc! {"
 Project: col(a), col(b)
-  UDFProject:
-  UDF foo = py_udf(col(__TruncateRootUDF_0-1-0__)) as b
+  UDF: foo
+  Expr = py_udf(col(__TruncateRootUDF_0-1-0__)) as b
   Passthrough Columns = col(__TruncateRootUDF_0-1-0__), col(a)
   Concurrency = Some(8)
   Resource request = { num_cpus = 8, num_gpus = 1 }
     Project: col(__TruncateRootUDF_0-1-0__), col(a)
       Project: col(a), col(__TruncateRootUDF_0-1-0__)
-        UDFProject:
-        UDF foo = py_udf(col(a)) as __TruncateRootUDF_0-1-0__
+        UDF: foo
+        Expr = py_udf(col(a)) as __TruncateRootUDF_0-1-0__
         Passthrough Columns = col(a)
         Concurrency = Some(8)
         Resource request = { num_cpus = 8, num_gpus = 1 }
@@ -748,14 +839,14 @@ Project: col(a), col(b)
         assert_optimized_plan_eq_with_projection_pushdown(
             project_plan,
             indoc! {"
-UDFProject:
-UDF foo = py_udf(col(__TruncateRootUDF_0-1-0__)) as b
+UDF: foo
+Expr = py_udf(col(__TruncateRootUDF_0-1-0__)) as b
 Passthrough Columns = col(a)
 Concurrency = Some(8)
 Resource request = { num_cpus = 8, num_gpus = 1 }
   Project: col(__TruncateRootUDF_0-1-0__), col(a)
-    UDFProject:
-    UDF foo = py_udf(col(a)) as __TruncateRootUDF_0-1-0__
+    UDF: foo
+    Expr = py_udf(col(a)) as __TruncateRootUDF_0-1-0__
     Passthrough Columns = col(a)
     Concurrency = Some(8)
     Resource request = { num_cpus = 8, num_gpus = 1 }
@@ -784,15 +875,15 @@ Resource request = { num_cpus = 8, num_gpus = 1 }
             project_plan.clone(),
             indoc! {"
 Project: col(a)
-  UDFProject:
-  UDF foo = py_udf(col(__TruncateRootUDF_0-0-0__)) as a
+  UDF: foo
+  Expr = py_udf(col(__TruncateRootUDF_0-0-0__)) as a
   Passthrough Columns = col(__TruncateRootUDF_0-0-0__)
   Concurrency = Some(8)
   Resource request = { num_cpus = 8, num_gpus = 1 }
     Project: col(__TruncateRootUDF_0-0-0__)
       Project: col(__TruncateRootUDF_0-0-0__)
-        UDFProject:
-        UDF foo = py_udf(col(a)) as __TruncateRootUDF_0-0-0__
+        UDF: foo
+        Expr = py_udf(col(a)) as __TruncateRootUDF_0-0-0__
         Passthrough Columns = col(a)
         Concurrency = Some(8)
         Resource request = { num_cpus = 8, num_gpus = 1 }
@@ -807,13 +898,13 @@ Project: col(a)
         assert_optimized_plan_eq_with_projection_pushdown(
             project_plan,
             indoc! {"
-UDFProject:
-UDF foo = py_udf(col(__TruncateRootUDF_0-0-0__)) as a
+UDF: foo
+Expr = py_udf(col(__TruncateRootUDF_0-0-0__)) as a
 Passthrough Columns = None
 Concurrency = Some(8)
 Resource request = { num_cpus = 8, num_gpus = 1 }
-  UDFProject:
-  UDF foo = py_udf(col(a)) as __TruncateRootUDF_0-0-0__
+  UDF: foo
+  Expr = py_udf(col(a)) as __TruncateRootUDF_0-0-0__
   Passthrough Columns = None
   Concurrency = Some(8)
   Resource request = { num_cpus = 8, num_gpus = 1 }
@@ -848,20 +939,20 @@ Resource request = { num_cpus = 8, num_gpus = 1 }
             project_plan.clone(),
             indoc! {"
 Project: col(c)
-  UDFProject:
-  UDF foo = py_udf(col(__TruncateRootUDF_0-0-0__), col(__TruncateRootUDF_0-0-1__)) as c
+  UDF: foo
+  Expr = py_udf(col(__TruncateRootUDF_0-0-0__), col(__TruncateRootUDF_0-0-1__)) as c
   Passthrough Columns = col(__TruncateRootUDF_0-0-0__), col(__TruncateRootUDF_0-0-1__)
   Concurrency = Some(8)
   Resource request = { num_cpus = 8, num_gpus = 1 }
     Project: col(__TruncateRootUDF_0-0-0__), col(__TruncateRootUDF_0-0-1__)
       Project: col(__TruncateRootUDF_0-0-0__), col(__TruncateRootUDF_0-0-1__)
-        UDFProject:
-        UDF foo = py_udf(col(b)) as __TruncateRootUDF_0-0-1__
+        UDF: foo
+        Expr = py_udf(col(b)) as __TruncateRootUDF_0-0-1__
         Passthrough Columns = col(a), col(b), col(__TruncateRootUDF_0-0-0__)
         Concurrency = Some(8)
         Resource request = { num_cpus = 8, num_gpus = 1 }
-          UDFProject:
-          UDF foo = py_udf(col(a)) as __TruncateRootUDF_0-0-0__
+          UDF: foo
+          Expr = py_udf(col(a)) as __TruncateRootUDF_0-0-0__
           Passthrough Columns = col(a), col(b)
           Concurrency = Some(8)
           Resource request = { num_cpus = 8, num_gpus = 1 }
@@ -877,18 +968,18 @@ Project: col(c)
         assert_optimized_plan_eq_with_projection_pushdown(
             project_plan,
             indoc! {"
-UDFProject:
-UDF foo = py_udf(col(__TruncateRootUDF_0-0-0__), col(__TruncateRootUDF_0-0-1__)) as c
+UDF: foo
+Expr = py_udf(col(__TruncateRootUDF_0-0-0__), col(__TruncateRootUDF_0-0-1__)) as c
 Passthrough Columns = None
 Concurrency = Some(8)
 Resource request = { num_cpus = 8, num_gpus = 1 }
-  UDFProject:
-  UDF foo = py_udf(col(b)) as __TruncateRootUDF_0-0-1__
+  UDF: foo
+  Expr = py_udf(col(b)) as __TruncateRootUDF_0-0-1__
   Passthrough Columns = col(__TruncateRootUDF_0-0-0__)
   Concurrency = Some(8)
   Resource request = { num_cpus = 8, num_gpus = 1 }
-    UDFProject:
-    UDF foo = py_udf(col(a)) as __TruncateRootUDF_0-0-0__
+    UDF: foo
+    Expr = py_udf(col(a)) as __TruncateRootUDF_0-0-0__
     Passthrough Columns = col(b)
     Concurrency = Some(8)
     Resource request = { num_cpus = 8, num_gpus = 1 }
@@ -908,9 +999,10 @@ Resource request = { num_cpus = 8, num_gpus = 1 }
             Field::new("b", DataType::Int64),
         ]);
         let scan_plan = dummy_scan_node(scan_op);
-        let stacked_actor_pool_project_expr =
-            create_actor_pool_udf(vec![create_actor_pool_udf(vec![resolved_col("a")])
-                .add(create_actor_pool_udf(vec![resolved_col("b")]))]);
+        let stacked_actor_pool_project_expr = create_actor_pool_udf(vec![
+            create_actor_pool_udf(vec![resolved_col("a")])
+                .add(create_actor_pool_udf(vec![resolved_col("b")])),
+        ]);
 
         // Add a Projection with actor pool UDF and resource request
         // Project([foo(foo(col("a")) + foo(col("b"))).alias("c")])
@@ -922,8 +1014,8 @@ Resource request = { num_cpus = 8, num_gpus = 1 }
             project_plan.clone(),
             indoc! {"
 Project: col(c)
-  UDFProject:
-  UDF foo = py_udf(col(__TruncateRootUDF_0-0-0__)) as c
+  UDF: foo
+  Expr = py_udf(col(__TruncateRootUDF_0-0-0__)) as c
   Passthrough Columns = col(__TruncateRootUDF_0-0-0__)
   Concurrency = Some(8)
   Resource request = { num_cpus = 8, num_gpus = 1 }
@@ -931,13 +1023,13 @@ Project: col(c)
       Project: col(__TruncateRootUDF_0-0-0__)
         Project: col(__TruncateAnyUDFChildren_1-0-0__), col(__TruncateAnyUDFChildren_1-0-1__), col(__TruncateAnyUDFChildren_1-0-0__) + col(__TruncateAnyUDFChildren_1-0-1__) as __TruncateRootUDF_0-0-0__
           Project: col(__TruncateAnyUDFChildren_1-0-0__), col(__TruncateAnyUDFChildren_1-0-1__)
-            UDFProject:
-            UDF foo = py_udf(col(b)) as __TruncateAnyUDFChildren_1-0-1__
+            UDF: foo
+            Expr = py_udf(col(b)) as __TruncateAnyUDFChildren_1-0-1__
             Passthrough Columns = col(a), col(b), col(__TruncateAnyUDFChildren_1-0-0__)
             Concurrency = Some(8)
             Resource request = { num_cpus = 8, num_gpus = 1 }
-              UDFProject:
-              UDF foo = py_udf(col(a)) as __TruncateAnyUDFChildren_1-0-0__
+              UDF: foo
+              Expr = py_udf(col(a)) as __TruncateAnyUDFChildren_1-0-0__
               Passthrough Columns = col(a), col(b)
               Concurrency = Some(8)
               Resource request = { num_cpus = 8, num_gpus = 1 }
@@ -953,19 +1045,19 @@ Project: col(c)
         assert_optimized_plan_eq_with_projection_pushdown(
             project_plan,
             indoc! {"
-UDFProject:
-UDF foo = py_udf(col(__TruncateRootUDF_0-0-0__)) as c
+UDF: foo
+Expr = py_udf(col(__TruncateRootUDF_0-0-0__)) as c
 Passthrough Columns = None
 Concurrency = Some(8)
 Resource request = { num_cpus = 8, num_gpus = 1 }
   Project: col(__TruncateAnyUDFChildren_1-0-0__) + col(__TruncateAnyUDFChildren_1-0-1__) as __TruncateRootUDF_0-0-0__
-    UDFProject:
-    UDF foo = py_udf(col(b)) as __TruncateAnyUDFChildren_1-0-1__
+    UDF: foo
+    Expr = py_udf(col(b)) as __TruncateAnyUDFChildren_1-0-1__
     Passthrough Columns = col(__TruncateAnyUDFChildren_1-0-0__)
     Concurrency = Some(8)
     Resource request = { num_cpus = 8, num_gpus = 1 }
-      UDFProject:
-      UDF foo = py_udf(col(a)) as __TruncateAnyUDFChildren_1-0-0__
+      UDF: foo
+      Expr = py_udf(col(a)) as __TruncateAnyUDFChildren_1-0-0__
       Passthrough Columns = col(b)
       Concurrency = Some(8)
       Resource request = { num_cpus = 8, num_gpus = 1 }
@@ -983,7 +1075,7 @@ Resource request = { num_cpus = 8, num_gpus = 1 }
         let scan_op = dummy_scan_operator(vec![Field::new("a", DataType::Int64)]);
         let scan_plan = dummy_scan_node(scan_op);
         let stacked_actor_pool_project_expr = create_actor_pool_udf(vec![
-            resolved_col("a").add(create_actor_pool_udf(vec![resolved_col("a")]))
+            resolved_col("a").add(create_actor_pool_udf(vec![resolved_col("a")])),
         ]);
 
         // Add a Projection with actor pool UDF and resource request
@@ -999,8 +1091,8 @@ Resource request = { num_cpus = 8, num_gpus = 1 }
             project_plan,
             indoc! {"
 Project: col(a), col(c)
-  UDFProject:
-  UDF foo = py_udf(col(__TruncateRootUDF_0-1-0__)) as c
+  UDF: foo
+  Expr = py_udf(col(__TruncateRootUDF_0-1-0__)) as c
   Passthrough Columns = col(__TruncateRootUDF_0-1-0__), col(a)
   Concurrency = Some(8)
   Resource request = { num_cpus = 8, num_gpus = 1 }
@@ -1008,8 +1100,8 @@ Project: col(a), col(c)
       Project: col(a), col(__TruncateRootUDF_0-1-0__)
         Project: col(__TruncateAnyUDFChildren_1-1-0__), col(a), col(a) + col(__TruncateAnyUDFChildren_1-1-0__) as __TruncateRootUDF_0-1-0__
           Project: col(a), col(__TruncateAnyUDFChildren_1-1-0__)
-            UDFProject:
-            UDF foo = py_udf(col(a)) as __TruncateAnyUDFChildren_1-1-0__
+            UDF: foo
+            Expr = py_udf(col(a)) as __TruncateAnyUDFChildren_1-1-0__
             Passthrough Columns = col(a)
             Concurrency = Some(8)
             Resource request = { num_cpus = 8, num_gpus = 1 }
@@ -1043,8 +1135,8 @@ Project: col(a), col(c)
         Project: col(a), col(result)
           Project: col(__TruncateAnyUDFChildren_0-1-0__), col(a), [col(a) + col(a)] + col(__TruncateAnyUDFChildren_0-1-0__) as result
             Project: col(a), col(__TruncateAnyUDFChildren_0-1-0__)
-              UDFProject:
-              UDF foo = py_udf(col(a)) as __TruncateAnyUDFChildren_0-1-0__
+              UDF: foo
+              Expr = py_udf(col(a)) as __TruncateAnyUDFChildren_0-1-0__
               Passthrough Columns = col(a)
               Concurrency = Some(8)
               Resource request = { num_cpus = 8, num_gpus = 1 }
@@ -1087,8 +1179,8 @@ Project: col(a), col(c)
         assert_optimized_plan_eq_with_projection_pushdown(
             project,
             indoc! {"
-        UDFProject:
-        UDF foo = py_udf(col(c)) as udf_results
+        UDF: foo
+        Expr = py_udf(col(c)) as udf_results
         Passthrough Columns = None
         Concurrency = Some(8)
         Resource request = { num_cpus = 8, num_gpus = 1 }
@@ -1118,7 +1210,7 @@ Project: col(a), col(c)
         // Select the `udf_results` column, so the UDFProject should apply column pruning to the other columns
         let plan = LogicalPlan::UDFProject(UDFProject::try_new(
             scan_node,
-            mock_udf.alias("udf_results_0"),
+            mock_udf.clone().alias("udf_results_0"),
             vec![resolved_col("a"), resolved_col("b")],
         )?)
         .arced();
@@ -1143,13 +1235,13 @@ Project: col(a), col(c)
         assert_optimized_plan_eq_with_projection_pushdown(
             plan,
             indoc! {"
-        UDFProject:
-        UDF foo = py_udf(col(a)) as udf_results_1
+        UDF: foo
+        Expr = py_udf(col(a)) as udf_results_1
         Passthrough Columns = col(udf_results_0)
         Concurrency = Some(8)
         Resource request = { num_cpus = 8, num_gpus = 1 }
-          UDFProject:
-          UDF foo = py_udf(col(a)) as udf_results_0
+          UDF: foo
+          Expr = py_udf(col(a)) as udf_results_0
           Passthrough Columns = col(a)
           Concurrency = Some(8)
           Resource request = { num_cpus = 8, num_gpus = 1 }
@@ -1215,8 +1307,8 @@ Project: col(a), col(c)
             indoc! {"
             Project: col(a), col(udf_results), col(b)
               Project: col(a), col(b), col(c), col(udf_results)
-                UDFProject:
-                UDF foo = py_udf(col(c)) as udf_results
+                UDF: foo
+                Expr = py_udf(col(c)) as udf_results
                 Passthrough Columns = col(a), col(b), col(c)
                 Concurrency = Some(8)
                 Resource request = { num_cpus = 8, num_gpus = 1 }
@@ -1229,4 +1321,72 @@ Project: col(a), col(c)
         )?;
         Ok(())
     }
+
+    #[test]
+    fn test_split_udf_in_filter_top() -> DaftResult<()> {
+        let scan_op = dummy_scan_operator(vec![Field::new("a", DataType::Int64)]);
+        let scan_node = dummy_scan_node(scan_op.clone());
+        let udf = create_filter_udf(vec![resolved_col("a")]);
+        let plan = scan_node.filter(udf)?.build();
+
+        assert_optimized_plan_with_rules_repr_eq(
+            plan,
+            indoc! {"
+        Project: col(a)
+          Filter: col(__SplitUDFsFromFilters_udf_0__)
+            UDF: foo
+            Expr = py_udf(col(a)) as __SplitUDFsFromFilters_udf_0__
+            Passthrough Columns = col(a)
+            Concurrency = None
+              DummyScanOperator
+              File schema = a#Int64
+              Partitioning keys = []
+              Output schema = a#Int64
+        "},
+            vec![RuleBatch::new(
+                vec![
+                    Box::new(SplitUDFsFromFilters::new()),
+                    Box::new(SplitUDFs::new()),
+                    Box::new(PushDownProjection::new()),
+                ],
+                RuleExecutionStrategy::Once,
+            )],
+        )?;
+        Ok(())
+    }
+
+    #[test]
+    fn test_split_udf_in_filter_middle() -> DaftResult<()> {
+        let scan_op = dummy_scan_operator(vec![Field::new("a", DataType::Int64)]);
+        let scan_node = dummy_scan_node(scan_op.clone());
+        let condition = create_actor_pool_udf(vec![resolved_col("a")]).not_eq(lit("hello"));
+        let plan = scan_node.filter(condition)?.build();
+
+        assert_optimized_plan_with_rules_repr_eq(
+            plan,
+            indoc! {r#"
+        Project: col(a)
+          Filter: col(__SplitUDFsFromFilters_udf_0__)
+            Project: col(a), col(__TruncateAnyUDFChildren_0-1-0__) != lit("hello") as __SplitUDFsFromFilters_udf_0__
+              UDF: foo
+              Expr = py_udf(col(a)) as __TruncateAnyUDFChildren_0-1-0__
+              Passthrough Columns = col(a)
+              Concurrency = Some(8)
+              Resource request = { num_cpus = 8, num_gpus = 1 }
+                DummyScanOperator
+                File schema = a#Int64
+                Partitioning keys = []
+                Output schema = a#Int64
+        "#},
+            vec![RuleBatch::new(
+                vec![
+                    Box::new(SplitUDFsFromFilters::new()),
+                    Box::new(SplitUDFs::new()),
+                    Box::new(PushDownProjection::new()),
+                ],
+                RuleExecutionStrategy::Once,
+            )],
+        )?;
+        Ok(())
+    }
 }
