@@ -1,10 +1,9 @@
 use std::sync::Arc;
 
 use daft_dsl::{
-    expr::bound_expr::BoundExpr,
-    functions::{scalar::ScalarFn, FunctionArgs},
-    python_udf::{PyScalarFn, RowWisePyFn},
     Column, ExprRef, ResolvedColumn,
+    expr::bound_expr::BoundExpr,
+    functions::{FunctionArgs, scalar::ScalarFn},
 };
 use daft_recordbatch::RecordBatch;
 use indexmap::IndexMap;
@@ -290,7 +289,7 @@ fn translate_clustering_spec_expr(
     //  - Ok(expr) with expr being the translation, or
     //  - Err(()) if no translation is possible in the new projection.
 
-    use daft_dsl::{binary_op, Expr};
+    use daft_dsl::{Expr, binary_op};
 
     match clustering_spec_expr.as_ref() {
         Expr::Column(Column::Resolved(ResolvedColumn::Basic(name))) => {
@@ -393,25 +392,14 @@ fn translate_clustering_spec_expr(
 
             Ok(expr.in_subquery(subquery.clone()))
         }
-        Expr::ScalarFn(ScalarFn::Python(PyScalarFn::RowWise(RowWisePyFn {
-            function_name: name,
-            inner: func,
-            return_dtype,
-            original_args,
-            args: children,
-        }))) => {
-            let new_children = children
+        Expr::ScalarFn(ScalarFn::Python(udf)) => {
+            let new_children = udf
+                .args()
                 .iter()
                 .map(|e| translate_clustering_spec_expr(e, old_colname_to_new_colname))
                 .collect::<Result<Vec<_>, _>>()?;
             Ok(Arc::new(Expr::ScalarFn(ScalarFn::Python(
-                PyScalarFn::RowWise(RowWisePyFn {
-                    function_name: name.clone(),
-                    inner: func.clone(),
-                    return_dtype: return_dtype.clone(),
-                    original_args: original_args.clone(),
-                    args: new_children,
-                }),
+                udf.with_new_children(new_children),
             ))))
         }
         // Cannot have agg exprs or references to other tables in clustering specs.
