@@ -1,21 +1,20 @@
 use std::sync::Arc;
 
-use common_error::{DaftError, DaftResult};
+use common_error::DaftResult;
 use common_resource_request::ResourceRequest;
 use common_treenode::TreeNode;
 use daft_dsl::{
-    count_actor_pool_udfs,
-    functions::{
-        python::{LegacyPythonUDF, UDFProperties},
-        FunctionExpr,
-    },
     Expr, ExprRef,
+    functions::{
+        FunctionExpr,
+        python::{LegacyPythonUDF, UDFProperties},
+    },
 };
-use daft_logical_plan::partitioning::{translate_clustering_spec, ClusteringSpec};
+use daft_logical_plan::partitioning::{ClusteringSpec, translate_clustering_spec};
 use itertools::Itertools;
 use serde::{Deserialize, Serialize};
 
-use crate::{impl_default_tree_display, PhysicalPlanRef};
+use crate::{PhysicalPlanRef, impl_default_tree_display};
 
 #[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
 pub struct ActorPoolProject {
@@ -33,11 +32,6 @@ impl ActorPoolProject {
     ) -> DaftResult<Self> {
         let clustering_spec = translate_clustering_spec(input.clustering_spec(), &projection);
 
-        let num_actor_pool_udfs: usize = count_actor_pool_udfs(&projection);
-        if !num_actor_pool_udfs == 1 {
-            return Err(DaftError::InternalError(format!("Expected ActorPoolProject to have exactly 1 actor pool UDF expression but found: {num_actor_pool_udfs}")));
-        }
-
         Ok(Self {
             input,
             projection,
