@@ -9,7 +9,7 @@ mod planner;
 use planner::PhysicalPlanTranslator;
 pub use planner::{AdaptivePlanner, MaterializedResults, QueryStageOutput, StageStats};
 
-use crate::{optimization::optimizer::PhysicalOptimizer, PhysicalPlanRef};
+use crate::{PhysicalPlanRef, optimization::optimizer::PhysicalOptimizer};
 mod translate;
 pub use translate::{
     extract_agg_expr, populate_aggregation_stages, populate_aggregation_stages_bound,
