@@ -135,7 +135,8 @@ pub mod pylib {
             hive_partitioning,
             infer_schema,
             schema=None,
-            file_path_column=None
+            file_path_column=None,
+            skip_glob=false
         ))]
         pub fn glob_scan(
             py: Python,
@@ -146,6 +147,7 @@ pub mod pylib {
             infer_schema: bool,
             schema: Option<PySchema>,
             file_path_column: Option<String>,
+            skip_glob: bool,
         ) -> PyResult<Self> {
             py.allow_threads(|| {
                 let executor = common_runtime::get_io_runtime(true);
@@ -158,6 +160,7 @@ pub mod pylib {
                     schema.map(|s| s.schema),
                     file_path_column,
                     hive_partitioning,
+                    skip_glob,
                 );
 
                 let operator = executor.block_within_async_context(task)??;
