@@ -272,7 +272,8 @@ impl DataType {
             | Self::Tensor(..)
             | Self::FixedShapeTensor(..)
             | Self::SparseTensor(..)
-            | Self::FixedShapeSparseTensor(..) => {
+            | Self::FixedShapeSparseTensor(..)
+            | Self::File => {
                 let physical = Box::new(self.to_physical());
                 let logical_extension = Self::Extension(
                     DAFT_SUPER_EXTENSION_NAME.into(),
@@ -282,15 +283,18 @@ impl DataType {
                 logical_extension.to_arrow()
             }
             #[cfg(feature = "python")]
-            Self::Python => Err(DaftError::TypeError(format!(
-                "Can not convert {self:?} into arrow type"
-            ))),
+            Self::Python => {
+                let physical = Box::new(Self::Binary);
+                let logical_extension = Self::Extension(
+                    DAFT_SUPER_EXTENSION_NAME.into(),
+                    physical,
+                    Some(self.to_json()?),
+                );
+                logical_extension.to_arrow()
+            }
             Self::Unknown => Err(DaftError::TypeError(format!(
                 "Can not convert {self:?} into arrow type"
             ))),
-            Self::File => Err(DaftError::TypeError(format!(
-                "Can not convert {self:?} into arrow type"
-            ))),
         }
     }
 
@@ -359,8 +363,7 @@ impl DataType {
                 Field::new("discriminant", UInt8),
                 Field::new("data", Binary),
                 Field::new("url", Utf8),
-                #[cfg(feature = "python")]
-                Field::new("io_config", Python),
+                Field::new("io_config", Binary),
             ]),
             _ => {
                 assert!(self.is_physical());
@@ -369,7 +372,10 @@ impl DataType {
         }
     }
 
+    /// Check if this datatype can be converted into an Arrow datatype.
+    /// This includes checking if the associated arrays can be converted into Arrow arrays.
     #[inline]
+    /// Is this DataType convertible to Arrow?
     pub fn is_arrow(&self) -> bool {
         self.to_arrow().is_ok()
     }
@@ -686,7 +692,7 @@ impl DataType {
                 return Err(DaftError::TypeError(format!(
                     "Expected input to be numeric, instead got {}",
                     self,
-                )))
+                )));
             }
         };
         Ok(data_type)
@@ -918,7 +924,7 @@ impl DataType {
 )]
 impl From<&ArrowType> for DataType {
     fn from(item: &ArrowType) -> Self {
-        let result = match item {
+        match item {
             ArrowType::Null => Self::Null,
             ArrowType::Boolean => Self::Boolean,
             ArrowType::Int8 => Self::Int8,
@@ -978,12 +984,11 @@ impl From<&ArrowType> for DataType {
                 Self::Struct(fields)
             }
             ArrowType::Extension(name, dtype, metadata) => {
-                if name == DAFT_SUPER_EXTENSION_NAME {
-                    if let Some(metadata) = metadata {
-                        if let Ok(daft_extension) = Self::from_json(metadata.as_str()) {
-                            return daft_extension;
-                        }
-                    }
+                if name == DAFT_SUPER_EXTENSION_NAME
+                    && let Some(metadata) = metadata
+                    && let Ok(daft_extension) = Self::from_json(metadata.as_str())
+                {
+                    return daft_extension;
                 }
                 Self::Extension(
                     name.clone(),
@@ -993,9 +998,7 @@ impl From<&ArrowType> for DataType {
             }
 
             _ => panic!("DataType :{item:?} is not supported"),
-        };
-
-        result
+        }
     }
 }
 
