@@ -3,6 +3,7 @@ pub use crate::{
     field::{Field, FieldID, FieldRef},
     image_format::ImageFormat,
     image_mode::ImageMode,
+    image_property::ImageProperty,
     schema::{Schema, SchemaRef},
-    time_unit::{infer_timeunit_from_format_string, TimeUnit},
+    time_unit::{TimeUnit, infer_timeunit_from_format_string},
 };
