@@ -5,7 +5,7 @@ pub mod schema;
 
 pub use datatype::{PyDataType, PyTimeUnit};
 
-use crate::{image_format::ImageFormat, image_mode::ImageMode};
+use crate::{image_format::ImageFormat, image_mode::ImageMode, image_property::ImageProperty};
 
 pub fn register_modules(parent: &Bound<PyModule>) -> PyResult<()> {
     parent.add_class::<datatype::PyDataType>()?;
@@ -14,6 +14,7 @@ pub fn register_modules(parent: &Bound<PyModule>) -> PyResult<()> {
     parent.add_class::<field::PyField>()?;
     parent.add_class::<ImageMode>()?;
     parent.add_class::<ImageFormat>()?;
+    parent.add_class::<ImageProperty>()?;
 
     Ok(())
 }
