@@ -31,7 +31,7 @@ pub enum IdentifierMode {
 /// Options helpers to convert session
 impl Options {
     /// Returns the binding `LookupMode` for the current `IdentifierMode`.
-    pub fn find_mode(&self) -> LookupMode {
+    pub fn lookup_mode(&self) -> LookupMode {
         match self.identifier_mode {
             IdentifierMode::Insensitive => LookupMode::Insensitive,
             IdentifierMode::Sensitive => LookupMode::Sensitive,
