@@ -10,9 +10,9 @@ use common_error::{DaftError, DaftResult};
 use daft_catalog::Identifier;
 use daft_core::prelude::*;
 use daft_dsl::{
-    functions::{scalar::ScalarFn, ScalarUDF},
-    has_agg, lit, null_lit, resolved_col, unresolved_col, Column, Expr, ExprRef, Operator, PlanRef,
-    Subquery, UnresolvedColumn,
+    Column, Expr, ExprRef, Operator, PlanRef, Subquery, UnresolvedColumn,
+    functions::{ScalarUDF, scalar::ScalarFn},
+    has_agg, lit, null_lit, resolved_col, unresolved_col,
 };
 use daft_functions::{
     invalid_argument_err,
@@ -20,8 +20,8 @@ use daft_functions::{
 };
 use daft_functions_utf8::{ilike, like, to_date, to_datetime};
 use daft_logical_plan::{
-    ops::{SetQuantifier, UnionStrategy},
     JoinOptions, LogicalPlanBuilder, LogicalPlanRef,
+    ops::{SetQuantifier, UnionStrategy},
 };
 use daft_session::Session;
 use itertools::Itertools;
@@ -179,7 +179,7 @@ impl SQLPlanner<'_> {
         // !! IMPORTANT: name resolution requires adding an alias to each CTE !!
         let ctes_with_alias = ctes
             .iter()
-            .map(|(alias, plan)| (alias.to_string(), plan.alias(alias.as_str())));
+            .map(|(alias, plan)| (alias.clone(), plan.alias(alias.as_str())));
         self.context.borrow_mut().bound_ctes.extend(ctes_with_alias);
         self
     }
@@ -949,7 +949,7 @@ impl SQLPlanner<'_> {
         let mut path = vec![];
         for part in &name.0 {
             let part = match part.quote_style {
-                Some('"') => part.value.to_string(),
+                Some('"') => part.value.clone(),
                 None => normalizer(&part.value),
                 Some(c) => unsupported_sql_err!(
                     "Daft only supports delimited identifiers with double-quotes, found {}",
@@ -1067,7 +1067,7 @@ impl SQLPlanner<'_> {
                     return Err(DaftError::ValueError(format!(
                         "Ambiguous column reference in join predicate: {full_name}"
                     ))
-                    .into())
+                    .into());
                 }
                 (Some(schema), None) | (None, Some(schema)) => {
                     if let Some(expr) = expr_from_idents(
@@ -1110,7 +1110,7 @@ impl SQLPlanner<'_> {
         match item {
             SelectItem::ExprWithAlias { expr, alias } => {
                 let expr = self.plan_expr(expr)?;
-                let alias = alias.value.to_string();
+                let alias = alias.value.clone();
                 self.bound_columns.insert(alias.clone(), expr.clone());
                 Ok(vec![expr.alias(alias)])
             }
@@ -1220,9 +1220,9 @@ impl SQLPlanner<'_> {
             Value::Boolean(b) => Literal::Boolean(*b),
             Value::Null => Literal::Null,
             _ => {
-                return Err(PlannerError::invalid_operation(
-                    format!("Only string, number, boolean and null literals are supported. Instead found: `{value}`"),
-                ))
+                return Err(PlannerError::invalid_operation(format!(
+                    "Only string, number, boolean and null literals are supported. Instead found: `{value}`"
+                )));
             }
         })
     }
@@ -1281,11 +1281,7 @@ impl SQLPlanner<'_> {
                     .collect::<SQLPlannerResult<Vec<_>>>()?;
 
                 let expr = expr.is_in(list);
-                if *negated {
-                    Ok(expr.not())
-                } else {
-                    Ok(expr)
-                }
+                if *negated { Ok(expr.not()) } else { Ok(expr) }
             }
             SQLExpr::InSubquery {
                 expr,
@@ -1314,11 +1310,7 @@ impl SQLPlanner<'_> {
                 let low = self.plan_expr(low)?;
                 let high = self.plan_expr(high)?;
                 let expr = expr.between(low, high);
-                if *negated {
-                    Ok(expr.not())
-                } else {
-                    Ok(expr)
-                }
+                if *negated { Ok(expr.not()) } else { Ok(expr) }
             }
             SQLExpr::Like {
                 negated,
@@ -1332,11 +1324,7 @@ impl SQLPlanner<'_> {
                 let expr = self.plan_expr(expr)?;
                 let pattern = self.plan_expr(pattern)?;
                 let expr = like(expr, pattern);
-                if *negated {
-                    Ok(expr.not())
-                } else {
-                    Ok(expr)
-                }
+                if *negated { Ok(expr.not()) } else { Ok(expr) }
             }
             SQLExpr::ILike {
                 negated,
@@ -1350,11 +1338,7 @@ impl SQLPlanner<'_> {
                 let expr = self.plan_expr(expr)?;
                 let pattern = self.plan_expr(pattern)?;
                 let expr = ilike(expr, pattern);
-                if *negated {
-                    Ok(expr.not())
-                } else {
-                    Ok(expr)
-                }
+                if *negated { Ok(expr.not()) } else { Ok(expr) }
             }
             SQLExpr::SimilarTo { .. } => unsupported_sql_err!("SIMILAR TO"),
             SQLExpr::RLike { .. } => unsupported_sql_err!("RLIKE"),
@@ -1528,7 +1512,7 @@ impl SQLPlanner<'_> {
                     })
                     .collect::<SQLPlannerResult<Vec<_>>>()?;
 
-                let s = Series::try_from(values)?;
+                let s = Series::from_literals(values)?;
                 let s = FixedSizeListArray::new(
                     Field::new("tuple", s.data_type().clone())
                         .to_fixed_size_list_field(exprs.len())?,
@@ -1550,8 +1534,7 @@ impl SQLPlanner<'_> {
                         let value = value.as_literal().ok_or_else(|| {
                             PlannerError::invalid_operation("Dictionary value is not a literal")
                         })?;
-                        let struct_field = Field::new(key, value.get_type());
-                        Ok((struct_field, value.clone()))
+                        Ok((key, value.clone()))
                     })
                     .collect::<SQLPlannerResult<_>>()?;
 
@@ -1609,7 +1592,7 @@ impl SQLPlanner<'_> {
                                 return Err(PlannerError::invalid_operation(format!(
                                     "Invalid interval unit: {}",
                                     &cap[2]
-                                )))
+                                )));
                             }
                         };
 
@@ -1685,12 +1668,18 @@ impl SQLPlanner<'_> {
                             DateTimeField::Hour => (0, 0, count * 3_600_000_000_000),
                             DateTimeField::Minute => (0, 0, count * 60_000_000_000),
                             DateTimeField::Second => (0, 0, count * 1_000_000_000),
-                            DateTimeField::Microsecond | DateTimeField::Microseconds => (0, 0, count * 1_000),
-                            DateTimeField::Millisecond | DateTimeField::Milliseconds => (0, 0, count * 1_000_000),
+                            DateTimeField::Microsecond | DateTimeField::Microseconds => {
+                                (0, 0, count * 1_000)
+                            }
+                            DateTimeField::Millisecond | DateTimeField::Milliseconds => {
+                                (0, 0, count * 1_000_000)
+                            }
                             DateTimeField::Nanosecond | DateTimeField::Nanoseconds => (0, 0, count),
-                            _ => return Err(PlannerError::invalid_operation(format!(
-                                "Invalid interval unit: {time_unit}. Expected one of: year, month, week, day, hour, minute, second, millisecond, microsecond, nanosecond"
-                            ))),
+                            _ => {
+                                return Err(PlannerError::invalid_operation(format!(
+                                    "Invalid interval unit: {time_unit}. Expected one of: year, month, week, day, hour, minute, second, millisecond, microsecond, nanosecond"
+                                )));
+                            }
                         };
 
                         Ok(Arc::new(Expr::Literal(Literal::Interval(
@@ -1837,7 +1826,7 @@ impl SQLPlanner<'_> {
                         let lower = self.plan_expr(lower)?;
                         let upper = self.plan_expr(upper)?;
                         let expr = self.plan_expr(expr)?;
-                        Ok(daft_functions_list::slice(expr, lower, upper))
+                        Ok(daft_functions::slice::slice(expr, lower, upper))
                     }
                     _ => {
                         unsupported_sql_err!("slice with only one bound not yet supported");
