@@ -2,7 +2,6 @@ from __future__ import annotations
 
 import pytest
 
-from daft.context import get_context
 from daft.expressions import col
 from tests.conftest import assert_df_equals, get_tests_daft_runner_name
 
@@ -11,14 +10,11 @@ def skip_invalid_join_strategies(join_strategy):
     if get_tests_daft_runner_name() == "native":
         if join_strategy not in [None, "hash"]:
             pytest.skip("Native executor fails for these tests")
-    elif not get_context().daft_execution_config.use_legacy_ray_runner and join_strategy == "sort_merge":
-        pytest.skip("Sort merge joins are not supported on Flotilla")
 
 
 @pytest.mark.parametrize(
     "join_strategy",
-    [None, "hash", "sort_merge", "sort_merge_aligned_boundaries", "broadcast"],
-    indirect=True,
+    [None, "hash", "sort_merge", "broadcast"],
 )
 def test_simple_join(join_strategy, daft_df, service_requests_csv_pd_df, repartition_nparts, with_morsel_size):
     skip_invalid_join_strategies(join_strategy)
@@ -40,8 +36,7 @@ def test_simple_join(join_strategy, daft_df, service_requests_csv_pd_df, reparti
 
 @pytest.mark.parametrize(
     "join_strategy",
-    [None, "hash", "sort_merge", "sort_merge_aligned_boundaries", "broadcast"],
-    indirect=True,
+    [None, "hash", "sort_merge", "broadcast"],
 )
 def test_simple_self_join(join_strategy, daft_df, service_requests_csv_pd_df, repartition_nparts, with_morsel_size):
     skip_invalid_join_strategies(join_strategy)
@@ -67,8 +62,7 @@ def test_simple_self_join(join_strategy, daft_df, service_requests_csv_pd_df, re
 
 @pytest.mark.parametrize(
     "join_strategy",
-    [None, "hash", "sort_merge", "sort_merge_aligned_boundaries", "broadcast"],
-    indirect=True,
+    [None, "hash", "sort_merge", "broadcast"],
 )
 def test_simple_join_missing_rvalues(
     join_strategy, daft_df, service_requests_csv_pd_df, repartition_nparts, with_morsel_size
@@ -95,8 +89,7 @@ def test_simple_join_missing_rvalues(
 
 @pytest.mark.parametrize(
     "join_strategy",
-    [None, "hash", "sort_merge", "sort_merge_aligned_boundaries", "broadcast"],
-    indirect=True,
+    [None, "hash", "sort_merge", "broadcast"],
 )
 def test_simple_join_missing_lvalues(
     join_strategy, daft_df, service_requests_csv_pd_df, repartition_nparts, with_morsel_size
