@@ -59,8 +59,7 @@ def test_decimal_sum(prec, partitions) -> None:
     assert res.to_pydict()["decimal128"] == [decimal.Decimal("108.001")]
 
     schema = res.schema()
-    expected_prec = min(38, prec + 19)  # see agg_ops.rs
-    assert schema["decimal128"].dtype == daft.DataType.decimal128(expected_prec, 3)
+    assert schema["decimal128"].dtype == daft.DataType.decimal128(38, 3)
 
 
 @pytest.mark.parametrize("prec, partitions", itertools.product([5, 30], [1, 2]))
@@ -72,8 +71,7 @@ def test_decimal_mean(prec, partitions) -> None:
     assert res.to_pydict()["decimal128"] == [decimal.Decimal("36.0003333")]
 
     schema = res.schema()
-    expected_prec = min(38, prec + 19)  # see agg_ops.rs
-    assert schema["decimal128"].dtype == daft.DataType.decimal128(expected_prec, 7)
+    assert schema["decimal128"].dtype == daft.DataType.decimal128(38, 7)
 
 
 @pytest.mark.parametrize("prec, partitions", itertools.product([5, 30], [1, 2]))
@@ -90,7 +88,7 @@ def test_decimal_stddev(prec, partitions) -> None:
 
 @pytest.mark.parametrize("prec, partitions", itertools.product([5, 30], [1, 2]))
 @pytest.mark.skipif(
-    get_context().daft_execution_config.use_experimental_distributed_engine is True,
+    get_context().daft_execution_config.use_legacy_ray_runner is False,
     reason="resource requests are not fully supported in Flotilla",
 )
 def test_decimal_grouped_sum(prec, partitions) -> None:
@@ -102,13 +100,12 @@ def test_decimal_grouped_sum(prec, partitions) -> None:
     res = df.groupby("group").sum().sort("group").collect()
     assert res.to_pydict() == {"group": [0, 1], "decimal128": [decimal.Decimal("9.000"), decimal.Decimal("99.001")]}
     schema = res.schema()
-    expected_prec = min(38, prec + 19)  # see agg_ops.rs
-    assert schema["decimal128"].dtype == daft.DataType.decimal128(expected_prec, 3)
+    assert schema["decimal128"].dtype == daft.DataType.decimal128(38, 3)
 
 
 @pytest.mark.parametrize("prec, partitions", itertools.product([5, 30], [1, 2]))
 @pytest.mark.skipif(
-    get_context().daft_execution_config.use_experimental_distributed_engine is True,
+    get_context().daft_execution_config.use_legacy_ray_runner is False,
     reason="resource requests are not fully supported in Flotilla",
 )
 def test_decimal_grouped_mean(prec, partitions) -> None:
@@ -120,8 +117,7 @@ def test_decimal_grouped_mean(prec, partitions) -> None:
     res = df.groupby("group").mean().sort("group").collect()
     assert res.to_pydict() == {"group": [0, 1], "decimal128": [decimal.Decimal("4.500"), decimal.Decimal("99.001")]}
     schema = res.schema()
-    expected_prec = min(38, prec + 19)  # see agg_ops.rs
-    assert schema["decimal128"].dtype == daft.DataType.decimal128(expected_prec, 7)
+    assert schema["decimal128"].dtype == daft.DataType.decimal128(38, 7)
 
 
 @pytest.mark.parametrize("prec, partitions", itertools.product([5, 30], [1, 2]))
