@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import datetime
 import decimal
+from unittest.mock import patch
 
 import pyarrow as pa
 import pytest
@@ -44,6 +45,26 @@ from pyiceberg.types import (
 import daft
 
 
+@pytest.fixture(scope="function", params=[False, True], ids=["no_mock", "with_mock"])
+def patch_scan_task_file_path_scheme(request):
+    """Fixture to patch ScanTask.catalog_scan_task to use file:/ scheme."""
+    use_mock = request.param
+
+    if use_mock:
+        from daft.daft import ScanTask
+
+        original_catalog_scan_task = ScanTask.catalog_scan_task
+
+        def patched_catalog_scan_task(file: str, **kwargs):
+            normalized_file = file.replace("file:///", "file:/")
+            return original_catalog_scan_task(file=normalized_file, **kwargs)
+
+        with patch("daft.daft.ScanTask.catalog_scan_task", side_effect=patched_catalog_scan_task):
+            yield
+    else:
+        yield
+
+
 @pytest.fixture(scope="function")
 def local_catalog(tmpdir):
     catalog = SqlCatalog(
@@ -62,15 +83,36 @@ def local_catalog(tmpdir):
     params=[
         pytest.param((UNPARTITIONED_PARTITION_SPEC, 1), id="unpartitioned"),
         pytest.param(
-            (PartitionSpec(PartitionField(source_id=1, field_id=1000, transform=IdentityTransform(), name="x")), 5),
+            (
+                PartitionSpec(
+                    PartitionField(
+                        source_id=1, field_id=1000, transform=IdentityTransform(), name="x_identity_partitioned"
+                    )
+                ),
+                5,
+            ),
             id="identity_partitioned",
         ),
         pytest.param(
-            (PartitionSpec(PartitionField(source_id=1, field_id=1000, transform=BucketTransform(4), name="x")), 3),
+            (
+                PartitionSpec(
+                    PartitionField(
+                        source_id=1, field_id=1000, transform=BucketTransform(4), name="x_bucket_partitioned"
+                    )
+                ),
+                3,
+            ),
             id="bucket_partitioned",
         ),
         pytest.param(
-            (PartitionSpec(PartitionField(source_id=1, field_id=1000, transform=TruncateTransform(2), name="x")), 3),
+            (
+                PartitionSpec(
+                    PartitionField(
+                        source_id=1, field_id=1000, transform=TruncateTransform(2), name="x_truncate_partitioned"
+                    )
+                ),
+                3,
+            ),
             id="truncate_partitioned",
         ),
     ],
@@ -229,9 +271,9 @@ def complex_table() -> tuple[pa.Table, Schema]:
             "binary": [b"foo", b"bar", b"baz"],
             "boolean": [True, False, True],
             "timestamp": [
-                datetime.datetime(2024, 2, 10),
-                datetime.datetime(2024, 2, 11),
-                datetime.datetime(2024, 2, 12),
+                datetime.datetime(2024, 2, 10, 12, 1, 40),
+                datetime.datetime(2024, 2, 11, 12, 2, 41),
+                datetime.datetime(2024, 2, 12, 12, 3, 42),
             ],
             "date": [datetime.date(2024, 2, 10), datetime.date(2024, 2, 11), datetime.date(2024, 2, 12)],
             "decimal": pa.array(
@@ -281,141 +323,232 @@ def complex_table() -> tuple[pa.Table, Schema]:
     [
         pytest.param(UNPARTITIONED_PARTITION_SPEC, 1, id="unpartitioned"),
         pytest.param(
-            PartitionSpec(PartitionField(source_id=1, field_id=1000, transform=IdentityTransform(), name="int")),
+            PartitionSpec(
+                PartitionField(
+                    source_id=1, field_id=1000, transform=IdentityTransform(), name="int_identity_partitioned"
+                )
+            ),
             3,
             id="int_identity_partitioned",
         ),
         pytest.param(
-            PartitionSpec(PartitionField(source_id=1, field_id=1000, transform=BucketTransform(2), name="int")),
+            PartitionSpec(
+                PartitionField(source_id=1, field_id=1000, transform=BucketTransform(2), name="int_bucket_partitioned")
+            ),
             2,
             id="int_bucket_partitioned",
         ),
         pytest.param(
-            PartitionSpec(PartitionField(source_id=1, field_id=1000, transform=TruncateTransform(2), name="int")),
+            PartitionSpec(
+                PartitionField(
+                    source_id=1, field_id=1000, transform=TruncateTransform(2), name="int_truncate_partitioned"
+                )
+            ),
             2,
             id="int_truncate_partitioned",
         ),
         pytest.param(
-            PartitionSpec(PartitionField(source_id=2, field_id=1000, transform=IdentityTransform(), name="float")),
+            PartitionSpec(
+                PartitionField(
+                    source_id=2, field_id=1000, transform=IdentityTransform(), name="float_identity_partitioned"
+                )
+            ),
             3,
             id="float_identity_partitioned",
         ),
         pytest.param(
-            PartitionSpec(PartitionField(source_id=3, field_id=1000, transform=IdentityTransform(), name="string")),
+            PartitionSpec(
+                PartitionField(
+                    source_id=3, field_id=1000, transform=IdentityTransform(), name="string_identity_partitioned"
+                )
+            ),
             3,
             id="string_identity_partitioned",
         ),
         pytest.param(
-            PartitionSpec(PartitionField(source_id=3, field_id=1000, transform=BucketTransform(2), name="string")),
+            PartitionSpec(
+                PartitionField(
+                    source_id=3, field_id=1000, transform=BucketTransform(2), name="string_bucket_partitioned"
+                )
+            ),
             2,
             id="string_bucket_partitioned",
         ),
         pytest.param(
-            PartitionSpec(PartitionField(source_id=3, field_id=1000, transform=TruncateTransform(2), name="string")),
+            PartitionSpec(
+                PartitionField(
+                    source_id=3, field_id=1000, transform=TruncateTransform(2), name="string_truncate_partitioned"
+                )
+            ),
             2,
             id="string_truncate_partitioned",
         ),
         pytest.param(
-            PartitionSpec(PartitionField(source_id=4, field_id=1000, transform=IdentityTransform(), name="binary")),
+            PartitionSpec(
+                PartitionField(
+                    source_id=4, field_id=1000, transform=IdentityTransform(), name="binary_identity_partitioned"
+                )
+            ),
             3,
             id="binary_identity_partitioned",
         ),
         pytest.param(
-            PartitionSpec(PartitionField(source_id=4, field_id=1000, transform=BucketTransform(2), name="binary")),
+            PartitionSpec(
+                PartitionField(
+                    source_id=4, field_id=1000, transform=BucketTransform(2), name="binary_bucket_partitioned"
+                )
+            ),
             2,
             id="binary_bucket_partitioned",
         ),
         pytest.param(
-            PartitionSpec(PartitionField(source_id=4, field_id=1000, transform=TruncateTransform(2), name="binary")),
+            PartitionSpec(
+                PartitionField(
+                    source_id=4, field_id=1000, transform=TruncateTransform(2), name="binary_truncate_partitioned"
+                )
+            ),
             2,
             id="binary_truncate_partitioned",
         ),
         pytest.param(
-            PartitionSpec(PartitionField(source_id=5, field_id=1000, transform=IdentityTransform(), name="boolean")),
+            PartitionSpec(
+                PartitionField(
+                    source_id=5, field_id=1000, transform=IdentityTransform(), name="bool_identity_partitioned"
+                )
+            ),
             2,
             id="bool_identity_partitioned",
         ),
         pytest.param(
-            PartitionSpec(PartitionField(source_id=6, field_id=1000, transform=IdentityTransform(), name="timestamp")),
+            PartitionSpec(
+                PartitionField(
+                    source_id=6, field_id=1000, transform=IdentityTransform(), name="datetime_identity_partitioned"
+                )
+            ),
             3,
             id="datetime_identity_partitioned",
         ),
         pytest.param(
-            PartitionSpec(PartitionField(source_id=6, field_id=1000, transform=BucketTransform(2), name="timestamp")),
-            1,
+            PartitionSpec(
+                PartitionField(
+                    source_id=6, field_id=1000, transform=BucketTransform(2), name="datetime_bucket_partitioned"
+                )
+            ),
+            2,
             id="datetime_bucket_partitioned",
         ),
         pytest.param(
-            PartitionSpec(PartitionField(source_id=6, field_id=1000, transform=YearTransform(), name="timestamp")),
+            PartitionSpec(
+                PartitionField(source_id=6, field_id=1000, transform=YearTransform(), name="datetime_year_partitioned")
+            ),
             1,
             id="datetime_year_partitioned",
         ),
         pytest.param(
-            PartitionSpec(PartitionField(source_id=6, field_id=1000, transform=MonthTransform(), name="timestamp")),
+            PartitionSpec(
+                PartitionField(
+                    source_id=6, field_id=1000, transform=MonthTransform(), name="datetime_month_partitioned"
+                )
+            ),
             1,
             id="datetime_month_partitioned",
         ),
         pytest.param(
-            PartitionSpec(PartitionField(source_id=6, field_id=1000, transform=DayTransform(), name="timestamp")),
+            PartitionSpec(
+                PartitionField(source_id=6, field_id=1000, transform=DayTransform(), name="datetime_day_partitioned")
+            ),
             3,
             id="datetime_day_partitioned",
         ),
         pytest.param(
-            PartitionSpec(PartitionField(source_id=6, field_id=1000, transform=HourTransform(), name="timestamp")),
+            PartitionSpec(
+                PartitionField(source_id=6, field_id=1000, transform=HourTransform(), name="datetime_hour_partitioned")
+            ),
             3,
             id="datetime_hour_partitioned",
         ),
         pytest.param(
-            PartitionSpec(PartitionField(source_id=7, field_id=1000, transform=IdentityTransform(), name="date")),
+            PartitionSpec(
+                PartitionField(
+                    source_id=7, field_id=1000, transform=IdentityTransform(), name="date_identity_partitioned"
+                )
+            ),
             3,
             id="date_identity_partitioned",
         ),
         pytest.param(
-            PartitionSpec(PartitionField(source_id=7, field_id=1000, transform=BucketTransform(2), name="date")),
+            PartitionSpec(
+                PartitionField(source_id=7, field_id=1000, transform=BucketTransform(2), name="date_bucket_partitioned")
+            ),
             2,
             id="date_bucket_partitioned",
         ),
         pytest.param(
-            PartitionSpec(PartitionField(source_id=7, field_id=1000, transform=YearTransform(), name="date")),
+            PartitionSpec(
+                PartitionField(source_id=7, field_id=1000, transform=YearTransform(), name="date_year_partitioned")
+            ),
             1,
             id="date_year_partitioned",
         ),
         pytest.param(
-            PartitionSpec(PartitionField(source_id=7, field_id=1000, transform=MonthTransform(), name="date")),
+            PartitionSpec(
+                PartitionField(source_id=7, field_id=1000, transform=MonthTransform(), name="date_month_partitioned")
+            ),
             1,
             id="date_month_partitioned",
         ),
         pytest.param(
-            PartitionSpec(PartitionField(source_id=7, field_id=1000, transform=DayTransform(), name="date")),
+            PartitionSpec(
+                PartitionField(source_id=7, field_id=1000, transform=DayTransform(), name="date_day_partitioned")
+            ),
             3,
             id="date_day_partitioned",
         ),
         pytest.param(
-            PartitionSpec(PartitionField(source_id=8, field_id=1000, transform=IdentityTransform(), name="decimal")),
+            PartitionSpec(
+                PartitionField(
+                    source_id=8, field_id=1000, transform=IdentityTransform(), name="decimal_identity_partitioned"
+                )
+            ),
             3,
             id="decimal_identity_partitioned",
         ),
         pytest.param(
-            PartitionSpec(PartitionField(source_id=8, field_id=1000, transform=BucketTransform(2), name="decimal")),
+            PartitionSpec(
+                PartitionField(
+                    source_id=8, field_id=1000, transform=BucketTransform(2), name="decimal_bucket_partitioned"
+                )
+            ),
             1,
             id="decimal_bucket_partitioned",
         ),
         pytest.param(
-            PartitionSpec(PartitionField(source_id=8, field_id=1000, transform=TruncateTransform(2), name="decimal")),
+            PartitionSpec(
+                PartitionField(
+                    source_id=8,
+                    field_id=1000,
+                    transform=TruncateTransform(2),
+                    name="decimal_truncate_partitioned",
+                )
+            ),
             3,
             id="decimal_truncate_partitioned",
         ),
         pytest.param(
             PartitionSpec(
-                PartitionField(source_id=1, field_id=1000, transform=BucketTransform(2), name="int"),
-                PartitionField(source_id=3, field_id=1000, transform=TruncateTransform(2), name="string"),
+                PartitionField(source_id=1, field_id=1000, transform=BucketTransform(2), name="int_bucket_partitioned"),
+                PartitionField(
+                    source_id=3, field_id=1000, transform=TruncateTransform(2), name="string_truncate_partitioned"
+                ),
             ),
             3,
             id="double_partitioned",
         ),
     ],
 )
-def test_complex_table_write_read(local_catalog, complex_table, partition_spec, num_partitions):
+def test_complex_table_write_read(
+    local_catalog, complex_table, partition_spec, num_partitions, patch_scan_task_file_path_scheme
+):
     pa_table, schema = complex_table
     table = local_catalog.create_table("default.test", schema, partition_spec=partition_spec)
     df = daft.from_arrow(pa_table)
