@@ -11,8 +11,7 @@ from daft.functions import dense_rank, rank
 from tests.conftest import assert_df_equals, get_tests_daft_runner_name
 
 pytestmark = pytest.mark.skipif(
-    get_tests_daft_runner_name() == "ray"
-    and get_context().daft_execution_config.use_experimental_distributed_engine is False,
+    get_tests_daft_runner_name() == "ray" and get_context().daft_execution_config.use_legacy_ray_runner is True,
     reason="requires Native Runner or Flotilla to be in use",
 )
 
